#include "BearAssembly.h"

void bear_assembly_new_empty(BearAssembly **p_this) {
  newo(p_this);
}

void bear_assembly_new_elementary(BearAssembly **p_this, ulong feature) {
  BearAssembly *this;

  this = newo(p_this);
  new(&this->_parts, this->numParts = 1);
  bear_part_new_elementary(&this->_parts[0], feature);
}

void bear_assembly_new_copy(BearAssembly **p_this, BearAssembly *other) {
  ulong p;
  BearAssembly *this;

  this = newo(p_this);
  new(&this->_parts, this->numParts = other->numParts);

  for (p = 0; p < this->numParts; p++) {
    bear_part_new_copy(&this->_parts[p], other->_parts[p]);
  }
}

void bear_assembly_new_parent(BearAssembly **p_this, BearAssembly *other) {
  ulong p;
  BearAssembly *this;

  if (bear_assembly_is_empty(other)) {
    throw("Cannot create the parent of an empty assembly");
  }

  this = newo(p_this);
  new(&this->_parts, this->numParts = other->numParts - 1);

  for (p = 0; p < this->numParts; p++) {
    bear_part_new_copy(&this->_parts[p], other->_parts[p]);
  }

}

void bear_assembly_new_glue(
    BearAssembly **p_this,
    BearAssembly *base,
    BearPart *attachment) {
  ulong p;
  BearAssembly *this;

  this = newo(p_this);
  new(&this->_parts, this->numParts = base->numParts + 1);

  for (p = 0; p < base->numParts; p++) {
    bear_part_new_copy(&this->_parts[p], base->_parts[p]);
  }

  bear_part_new_copy(&this->_parts[p], attachment);
}

void bear_assembly_new_melt(
    BearAssembly **p_this,
    BearAssembly *base,
    BearPart *attachment) {
  ulong p;
  BearAssembly *this;

  if (!bear_assembly_can_melt(base, attachment)) {
    throw("Attachment cannot be melted to the base");
  }

  this = newo(p_this);
  new(&this->_parts, this->numParts = base->numParts);

  for (p = 0; p < base->numParts - 1; p++) {
    bear_part_new_copy(&this->_parts[p], base->_parts[p]);
  }

  bear_part_new_melt(&this->_parts[p], base->_parts[p], attachment);
}

void bear_assembly_new_load(BearAssembly **p_this, File *f) {
  BearAssembly *this;

  this = newo(p_this);
  load_label("[BearAssembly]", f);
  load_value(this->numParts, f);
  load_new_array(this->_parts, this->numParts, bear_part, f);
  load_label("[/BearAssembly]", f);
}

void bear_assembly_delete(BearAssembly **p_this) {
  BearAssembly *this;

  if ((this = *p_this) == null) {
    return;
  }

  delete(&this->_parts, this->numParts, bear_part);
  deleteo(p_this);
}

void bear_assembly_print(BearAssembly *this, Printer *p) {
  char *_s = null;

  sprint(p, "BearAssembly");
  string_new_bear_assembly(&_s, this);
  rprint(p, "%s", _s);
  string_delete(&_s);
  fprint(p);
}

void bear_assembly_save(BearAssembly *this, File *f) {
  save_label("[BearAssembly]", f);
  save_value(this->numParts, f);
  save_array(this->_parts, this->numParts, bear_part, f);
  save_label("[/BearAssembly]", f);
}

bool bear_assembly_equals(BearAssembly *this, BearAssembly *other) {
  ulong p;

  if (this->numParts != other->numParts) {
    return false;
  }

  for (p = 0; p < this->numParts; p++) {
    if (!bear_part_equals(this->_parts[p], other->_parts[p])) {
      return false;
    }
  }

  return true;
}

void string_new_bear_assembly(char **p_this, BearAssembly *assembly) {
  char *_s = null;
  ulong p;
  BearPart *part;

  string_new_copy(p_this, "e|");

  for (p = 0; p < assembly->numParts; p++) {
    if (p > 0) {
      string_renew_append(p_this, "|");
    }

    part = assembly->_parts[p];
    string_new_bear_part(&_s, part);
    string_renew_append(p_this, _s);
    string_delete(&_s);
  }
}

bool bear_assembly_is_empty(BearAssembly *this) {
  return this->numParts == 0;
}

bool bear_assembly_last_part_is_repeat(BearAssembly *this) {
  ulong p;
  BearPart *last;

  last = bear_assembly_last_part(this);

  for (p = 0; p < this->numParts - 1; p++) {
    if (bear_part_equals(last, this->_parts[p])) {
      return true;
    }
  }

  return false;
}

bool bear_assembly_is_parent(BearAssembly *this, BearAssembly *other) {
  ulong p;

  if (other->numParts != this->numParts + 1) {
    return false;
  }

  for (p = 0; p < this->numParts; p++) {
    if (!bear_part_equals(this->_parts[p], other->_parts[p])) {
      return false;
    }
  }

  return true;
}

bool bear_assembly_is_sibling(BearAssembly *this, BearAssembly *other) {
  ulong p;

  if (bear_assembly_equals(this, other)) {
    return false;
  }

  if (other->numParts != this->numParts) {
    return false;
  }

  for (p = 0; p < this->numParts - 1; p++) {
    if (!bear_part_equals(this->_parts[p], other->_parts[p])) {
      return false;
    }
  }

  return true;
}

bool bear_assembly_can_melt(BearAssembly *base, BearPart *attachment) {
  if (bear_assembly_is_empty(base)) {
    return false;
  }

  return bear_part_can_melt(bear_assembly_last_part(base), attachment);
}

BearPart *bear_assembly_first_part(BearAssembly *this) {
  if (this->numParts == 0) {
    throw("Cannot get the first part of an empty assembly");
  }

  return this->_parts[0];
}

BearPart *bear_assembly_last_part(BearAssembly *this) {
  if (this->numParts == 0) {
    throw("Cannot get the last part of an empty assembly");
  }

  return this->_parts[this->numParts - 1];
}

ulong bear_assembly_total_num_features(BearAssembly *this) {
  ulong n;
  ulong p;

  for (p = 0, n = 0; p < this->numParts; p++) {
    n += this->_parts[p]->numFeatures;
  }

  return n;
}

sbyte bear_assembly_compare(BearAssembly *a, BearAssembly *b, void *x) {
  sbyte c;
  ulong p;

  if (a->numParts < b->numParts) {
    return -1;
  }

  if (a->numParts > b->numParts) {
    return +1;
  }

  for (p = 0; p < a->numParts; p++) {
    if ((c = bear_part_compare(a->_parts[p], b->_parts[p], null)) < 0) {
      return -1;
    }

    if (c > 0) {
      return +1;
    }
  }

  return 0;
}
