#include "BearPart.h"
#include <limits.h>

void bear_part_new_elementary(BearPart **p_this, ulong feature) {
  BearPart *this;

  this = newo(p_this);
  this->numFeatures = 1;
  new(&this->_features, 1);
  this->_features[0] = feature;
}

void bear_part_new_copy(BearPart **p_this, BearPart *other) {
  BearPart *this;

  this = newo(p_this);
  this->numFeatures = other->numFeatures;
  new_copy(&this->_features, this->numFeatures, other->_features);
}

void bear_part_new_load(BearPart **p_this, File *f) {
  BearPart *this;

  this = newo(p_this);
  load_label("[BearPart]", f);
  load_value(this->numFeatures, f);
  load_new_values(this->_features, this->numFeatures, f);
  load_label("[/BearPart]", f);
}

void bear_part_new_melt(BearPart **p_this, BearPart *a, BearPart *b) {
  ulong f;
  ulong fa;
  ulong fb;
  ulong feature_a;
  ulong feature_b;
  BearPart *this;

  this = newo(p_this);
  new(&this->_features, this->numFeatures = a->numFeatures + b->numFeatures);

  for (f = 0, fa = 0, fb = 0; f < this->numFeatures; f++) {
    feature_a = fa == a->numFeatures ? ULONG_MAX : a->_features[fa];
    feature_b = fb == b->numFeatures ? ULONG_MAX : b->_features[fb];

    if (feature_a == feature_b) {
      if (feature_a == ULONG_MAX) {
        throw("Impossible: ran out of both inputs");
      }

      throw("Feature %lu was present in both input parts", feature_a);
    }

    if (feature_a < feature_b) {
      this->_features[f] = feature_a;
      fa++;
    } else {
      this->_features[f] = feature_b;
      fb++;
    }
  }

  if (fa != a->numFeatures) {
    throw("Impossible: %lu v %lu", fa, a->numFeatures);
  }

  if (fb != b->numFeatures) {
    throw("Impossible: %lu v %lu", fb, b->numFeatures);
  }
}

void bear_part_delete(BearPart **p_this) {
  BearPart *this;

  if ((this = *p_this) == null) {
    return;
  }

  deletev(&this->_features);

  deleteo(p_this);
}

void bear_part_print(BearPart *this, Printer *p) {
  char *_s = null;

  sprint(p, "BearPart");
  string_new_bear_part(&_s, this);
  rprint(p, _s);
  string_delete(&_s);
  fprint(p);
}

void bear_part_save(BearPart *this, File *f) {
  save_label("[BearPart]", f);
  save_value(this->numFeatures, f);
  save_values(this->_features, this->numFeatures, f);
  save_label("[/BearPart]", f);
}

bool bear_part_equals(BearPart *this, BearPart *other) {
  return other->numFeatures == this->numFeatures
    && values_equal(other->_features, this->_features, this->numFeatures);
}

bool bear_part_can_melt(BearPart *this, BearPart *other) {
  ulong fa;
  ulong fb;
  ulong feature_a;
  ulong feature_b;

  for (fa = 0, fb = 0; fa < this->numFeatures || fb < other->numFeatures;) {
    feature_a = fa == this->numFeatures ? ULONG_MAX : this->_features[fa];
    feature_b = fb == other->numFeatures ? ULONG_MAX : other->_features[fb];

    if (feature_a == feature_b) {
      if (feature_a == ULONG_MAX) {
        throw("Impossible: ran out of both inputs");
      }

      return false;
    }

    if (feature_a < feature_b) {
      fa++;
    } else {
      fb++;
    }
  }

  return true;
}

void string_new_bear_part(char **p_this, BearPart *part) {
  ulong f;

  string_new_empty(p_this);

  for (f = 0; f < part->numFeatures; f++) {
    if (f > 0) {
      string_renew_append(p_this, "-");
    }

    string_renew_append_f(p_this, "%lu", part->_features[f]);
  }
}

sbyte bear_part_compare(BearPart *a, BearPart *b, void *x) {
  ulong f;

  if (a->numFeatures < b->numFeatures) {
    return -1;
  }

  if (a->numFeatures > b->numFeatures) {
    return +1;
  }

  for (f = 0; f < a->numFeatures; f++) {
    if (a->_features[f] < b->_features[f]) {
      return -1;
    }

    if (a->_features[f] > b->_features[f]) {
      return +1;
    }
  }

  return 0;
}

sbyte bear_part_pointer_compare(BearPart **a, BearPart **b, void *x) {
  return bear_part_compare(*a, *b, x);
}
