/* Object class for a Bear part, which is a symbolic representation of the
 * features present in a BearHypercube. Immutable class.
 */
#ifndef BEAR_PART_H
#define BEAR_PART_H

#ifdef __cplusplus
extern "C" {
#endif

#include "jpc.h"

typedef struct {
  ulong numFeatures;
  ulong *_features;
} BearPart;

/* Constructor for an elementary part, i.e., a part with just one feature.
 */
void bear_part_new_elementary(BearPart **p_this, ulong feature);

/* Copy constructor.
 */
void bear_part_new_copy(BearPart **p_this, BearPart *other);

/* Load.
 */
void bear_part_new_load(BearPart **p_this, File *f);

/* Constructor for melting two parts together, i.e., combining their features.
 * Throws if there are any features in common.
 */
void bear_part_new_melt(BearPart **p_this, BearPart *a, BearPart *b);

/* Destructor.
 */
void bear_part_delete(BearPart **p_this);

/* Print.
 */
void bear_part_print(BearPart *this, Printer *p);

/* Save.
 */
void bear_part_save(BearPart *this, File *f);

/* Equals.
 */
bool bear_part_equals(BearPart *this, BearPart *other);

/* Whether another part can be melted with this one, which is only true if they
 * have no features in common.
 */
bool bear_part_can_melt(BearPart *this, BearPart *other);

/* Create a string with a single-line representation of the part.
 */
void string_new_bear_part(char **p_this, BearPart *part);

/* Comparator. Prefers a fewer number of features, and then the first
 * corresponding feature number that is lower.
 */
sbyte bear_part_compare(BearPart *a, BearPart *b, void *x);

/* Pointer comparator.
 */
sbyte bear_part_pointer_compare(BearPart **a, BearPart **b, void *x);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* _H */
