#include "Json.h"
#include <ctype.h>

#define INDENT_ 2

static void object_new_(JsonObject **p_this);

static char *skip_(char **p_p);

static bool get_(char *expected, char **p_p, char **p_e);

static bool get_sep_(char **p_p, char **p_e);

static void pair_new_(JsonObjectPair **p_this, char *key, Json *j);

static void pair_delete_(JsonObjectPair **p_this);

static void check_entire_(char *s, char *p, char **p_e);


void json_new(Json **p_this, char **p_p) {
  char *p;
  char *_s1 = null;
  char *_s2 = null;
  Json *this;

  this = newo(p_this);

  p = skip_(p_p);

  switch (*p) {
    case '\0':
      this->type = JSON_EMPTY;
      *p_p = p;

      return;

    case 'n':
      this->type = JSON_NULL;

      if (!get_("null", &p, &this->_e) || !get_sep_(&p, &this->_e)) {
        string_renew_indent(&this->_e, 1);
        string_renew_prepend(
            &this->_e,
            "Json: Read 'n' and expected it to be 'null':\n");
      }

      *p_p = p;
      break;

    case 't':
      this->type = JSON_TRUE;

      if (!get_("true", &p, &this->_e) || !get_sep_(&p, &this->_e)) {
        string_renew_indent(&this->_e, 1);
        string_renew_prepend(
            &this->_e,
            "Json: Read 't' and expected it to be 'true':\n");
      }

      *p_p = p;

      break;

    case 'f':
      this->type = JSON_FALSE;

      if (!get_("false", &p, &this->_e)) {
        string_renew_indent(&this->_e, 1);
        string_renew_prepend(
            &this->_e,
            "Json: Read 'f' and expected it to be 'false':\n");
      }

      *p_p = p;

      break;

    case '"':
      this->type = JSON_STRING;
      json_string_new((JsonString **)&this->_o, &p, &this->_e);
      *p_p = p;
      break;

    case '{':
      this->type = JSON_OBJECT;
      json_object_new((JsonObject **)&this->_o, &p, &this->_e);
      *p_p = p;

      break;

    case '[':
      this->type = JSON_ARRAY;
      json_array_new((JsonArray **)&this->_o, &p, &this->_e);
      *p_p = p;
      break;

    default:
      if (*p == '-' || isdigit(*p)) {
        this->type = JSON_NUMBER;
        json_number_new((JsonNumber **)&this->_o, &p, &this->_e);
        *p_p = p;
      } else {
        this->type = JSON_UNKNOWN;
        string_new_escape_char(&_s1, *p);
        string_new_escape(&_s2, p);
        string_new_f(
            &this->_e,
            "Json: Unknown first character '%s' in '%s'",
            _s1,
            _s2);
        string_delete(&_s2);
        string_delete(&_s1);
        *p_p = p + 1;
      }

      break;
  }
}

void json_new_empty(Json **p_this) {
  Json *this;

  this = newo(p_this);

  this->type = JSON_EMPTY;
}

void json_new_empty_array(Json **p_this) {
  Json *this;

  this = newo(p_this);

  this->type = JSON_ARRAY;
  json_array_new_empty((JsonArray **)&this->_o);
}

void json_new_entire(Json **p_this, char *s) {
  char *p;
  Json *this;

  p = s;

  json_new(p_this, &p);
  this = *p_this;

  skip_(&p);

  check_entire_(s, p, &this->_e);
}

void json_new_copy(Json **p_this, Json *other) {
  Json *this;

  this = newo(p_this);

  this->type = other->type;

  if (other->_e != null) {
    string_new_copy(&this->_e, other->_e);
  }

  switch (other->type) {
    case JSON_STRING:
      json_string_new_copy(
          (JsonString **)&this->_o,
          json_get_json_string(other));
      break;
    case JSON_NUMBER:
      json_number_new_copy(
          (JsonNumber **)&this->_o,
          json_get_json_number(other));
      break;
    case JSON_ARRAY:
      json_array_new_copy(
          (JsonArray **)&this->_o,
          json_get_json_array(other));
      break;
    case JSON_OBJECT:
      json_object_new_copy(
          (JsonObject **)&this->_o,
          json_get_json_object(other));
      break;
    default:
      break;
  }
}

void json_new_extract(Json **p_this, Json *other, char *path) {
  bool have_digit;
  bool leading_zero;
  bool recursive;
  char c;
  char *_err = null;
  char *_key = null;
  char *p;
  char *_next_path = null;
  char *q;
  char *r;
  char *_s = null;
  char *_s2 = null;
  ulong len;
  ulong n;
  ulong u;
  long index;
  Json *child;
  Json *_next_json = null;
  JsonObjectPair *pair;
  JsonObjectPair **pairs;
  JsonObjectPairs *_pairs = null;
  Json *_j = null;
  Json *this;

  if (json_is_unknown(other) || json_is_empty(other)) {
    json_new_empty(p_this);

    return;
  }

  p = path;
  skip_(&p);

  if (!get_("$", &p, &_err)) {
    newo(p_this);
    this = *p_this;

    donate(&_err, &this->_e);
    string_renew_indent(&this->_e, 1);
    string_new_escape(&_s, p);
    string_renew_prepend_f(
        &this->_e,
        "Json: Expected path to begin with '$', but got '%s':\n",
        _s);
    string_delete(&_s);
    this->type = JSON_UNKNOWN;

    return;
  }

  if (*p == '\0') {
    json_new_copy(p_this, other);

    return;
  }

  switch (*p) {
    case '.':
      q = ++p;
      recursive = false;

      if (*q == '.') {
        recursive = true;
        q = ++p;
      }

      for (; (c = *q) != '\0' && (c == '_' || isalnum(c)); q++) {
        if (q == p && isdigit(c)) {
          newo(p_this);
          this = *p_this;

          string_new_escape(&_s, p);
          string_new_escape(&_s2, path);
          string_renew_prepend_f(
              &this->_e,
              "Json: Key string after '$.%s' in path cannot start with digit: "
              "got '%s' in '%s'",
              recursive ? "." : "",
              _s,
              _s2);
          string_delete(&_s2);
          string_delete(&_s);
          this->type = JSON_UNKNOWN;

          return;
        }
      }

      len = q - p;

      if (len == 0) {
        newo(p_this);
        this = *p_this;

        string_new_escape(&_s, p);
        string_new_escape(&_s2, path);
        string_renew_prepend_f(
            &this->_e,
            "Json: Expected key string after '$.%s' in path "
            "but got '%s' in '%s'",
              recursive ? "." : "",
            _s,
            _s2);
        string_delete(&_s2);
        string_delete(&_s);
        this->type = JSON_UNKNOWN;

        return;
      }

      if (!json_is_object(other)) {
        json_new_empty(p_this);

        return;
      }

      new(&_key, len + 1);

      for (r = _key; p < q; p++, r++) {
        *r = *p;
      }

      *r = '\0';
      string_new_concat_until_null(&_next_path, "$", q, null);

      if (recursive) {
        json_new_empty_array(p_this);
        this = *p_this;

        json_object_pairs_new(&_pairs, json_get_json_object(other));
        n = _pairs->cardinality;
        pairs = _pairs->_pairs;

        for (u = 0; u < n; u++) {
          pair = pairs[u];

          if (string_equals(pair->key, _key)) {
            json_new_extract(&_j, pair->j, _next_path);
            if (_j->_e != null) {
              donate(&_j->_e, &this->_e);
              string_renew_indent(&this->_e, 1);
              string_new_escape(&_s, path);
              string_renew_prepend_f(
                  &this->_e,
                  "Json: Error in sub-extract for path '%s':\n",
                  _s);
              string_delete(&_s);
              string_delete(&_key);
              json_delete(&_j);
              string_delete(&_next_path);
              json_object_pairs_delete(&_pairs);

              return;
            } else if (!json_is_empty(_j)) {
              json_append_array(this, &_j);
            } else {
              json_delete(&_j);
            }
          } else if (json_is_object(pair->j)) {
            json_new_extract(&_j, pair->j, path);

            if (_j->_e != null) {
              donate(&_j->_e, &this->_e);
              string_renew_indent(&this->_e, 1);
              string_new_escape(&_s, path);
              string_renew_prepend_f(
                  &this->_e,
                  "Json: Error in sub-extract for path '%s':\n",
                  _s);
              string_delete(&_s);
              string_delete(&_key);
              json_delete(&_j);
              string_delete(&_next_path);
              json_object_pairs_delete(&_pairs);

              return;
            } else if (json_is_array(_j)) {
              json_append_array_with_array(this, &_j);
            } else {
              json_delete(&_j);
            }
          }
        }

        json_object_pairs_delete(&_pairs);
        string_delete(&_key);
        string_delete(&_next_path);

        return;
      }

      child = json_object_get(json_get_json_object(other), _key);
      string_delete(&_key);

      if (child == null) {
        string_delete(&_next_path);
        json_new_empty(p_this);

        return;
      }

      json_new_extract(p_this, child, _next_path);
      this = *p_this;

      if (this->_e != null) {
        string_renew_indent(&this->_e, 1);
        string_new_escape(&_s, path);
        string_renew_prepend_f(
            &this->_e,
            "Json: Error in sub-extract for path '%s':\n",
            _s);
        string_delete(&_s);
      }
      string_delete(&_next_path);

      return;

    case '[':
      if ((c = *++p) == '\'') {
        for (q = ++p; (c = *q) != '\''; q++) {
          if (c == '\0') {
            newo(p_this);
            this = *p_this;

            string_new_escape(&_s, p);
            string_new_escape(&_s2, path);
            string_renew_prepend_f(
                &this->_e,
                "Json: String ended before quoted key string in path was "
                "complete: got '%s' in '%s'",
                _s,
                _s2);
            string_delete(&_s2);
            string_delete(&_s);
            this->type = JSON_UNKNOWN;

            return;
          }
        }

        len = q - p;
        new(&_key, len + 1);

        for (r = _key; p < q; p++, r++) {
          *r = *p;
        }

        *r = '\0';

        if (*++q != ']') {
          newo(p_this);
          this = *p_this;

          string_new_escape(&_s, q);
          string_new_escape(&_s2, path);
          string_renew_prepend_f(
              &this->_e,
              "Json: Didn't get closing ']' after key string '%s' in path: "
              "got '%s' in '%s'",
              _key,
              _s,
              _s2);
          string_delete(&_key);
          string_delete(&_s2);
          string_delete(&_s);
          this->type = JSON_UNKNOWN;

          return;
        }

        if (!json_is_object(other)) {
          string_delete(&_key);
          json_new_empty(p_this);

          return;
        }

        child = json_object_get(json_get_json_object(other), _key);
        string_delete(&_key);

        if (child == null) {
          json_new_empty(p_this);

          return;
        }

        json_new_copy(&_next_json, child);
        string_new_concat_until_null(&_next_path, "$", q + 1, null);

        json_new_extract(p_this, _next_json, _next_path);
        this = *p_this;

        if (this->_e != null) {
          string_renew_indent(&this->_e, 1);
          string_new_escape(&_s, path);
          string_renew_prepend_f(
              &this->_e,
              "Json: Error in sub-extract for path '%s':\n",
              _s);
          string_delete(&_s);
        }
        string_delete(&_next_path);
        json_delete(&_next_json);

        return;
      } else if (c == '-' || isdigit(c)) {
        have_digit = c != '-';
        leading_zero = c == '0';

        for (q = p + 1; (c = *q) != ']'; q++) {
          if (c == '\0') {
            newo(p_this);
            this = *p_this;

            string_new_escape(&_s, p);
            string_new_escape(&_s2, path);
            string_renew_prepend_f(
                &this->_e,
                "Json: String ended before integer key in path was "
                "complete: got '%s' in '%s'",
                _s,
                _s2);
            string_delete(&_s2);
            string_delete(&_s);
            this->type = JSON_UNKNOWN;

            return;
          }

          if (leading_zero) {
            newo(p_this);
            this = *p_this;

            string_new_escape(&_s, p);
            string_new_escape(&_s2, path);
            string_renew_prepend_f(
                &this->_e,
                "Json: Invalid digit after leading '0' in integer key in "
                "path: got '%s' in '%s'",
                _s,
                _s2);
            string_delete(&_s2);
            string_delete(&_s);
            this->type = JSON_UNKNOWN;

            return;
          }

          if (!have_digit && c == '0') {
            leading_zero = true;
          }

          have_digit = true;
        }
        if (!have_digit) {
          newo(p_this);
          this = *p_this;

          string_new_escape(&_s, p);
          string_new_escape(&_s2, path);
          string_renew_prepend_f(
              &this->_e,
              "Json: Invalid integer key '-' in path: got '%s' in '%s'",
              _s,
              _s2);
          string_delete(&_s2);
          string_delete(&_s);
          this->type = JSON_UNKNOWN;

          return;
        }

        if (!json_is_array(other)) {
          json_new_empty(p_this);

          return;
        }

        len = q - p;
        new(&_key, len + 1);

        for (r = _key; p < q; p++, r++) {
          *r = *p;
        }

        *r = '\0';
        index = parse_long(_key);
        string_delete(&_key);
        child = json_array_get(json_get_json_array(other), index);
        string_delete(&_key);

        if (child == null) {
          json_new_empty(p_this);

          return;
        }

        json_new_copy(&_next_json, child);
        string_new_concat_until_null(&_next_path, "$", q + 1, null);

        json_new_extract(p_this, _next_json, _next_path);
        this = *p_this;

        if (this->_e != null) {
          string_renew_indent(&this->_e, 1);
          string_new_escape(&_s, path);
          string_renew_prepend_f(
              &this->_e,
              "Json: Error in sub-extract for path '%s':\n",
              _s);
          string_delete(&_s);
        }

        string_delete(&_next_path);
        json_delete(&_next_json);

        return;
      } else {
        newo(p_this);
        this = *p_this;

        string_new_escape(&_s, p);
        string_new_escape(&_s2, path);
        string_renew_prepend_f(
            &this->_e,
            "Json: Expected quote (') or integer after '[' in path "
            "but got '%s' in '%s'",
            _s,
            _s2);
        string_delete(&_s2);
        string_delete(&_s);
        this->type = JSON_UNKNOWN;

        return;
      }

    default:
      newo(p_this);
      this = *p_this;

      string_new_escape(&_s, p);
      string_new_escape(&_s2, path);
      string_renew_prepend_f(
          &this->_e,
          "Json: Expected '.' or '[' after '$' in path for object, "
          "but got '%s' in '%s'",
          _s,
          _s2);
      string_delete(&_s2);
      string_delete(&_s);
      this->type = JSON_UNKNOWN;

      return;
  }
}

void json_delete(Json **p_this) {
  Json *this;

  if ((this = *p_this) == null) {
    return;
  }

  string_delete(&this->_e);

  switch (this->type) {
    case JSON_STRING:
      json_string_delete((JsonString **)&this->_o);
      break;
    case JSON_NUMBER:
      json_number_delete((JsonNumber **)&this->_o);
      break;
    case JSON_ARRAY:
      json_array_delete((JsonArray **)&this->_o);
      break;
    case JSON_OBJECT:
      json_object_delete((JsonObject **)&this->_o);
      break;
    default:
      break;
  }

  deleteo(p_this);
}

void json_print(Json *this, Printer *p) {
  char *_s = null;
  char *_s2 = null;
  char **_s2s = null;
  ulong num_s2s;
  ulong u;
  void (*print)(void *this, Printer *) = null;

  sprint(p, "Json");

  switch (this->type) {
    case JSON_EMPTY:
      break;
    case JSON_UNKNOWN:
      break;
    case JSON_NULL:
      break;
    case JSON_TRUE:
      break;
    case JSON_FALSE:
      break;
    case JSON_STRING:
      print = (void (*)(void *, Printer *))json_string_print;
      break;
    case JSON_NUMBER:
      print = (void (*)(void *, Printer *))json_number_print;
      break;
    case JSON_ARRAY:
      print = (void (*)(void *, Printer *))json_array_print;
      break;
    case JSON_OBJECT:
      print = (void (*)(void *, Printer *))json_object_print;
      break;
    default:
      throw("Impossible: type %d", this->type);
      break;
  }

  rprint(p, "Type: %s", json_get_type_string(this));

  if (this->_e != null) {
    sprint(p, "Errors");
    rprint(p, this->_e);
    fprint(p);
  }

  string_new_json(&_s, this, false, 0);
  string_new_json(&_s2, this, true, 0);
  num_s2s = strings_new_split(&_s2s, _s2, '\n');

  if (num_s2s == 0) {
    throw("Impossible");
  }

  string_renew_prepend(&_s2s[0], "'");
  string_renew_append(&_s2s[num_s2s - 1], "'");

  if (string_equals(_s, _s2)) {
    rprint(p, "As string: '%s'", _s);
  } else {
    rprint(p, "Efficient: '%s'", _s);
    sprint(p, "Pretty");

    for (u = 0; u < num_s2s; u++) {
      rprint(p, "%s", _s2s[u]);
    }

    fprint(p);
  }

  delete(&_s2s, num_s2s, string);
  string_delete(&_s2);
  string_delete(&_s);

  if (print != null) {
    print(this->_o, p);
  }

  fprint(p);
}

char *json_get_type_string(Json *this) {
  switch (this->type) {
    case JSON_EMPTY:
      return "JsonEmpty";
    case JSON_UNKNOWN:
      return "JsonUnknown";
    case JSON_NULL:
      return "JsonNull";
    case JSON_TRUE:
      return "JsonTrue";
    case JSON_FALSE:
      return "JsonFalse";
    case JSON_STRING:
      return "JsonString";
    case JSON_NUMBER:
      return "JsonNumber";
    case JSON_ARRAY:
      return "JsonArray";
    case JSON_OBJECT:
      return "JsonObject";
    default:
      throw("Impossible: type %d", this->type);
      break;
  }
  return null;
}

bool json_is_empty(Json *this) {
  return this->type == JSON_EMPTY;
}

bool json_is_unknown(Json *this) {
  return this->type == JSON_UNKNOWN;
}

bool json_is_null(Json *this) {
  return this->type == JSON_NULL;
}

bool json_is_scalar(Json *this) {
  switch (this->type) {
    case JSON_NULL:
    case JSON_TRUE:
    case JSON_FALSE:
    case JSON_STRING:
    case JSON_NUMBER:
      return true;
    default:
      return false;
  }
}

bool json_is_value(Json *this) {
  switch (this->type) {
    case JSON_EMPTY:
    case JSON_UNKNOWN:
      return false;
    default:
      return true;
  }
}

bool json_is_array(Json *this) {
  return this->type == JSON_ARRAY;
}

bool json_is_object(Json *this) {
  return this->type == JSON_OBJECT;
}

JsonString *json_get_json_string(Json *this) {
  if (this->type != JSON_STRING) {
    throw("Json object is a %s, not a JsonString",
        json_get_type_string(this));
  }

  return this->_o;
}

JsonNumber *json_get_json_number(Json *this) {
  if (this->type != JSON_NUMBER) {
    throw("Json object is a %s, not a JsonNumber",
        json_get_type_string(this));
  }

  return this->_o;
}

JsonArray *json_get_json_array(Json *this) {
  if (this->type != JSON_ARRAY) {
    throw("Json object is a %s, not a JsonArray",
        json_get_type_string(this));
  }

  return this->_o;
}

JsonObject *json_get_json_object(Json *this) {
  if (this->type != JSON_OBJECT) {
    throw("Json object is a %s, not a JsonObject",
        json_get_type_string(this));
  }

  return this->_o;
}

bool json_get_bool(Json *this) {
  bool b = false;
  char *_s = null;

  if (!json_try_get_bool(this, &b)) {
    string_new_json(&_s, this, true, 0);
    throw("Could not get bool value from Json '%s'", _s);
    string_delete(&_s);
  }

  return b;
}

ulong json_get_ulong(Json *this) {
  char *_s = null;
  ulong u;

  if (!json_try_get_ulong(this, &u)) {
    string_new_json(&_s, this, true, 0);
    throw("Could not get ulong value from Json '%s'", _s);
    string_delete(&_s);
  }

  return u;
}

ulong json_number_get_ulong(JsonNumber *this) {
  ulong u;

  if (!json_number_try_get_ulong(this, &u)) {
    throw("Could not get ulong value from JsonNumber '%s'", this->_s);
  }

  return u;
}

long json_get_long(Json *this) {
  char *_s = null;
  long l;

  if (!json_try_get_long(this, &l)) {
    string_new_json(&_s, this, true, 0);
    throw("Could not get long value from Json '%s'", _s);
    string_delete(&_s);
  }
  return l;
}

long json_number_get_long(JsonNumber *this) {
  long l;

  if (!json_number_try_get_long(this, &l)) {
    throw("Could not get long value from JsonNumber '%s'", this->_s);
  }

  return l;
}

double json_get_double(Json *this) {
  char *_s = null;
  double d;

  if (!json_try_get_double(this, &d)) {
    string_new_json(&_s, this, true, 0);
    throw("Could not get double value from Json '%s'", _s);
    string_delete(&_s);
  }
  return d;
}

double json_number_get_double(JsonNumber *this) {
  double d;

  if (!json_number_try_get_double(this, &d)) {
    throw("Could not get double value from JsonNumber '%s'", this->_s);
  }

  return d;
}

char *json_get_string(Json *this) {
  char *s;
  char *_s = null;

  if (!json_try_get_string(this, &s)) {
    string_new_json(&_s, this, true, 0);
    throw("Could not get string value from Json '%s'", _s);
    string_delete(&_s);
  }

  return s;
}

bool json_try_get_bool(Json *this, bool *p_b) {
  switch (this->type) {
    case JSON_TRUE:
      *p_b = true;
      return true;
    case JSON_FALSE:
      *p_b = false;
      return true;
    default:
      return false;
  }
}

bool json_try_get_ulong(Json *this, ulong *p_u) {
  if (this->type != JSON_NUMBER) {
    return false;
  }

  return json_number_try_get_ulong(json_get_json_number(this), p_u);
}

bool json_number_try_get_ulong(JsonNumber *this, ulong *p_u) {
  if (!this->integer || this->negative || this->_s == null) {
    return false;
  }

  if (!try_parse_ulong(p_u, this->_s)) {
    throw("Impossible: Json '%s' is a positive integer number but couldn't be "
        "parsed as a ulong",
        this->_s);
  }

  return true;
}

bool json_try_get_long(Json *this, long *p_l) {
  if (this->type != JSON_NUMBER) {
    return false;
  }

  return json_number_try_get_long(json_get_json_number(this), p_l);
}

bool json_number_try_get_long(JsonNumber *this, long *p_l) {
  if (!this->integer || this->_s == null) {
    return false;
  }

  if (!try_parse_long(p_l, this->_s)) {
    throw("Impossible: Json '%s' is an integer number but couldn't be "
        "parsed as a long",
        this->_s);
  }

  return true;
}

bool json_try_get_double(Json *this, double *p_d) {
  if (this->type != JSON_NUMBER) {
    return false;
  }

  return json_number_try_get_double(json_get_json_number(this), p_d);
}

bool json_number_try_get_double(JsonNumber *this, double *p_d) {
  if (this->_s == null) {
    return false;
  }

  if (!try_parse_double(p_d, this->_s)) {
    throw("Impossible: Json '%s' is a number but couldn't be "
        "parsed as a double",
        this->_s);
  }

  return true;
}

bool json_try_get_string(Json *this, char **p_s) {
  JsonString *js;

  if (this->type != JSON_STRING) {
    return false;
  }

  js = json_get_json_string(this);

  if (js->_s == null) {
    return false;
  }

  *p_s =  js->_s;

  return true;
}

void json_write_file(Json *this, File *file, bool pretty, byte i) {
  switch (this->type) {
    case JSON_EMPTY:
    case JSON_UNKNOWN:
      return;
    case JSON_NULL:
      file_write_string(file, "null");
      break;
    case JSON_TRUE:
      file_write_string(file, "true");
      break;
    case JSON_FALSE:
      file_write_string(file, "false");
      break;
    case JSON_STRING:
      json_string_write_file(json_get_json_string(this), file);
      break;
    case JSON_NUMBER:
      json_number_write_file(json_get_json_number(this), file);
      break;
    case JSON_ARRAY:
      json_array_write_file(json_get_json_array(this), file, pretty, i);
      break;
    case JSON_OBJECT:
      json_object_write_file(json_get_json_object(this), file, pretty, i);
      break;
    default:
      throw("Impossible: type %d", this->type);
      break;
  }
}

void json_append_array(Json *this, Json **p_other) {
  char *_err = null;
  JsonArray *a;
  Json *other;

  if (!json_is_array(this)) {
    throw("Can only append to an array");
  }

  other = *p_other;
  a = json_get_json_array(this);

  if (other->_e != null) {
    if (this->_e == null) {
      string_new_empty(&this->_e);
    }

    string_new_copy(&_err, other->_e);
    string_renew_indent(&_err, 1);
    string_renew_append_f(
        &this->_e,
        "JsonArray: Error in element %lu:\n%s",
        a->length,
        _err);
    string_delete(&_err);
  }

  json_array_append(a, p_other);
}

void json_append_array_with_array(Json *this, Json **p_other) {
  if (!json_is_array(this)) {
    throw("Can only append to an array");
  }

  if (!json_is_array(*p_other)) {
    throw("Can only append an array to an array");
  }

  json_array_append_array(json_get_json_array(this), p_other);
}

void json_donate_to_object(Json *this, char **p_key, Json **p_other) {
  char *_err = null;
  Json *other;

  if (!json_is_object(this)) {
    throw("Can only donate to an object");
  }

  other = *p_other;

  if (other->_e != null) {
    if (this->_e == null) {
      string_new_empty(&this->_e);
    }

    string_new_copy(&_err, other->_e);
    string_renew_indent(&_err, 1);
    string_renew_append_f(
        &this->_e,
        "JsonObject: Error in member '%s':\n%s",
        *p_key,
        _err);
    string_delete(&_err);
  }

  json_object_donate(json_get_json_object(this), p_key, p_other);
}

void string_new_json(char **p_this, Json *j, bool pretty, byte i) {
  byte b;
  char *p;
  char *this;
  ulong len;
  File *_f = null;
  MemoryFile *mf;

  file_new_memory(&_f);
  json_write_file(j, _f, pretty, i);
  mf = _f->_mf;

  len = memory_file_get_size(mf);
  memory_file_rewind(mf);

  new(p_this, len + 1);
  this = *p_this;

  for (p = this; memory_file_try_read_byte(mf, &b); p++) {
    *p = (char)b;
  }

  *p = '\0';

  file_delete(&_f);
}

void json_string_new(JsonString **p_this, char **p_p, char **p_e) {
  bool done;
  bool escaped;
  char c;
  char *e;
  char *op;
  char *p;
  char *q;
  char *r;
  char *_s = null;
  char *_s2 = null;
  char *_s3 = null;
  char *y;
  char *z;
  char digits[4];
  ulong d;
  ulong n;
  JsonString *this;

  this = newo(p_this);

  p = skip_(p_p);

  if (!get_("\"", &p, p_e)) {
    string_renew_indent(p_e, 1);
    string_renew_prepend(p_e, "JsonString: Couldn't read initial '\"':\n");
    *p_p = p;

    return;
  }

  n = 0;
  y = p;

  for (done = false, escaped = false; !done; p++) {
    c = *p;

    if (c == '\0') {
      done = true;
      break;
    }

    if (escaped) {
      switch (c) {
        case '"':
        case '\\':
        case '/':
        case 'b':
        case 'f':
        case 'n':
        case 'r':
        case 't':
          n += 2;
          break;

        case 'u':
          for (d = 0, p++; d < 4; d++, p++) {
            c = *p;
            if (!isxdigit(c)) {
              done = true;
              break;
            }
          }

          if (!done) {
            n += 6;
            p--;
          }

          break;

        default:
          done = true;
          break;
      }

      escaped = false;
    } else {
      if (c == '"') {
        done = true;
        break;
      }

      if (c == '\\') {
        escaped = true;
      } else {
        n++;
      }
    }
  }

  new(&this->_s, n + 1);
  q = this->_s;

  z = q + n;

  for (p = y, escaped = false;; p++) {
    c = *p;

    if (c == '\0') {
      string_new_escape(&_s, *p_p);
      string_new_f(
          p_e,
          "JsonString: String ended without closing '\"': '%s'",
          _s);
      string_delete(&_s);
      *p_p = p;

      return;
    }

    if (escaped) {
      switch (c) {
        case '"':
        case '\\':
        case '/':
        case 'b':
        case 'f':
        case 'n':
        case 'r':
        case 't':
          *q++ = '\\';
          *q++ = c;
          break;

        case 'u':
          op = p - 1;

          for (r = digits, e = r + 4, p++; r < e; r++, p++) {
            c = *p;

            if (!isxdigit(c)) {
              string_new_escape(&_s, *p_p);
              string_new_escape(&_s2, op);
              string_new_f(
                  p_e,
                  "JsonString: Illegal Unicode escape sequence in '%s' "
                  "at '%s'",
                  _s,
                  _s2);
              string_delete(&_s2);
              string_delete(&_s);
              *p_p = op;
              *q = '\0';

              if (q != z) {
                throw("Impossible");
              }

              return;
            }

            *r = tolower(c);
          }

          *q++ = '\\';
          *q++ = 'u';

          for (r = digits; r < e; r++) {
            *q++ = *r;
          }

          p--;
          break;

        default:
          string_new_escape(&_s, *p_p);
          string_new_escape_char(&_s2, c);
          string_new_escape(&_s3, p - 1);
          string_new_f(
              p_e,
              "JsonString: Illegal escape sequence '\\\\%s' in '%s' at '%s'",
              _s2,
              _s,
              _s3);
          string_delete(&_s3);
          string_delete(&_s2);
          string_delete(&_s);
          *p_p = p - 1;
          *q = '\0';

          if (q != z) {
            throw("Impossible");
          }

          return;
      }

      escaped = false;
    } else {
      if (c == '"') {
        *p_p = p + 1;
        *q = '\0';

        if (q != z) {
          throw("Impossible");
        }

        return;
      }
      if (c == '\\') {
        escaped = true;
      } else {
        *q++ = c;
      }
    }
  }

  return;
}

void json_string_new_copy(JsonString **p_this, JsonString *other) {
  JsonString *this;

  this = newo(p_this);

  string_new_copy(&this->_s, other->_s);
}

void json_string_delete(JsonString **p_this) {
  JsonString *this;

  if ((this = *p_this) == null) {
    return;
  }

  string_delete(&this->_s);

  deleteo(p_this);
}

void json_string_print(JsonString *this, Printer *p) {
  char *_s = null;

  sprint(p, "JsonString");

  if (this->_s == null) {
    rprint(p, "No value stored");
  } else {
    string_new_escape(&_s, this->_s);
    rprint(p, "Value: '%s'", _s);
    string_delete(&_s);
  }

  fprint(p);
}

void json_string_write_file(JsonString *this, File *file) {
  file_write_char(file, '"');

  if (this->_s != null) {
    file_write_string(file, this->_s);
  }

  file_write_char(file, '"');
}

void json_number_new(JsonNumber **p_this, char **p_p, char **p_e) {
  char *p;
  char *q;
  char *_s = null;
  char *_s2 = null;
  char *y;
  char *z;
  ulong n;
  JsonNumber *this;

  this = newo(p_this);

  p = skip_(p_p);

  y = p;
  this->integer = true;

  if (*p == '-') {
    this->negative = true;
    p++;
  }

  if (!isdigit(*p)) {
    string_new_escape(&_s, y);

    if (p == y) {
      string_new_f(
          p_e,
          "JsonNumber: Expected first character to be a digit but got '%s'",
          _s);
    } else {
      string_new_escape(&_s2, p);
      string_new_f(
          p_e,
          "JsonNumber: Expected digit after '-' but got '%s' in '%s'",
          _s2,
          _s);
      string_delete(&_s2);
    }

    string_delete(&_s);

    return;
  }

  if (*p == '0') {
    p++;

    if (isdigit(*p)) {
      string_new_escape(&_s, y);
      string_new_escape(&_s2, p);
      string_new_f(
          p_e,
          "JsonNumber: Cannot have digit after '0': got '%s' in '%s'",
          _s2,
          _s);
      string_delete(&_s2);
      string_delete(&_s);

      return;
    }
  } else {
    while (isdigit(*p)) {
      p++;
    }
  }

  if (*p == '.') {
    this->integer = false;
    p++;
    if (!isdigit(*p)) {
      string_new_escape(&_s, y);
      string_new_escape(&_s2, p);
      string_new_f(
          p_e,
          "JsonNumber: Expected digit after '.' but got '%s' in '%s'",
          _s2,
          _s);
      string_delete(&_s2);
      string_delete(&_s);

      return;
    }

    while (isdigit(*p)) {
      p++;
    }
  }
  if (*p == 'e' || *p == 'E') {
    this->integer = false;
    p++;

    if (*p == '+' || *p == '-') {
      p++;
    }

    if (!isdigit(*p)) {
      string_new_escape(&_s, y);
      string_new_escape(&_s2, p);
      string_new_f(
          p_e,
          "JsonNumber: Expected digit in exponent but got '%s' in '%s'",
          _s2,
          _s);
      string_delete(&_s2);
      string_delete(&_s);

      return;
    }

    while (isdigit(*p)) {
      p++;
    }
  }

  if (!get_sep_(&p, p_e)) {
    string_renew_indent(p_e, 1);
    string_renew_prepend(p_e, "JsonNumber: Number not terminated properly:\n");

    return;
  }

  n = p - y;

  new(&this->_s, n + 1);
  q = this->_s;

  for (z = p, p = y; p < z; p++, q++) {
    *q = tolower(*p);
  }

  *q = '\0';

  if (q != this->_s + n) {
    throw("Impossible");
  }

  *p_p = p;

  return;
}

void json_number_new_copy(JsonNumber **p_this, JsonNumber *other) {
  JsonNumber *this;

  this = newo(p_this);

  this->negative = other->negative;
  this->integer = other->integer;

  string_new_copy(&this->_s, other->_s);

  return;
}

void json_number_delete(JsonNumber **p_this) {
  JsonNumber *this;

  if ((this = *p_this) == null) {
    return;
  }

  string_delete(&this->_s);

  deleteo(p_this);

  return;
}

void json_number_print(JsonNumber *this, Printer *p) {
  sprint(p, "JsonNumber");

  if (this->_s == null) {
    rprint(p, "No value stored");
  } else {
    rprint(p, "Value: %s", this->_s);
  }

  rprint(p, "Negative: %s", bool_string(this->negative));
  rprint(p, "Integer: %s", bool_string(this->integer));
  fprint(p);
}

void json_number_write_file(JsonNumber *this, File *file) {
  if (this->_s != null) {
    file_write_string(file, this->_s);
  }
}

void json_array_new(JsonArray **p_this, char **p_p, char **p_e) {
  bool comma;
  bool some;
  char *lp;
  char *op;
  char *p;
  char *_s = null;
  char *_s2 = null;
  Json *_j = null;
  JsonArray *this;

  json_array_new_empty(p_this);
  this = *p_this;

  op = *p_p;
  p = skip_(p_p);

  if (!get_("[", &p, p_e)) {
    string_renew_indent(p_e, 1);
    string_renew_prepend(p_e, "JsonArray: Expected to start with '[':\n");
    *p_p = p;

    return;
  }

  *p_p = p;

  for (some = false, comma = false;;) {
    p = skip_(p_p);

    switch (*p) {
      case '\0':
        string_new_escape(&_s, op);
        string_new_f(
            p_e,
            "JsonArray: String ended before array complete in '%s'",
            _s);
        string_delete(&_s);
        *p_p = p;

        return;

      case ']':
        if (comma) {
          string_new_escape(&_s, op);
          string_new_escape(&_s2, p);
          string_new_f(
              p_e,
              "JsonArray: Got ']' but expecting element after ',' "
              "at '%s' in '%s'",
              _s2,
              _s);
          string_delete(&_s2);
          string_delete(&_s);
        }

        *p_p = p + 1;

        return;

      case ',':
        if (!some) {
          string_new_escape(&_s, op);
          string_new_escape(&_s2, p);
          string_new_f(
              p_e,
              "JsonArray: Got ',' before any elements at '%s' in '%s'",
              _s2,
              _s);
          string_delete(&_s2);
          string_delete(&_s);
          *p_p = p;

          return;
        }

        if (comma) {
          string_new_escape(&_s, op);
          string_new_escape(&_s2, p);
          string_new_f(
              p_e,
              "JsonArray: Got ',' after another ',' at '%s' in '%s'",
              _s2,
              _s);
          string_delete(&_s2);
          string_delete(&_s);
          *p_p = p;

          return;
        }

        comma = true;
        *p_p = ++p;

        break;

      default:
        if (some && !comma) {
          string_new_escape(&_s, op);
          string_new_escape(&_s2, p);
          string_new_f(
              p_e,
              "JsonArray: Got element after another element without "
              "separating ',' at '%s' in '%s'",
              _s2,
              _s);
          string_delete(&_s2);
          string_delete(&_s);
          *p_p = p;

          return;
        }

        lp = p;
        json_new(&_j, &p);

        if (!json_is_value(_j)) {
          string_new_escape(&_s, op);
          string_new_escape(&_s2, lp);
          string_new_f(
              p_e,
              "JsonArray: Got non-value element of type %s at '%s' in '%s'",
              json_get_type_string(_j),
              _s2,
              _s);
          string_delete(&_s2);
          string_delete(&_s);
          json_delete(&_j);
          *p_p = p;

          return;
        }

        if (_j->_e != null) {
          string_new_indent(p_e, _j->_e, 1);
          string_renew_prepend_f(
              p_e,
              "JsonArray: Error in element %lu:\n",
              this->length);
          json_array_append(this, &_j);
          *p_p = p;

          return;
        }

        json_array_append(this, &_j);
        some = true;
        comma = false;
        break;
    }

    *p_p = p;
  }
}

void json_array_new_empty(JsonArray **p_this) {
  JsonArray *this;

  this = newo(p_this);

  new(&this->_js, 0);
}

void json_array_new_entire(JsonArray **p_this, char *s) {
  char *p;
  char *_err = null;

  p = s;
  json_array_new(p_this, &p, &_err);

  skip_(&p);

  check_entire_(s, p, &_err);

  if (_err != null) {
    throw(_err);
  }
}

void json_array_new_copy(JsonArray **p_this, JsonArray *other) {
  ulong u;
  Json *_j = null;
  JsonArray *this;

  json_array_new_empty(p_this);
  this = *p_this;

  for (u = 0; u < other->length; u++) {
    json_new_copy(&_j, other->_js[u]);
    json_array_append(this, &_j);
  }
}

void json_array_delete(JsonArray **p_this) {
  JsonArray *this;

  if ((this = *p_this) == null) {
    return;
  }

  delete(&this->_js, this->length, json);

  deleteo(p_this);
}

void json_array_print(JsonArray *this, Printer *p) {
  ulong u;
  ulong num;

  sprint(p, "JsonArray");
  rprint(p, "Number of elements: %lu", num = this->length);

  if (num > 0) {
    sprint(p, "Elements");

    for (u = 0; u < num; u++) {
      sprint(p, "Element %lu", u);
      json_print(this->_js[u], p);
      fprint(p);
    }

    fprint(p);
  }

  fprint(p);
}

void json_array_append(JsonArray *this, Json **p_j) {
  append_donate(&this->_js, this->length, p_j);
}

void json_array_append_array(JsonArray *this, Json **p_j) {
  ulong n;
  ulong u;
  Json *j;
  Json **js;
  JsonArray *a;

  j = *p_j;

  if (!json_is_array(j)) {
    throw("Can only append an array to an array");
  }

  a = json_get_json_array(j);

  n = a->length;
  js = a->_js;

  for (u = 0; u < n; u++) {
    json_array_append(this, &js[u]);
  }

  json_delete(p_j);
}

ulong json_array_length(JsonArray *this) {
  return this->length;
}

Json *json_array_get(JsonArray *this, long index) {
  ulong n;

  n = this->length;

  if (index < 0) {
    index += n;
  }

  if (index < 0 || index >= n) {
    return null;
  }

  return this->_js[index];
}

void json_array_write_file(JsonArray *this, File *file, bool pretty, byte i) {
  ushort k;
  ulong num;
  ulong u;
  Json *j;
  Json **js;

  file_write_char(file, '[');

  num = this->length;

  if (pretty) {
    i++;
    file_write_char(file, '\n');

    if (num > 0) {
      for (k = 0; k < (ushort)i * INDENT_; k++) {
        file_write_char(file, ' ');
      }
    }
  }

  js = this->_js;

  for (u = 0; u < num; u++) {
    if (!pretty && u > 0) {
      file_write_char(file, ',');
    }

    j = js[u];
    json_write_file(j, file, pretty, i);

    if (pretty) {
      if (u < num - 1) {
        file_write_string(file, ",");
      }

      file_write_string(file, "\n");

      if (u < num - 1) {
        for (k = 0; k < (ushort)i * INDENT_; k++) {
          file_write_char(file, ' ');
        }
      }
    }
  }

  if (pretty) {
    i--;

    for (k = 0; k < (ushort)i * INDENT_; k++) {
      file_write_char(file, ' ');
    }
  }

  file_write_char(file, ']');
}

void long_array_new_from_json_entire(long **p_this, ulong *p_n, char *s) {
  ulong n;
  ulong u;
  JsonArray *_ja = null;
  long *this;

  json_array_new_entire(&_ja, s);

  n = *p_n = json_array_length(_ja);

  new(p_this, n);
  this = *p_this;

  for (u = 0; u < n; u++) {
    this[u] = json_get_long(json_array_get(_ja, u));
  }

  json_array_delete(&_ja);
}

ulong *ulong_array_new_from_json_entire(ulong **p_this, ulong *p_n, char *s) {
  ulong n;
  ulong u;
  JsonArray *_ja = null;
  ulong *this;

  json_array_new_entire(&_ja, s);

  n = *p_n = json_array_length(_ja);

  this = new(p_this, n);

  for (u = 0; u < n; u++) {
    this[u] = json_get_ulong(json_array_get(_ja, u));
  }

  json_array_delete(&_ja);

  return this;
}

char **string_array_new_from_json_entire(char ***p_this, ulong *p_n, char *s) {
  ulong n;
  ulong u;
  JsonArray *_ja = null;
  char **this;

  json_array_new_entire(&_ja, s);

  n = *p_n = json_array_length(_ja);

  this = new(p_this, n);

  for (u = 0; u < n; u++) {
    string_new_copy(&this[u], json_get_string(json_array_get(_ja, u)));
  }

  json_array_delete(&_ja);

  return this;
}

void json_object_new(JsonObject **p_this, char **p_p, char **p_e) {
  bool some;
  byte phase;
  char *_key = null;
  char *m;
  char *lp;
  char *op;
  char *p;
  char *_s = null;
  char *_s2 = null;
  Json *_j = null;
  JsonObject *this;

  object_new_(p_this);
  this = *p_this;

  op = *p_p;
  p = skip_(p_p);

  if (!get_("{", &p, p_e)) {
    string_renew_indent(p_e, 1);
    string_renew_prepend(p_e, "JsonObject: Expected to start with '{':\n");
    *p_p = p;
    dictionary_finalize(this->_d);

    return;
  }

  *p_p = p;

  for (some = false, phase = 0;;) {
    p = skip_(p_p);

    switch (*p) {
      case '\0':
        string_new_escape(&_s, op);
        string_new_f(
            p_e,
            "JsonObject: String ended before object complete in '%s'",
            _s);
        string_delete(&_s);
        *p_p = p;
        string_delete(&_key);
        dictionary_finalize(this->_d);

        return;

      case '}':
        if (phase != 0) {
          string_new_escape(&_s, op);
          string_new_escape(&_s2, p);
          m =
            phase == 3 ? "pair after ','" :
            phase == 2 ? "':' and value after key " :
            "value after key and ':'";
          string_new_f(
              p_e,
              "JsonObject: Got '}' but expected %s at '%s' in '%s'",
              m,
              _s2,
              _s);
          string_delete(&_s2);
          string_delete(&_s);
        }

        *p_p = p + 1;
        string_delete(&_key);
        dictionary_finalize(this->_d);

        return;

      case ',':
        if (!some || phase != 0) {
          string_new_escape(&_s, op);
          string_new_escape(&_s2, p);
          m =
            !some ? "before any elements" :
            phase == 3 ? "after another ','" :
            phase == 2 ? "after key but before ':' and value" :
            "after key and ':' but before value";
          string_new_f(
              p_e,
              "JsonObject: Got ',' %s at '%s' in '%s'",
              m,
              _s2,
              _s);
          string_delete(&_s2);
          string_delete(&_s);
          *p_p = p;
          string_delete(&_key);
          dictionary_finalize(this->_d);

          return;
        }

        phase = 3;
        *p_p = ++p;

        break;

      case ':':
        if (phase != 2) {
          string_new_escape(&_s, op);
          string_new_escape(&_s2, p);
          m =
            phase == 0 ? (!some ? "before the first key" : "instead of ','") :
            phase == 3 ? "after ','" :
            "after another ':'";
          string_new_f(
              p_e,
              "JsonObject: Got ':' %s at '%s' in '%s'",
              m,
              _s2,
              _s);
          string_delete(&_s2);
          string_delete(&_s);
          *p_p = p;
          string_delete(&_key);
          dictionary_finalize(this->_d);

          return;
        }

        phase = 1;
        *p_p = ++p;

        break;

      default:
        if (phase == 2 || (some && phase == 0)) {
          m =
            phase == 2 ? "after key string" :
            "after pair without needed ','";
          string_new_escape(&_s, op);
          string_new_escape(&_s2, p);
          string_new_f(
              p_e,
              "JsonObject: Got element %s at '%s' in '%s'",
              m,
              _s2,
              _s);
          string_delete(&_s2);
          string_delete(&_s);
          *p_p = p;
          string_delete(&_key);
          dictionary_finalize(this->_d);

          return;
        }

        lp = p;
        json_new(&_j, &p);

        if (phase == 0 || phase == 3) {
          if (_j->type != JSON_STRING) {
            string_new_escape(&_s, op);
            string_new_escape(&_s2, lp);
            string_new_f(
                p_e,
                "JsonObject: Expected key string but got element "
                "of type %s at '%s' in '%s'",
                json_get_type_string(_j),
                _s2,
                _s);
            string_delete(&_s2);
            string_delete(&_s);
            json_delete(&_j);
            *p_p = p;
            string_delete(&_key);
            dictionary_finalize(this->_d);

            return;
          }

          string_new_copy(&_key, json_get_json_string(_j)->_s);
          json_delete(&_j);
          phase = 2;
        } else {
          if (!json_is_value(_j)) {
            string_new_escape(&_s, op);
            string_new_escape(&_s2, lp);
            string_new_f(
                p_e,
                "JsonObject: Got non-value element of type %s at '%s' in '%s'",
                json_get_type_string(_j),
                _s2,
                _s);
            string_delete(&_s2);
            string_delete(&_s);
            json_delete(&_j);
            *p_p = p;
            string_delete(&_key);
            dictionary_finalize(this->_d);

            return;
          }

          if (_j->_e != null) {
            string_new_indent(p_e, _j->_e, 1);
            string_renew_prepend_f(p_e, "JsonObject: Error in element:\n");
            json_object_donate(this, &_key, &_j);
            *p_p = p;
            dictionary_finalize(this->_d);

            return;
          }

          json_object_donate(this, &_key, &_j);
          some = true;
          phase = 0;
        }

        break;
    }

    *p_p = p;
  }

  dictionary_finalize(this->_d);
}

void json_object_new_empty(JsonObject **p_this) {
  JsonObject *this;

  object_new_(p_this);
  this = *p_this;

  dictionary_finalize(this->_d);
}

void json_object_new_copy(JsonObject **p_this, JsonObject *other) {
  char *_key = null;
  ulong n;
  ulong u;
  Json *_j = null;
  JsonObject *this;
  JsonObjectPair *pair;
  JsonObjectPair **pairs;
  JsonObjectPairs *_pairs = null;

  object_new_(p_this);
  this = *p_this;

  json_object_pairs_new(&_pairs, other);

  n = _pairs->cardinality;
  pairs = _pairs->_pairs;

  for (u = 0; u < n; u++) {
    pair = pairs[u];
    string_new_copy(&_key, pair->key);
    json_new_copy(&_j, pair->j);
    json_object_donate(this, &_key, &_j);
  }

  json_object_pairs_delete(&_pairs);
  dictionary_finalize(this->_d);
}

void json_object_delete(JsonObject **p_this) {
  JsonObject *this;

  if ((this = *p_this) == null) {
    return;
  }

  dictionary_delete(&this->_d);

  deleteo(p_this);
}

void json_object_print(JsonObject *this, Printer *p) {
  sprint(p, "JsonObject");
  dictionary_print(this->_d, p);
  fprint(p);
}

void json_object_donate(JsonObject *this, char **p_key, Json **p_j) {
  bool finalized;

  if ((finalized = dictionary_is_finalized(this->_d))) {
    dictionary_reopen(this->_d);
  }

  dictionary_donate(this->_d, p_key, p_j);

  if (finalized) {
    dictionary_finalize(this->_d);
  }
}

void *json_object_get(JsonObject *this, char *key) {
  return dictionary_find(this->_d, key);
}

ulong json_object_cardinality(JsonObject *this) {
  return dictionary_cardinality(this->_d);
}

void json_object_write_file(
    JsonObject *this,
    File *file,
    bool pretty,
    byte i) {
  ushort k;
  ulong num;
  ulong u;
  DictionaryPair *pair;
  DictionaryPairs *_pairs = null;

  file_write_char(file, '{');

  dictionary_pairs_new(&_pairs, this->_d, true);
  num = _pairs->cardinality;

  if (pretty) {
    i++;
    file_write_char(file, '\n');

    if (num > 0) {
      for (k = 0; k < (ushort)i * INDENT_; k++) {
        file_write_char(file, ' ');
      }
    }
  }

  for (u = 0; u < num; u++) {
    if (!pretty && u > 0) {
      file_write_char(file, ',');
    }

    pair = _pairs->_pairs[u];
    file_write_char(file, '"');
    file_write_string(file, pair->key);
    file_write_string(file, "\":");

    if (pretty) {
      file_write_char(file, ' ');
    }

    json_write_file((Json *)pair->object, file, pretty, i);

    if (pretty) {
      if (u < num - 1) {
        file_write_string(file, ",");
      }

      file_write_string(file, "\n");

      if (u < num - 1) {
        for (k = 0; k < (ushort)i * INDENT_; k++) {
          file_write_char(file, ' ');
        }
      }
    }
  }

  dictionary_pairs_delete(&_pairs);

  if (pretty) {
    i--;

    for (k = 0; k < (ushort)i * INDENT_; k++) {
      file_write_char(file, ' ');
    }
  }

  file_write_char(file, '}');
}

void json_object_pairs_new(JsonObjectPairs **p_this, JsonObject *j) {
  ulong num_pairs;
  ulong u;
  DictionaryPair *dpair;
  DictionaryPair **dps;
  DictionaryPairs *_dpairs = null;
  JsonObjectPair **pairs;
  JsonObjectPairs *this;

  this = newo(p_this);

  dictionary_pairs_new(&_dpairs, j->_d, true);
  dps = _dpairs->_pairs;

  num_pairs = this->cardinality = _dpairs->cardinality;

  new(&this->_pairs, num_pairs);
  pairs = this->_pairs;

  for (u = 0; u < num_pairs; u++) {
    dpair = dps[u];
    pair_new_(&pairs[u], dpair->key, dpair->object);
  }

  dictionary_pairs_delete(&_dpairs);
}

void json_object_pairs_delete(JsonObjectPairs **p_this) {
  JsonObjectPairs *this;

  if ((this = *p_this) == null) {
    return;
  }

  delete_(&this->_pairs, this->cardinality, pair);
  deleteo(p_this);
}


static void object_new_(JsonObject **p_this) {
  JsonObject *this;

  this = newo(p_this);

  dictionary_new(&this->_d, json_delete, json_print);
}

static void pair_new_(JsonObjectPair **p_this, char *key, Json *j) {
  JsonObjectPair *this;

  this = newo(p_this);

  this->key = key;
  this->j = j;
}

static void pair_delete_(JsonObjectPair **p_this) {
  JsonObjectPair *this;

  if ((this = *p_this) == null) {
    return;
  }

  deleteo(p_this);
}

static char *skip_(char **p_p) {
  char *p;

  for (p = *p_p; isspace(*p); p++) {
  }

  return (*p_p = p);
}

static bool get_(char *expected, char **p_p, char **p_e) {
  char c;
  char *e;
  char *p;
  char *_s = null;

  for (e = expected, p = *p_p; (c = *e) != '\0'; e++, p++) {
    if (*p != c) {
      string_new_escape(&_s, *p_p);
      string_new_f(
          p_e,
          "Expected '%s' but got '%s': error at offset %lu",
          expected,
          _s,
          p - *p_p);
      string_delete(&_s);
      *p_p = p;

      return false;
    }
  }

  *p_p = p;

  return true;
}

static bool get_sep_(char **p_p, char **p_e) {
  char c;
  char *_s = null;

  c = **p_p;

  if (c != '\0' && !isspace(c) && !ispunct(c)) {
    string_new_escape(&_s, *p_p);
    string_new_f(
        p_e,
        "Expected white space or punctuation but got '%s'",
        _s);
    string_delete(&_s);

    return false;
  }

  return true;
}

static void check_entire_(char *s, char *p, char **p_e) {
  char *_s2 = null;
  char *_s3 = null;

  if (*p != '\0') {
    if (*p_e != null) {
      string_renew_indent(p_e, 1);
    }

    string_new_escape(&_s2, s);
    string_new_escape(&_s3, p);
    string_renew_prepend_f(
        p_e,
        "Json: Expected to use entire string '%s' but didn't use '%s'%s",
        _s2,
        _s3,
        *p_e != null ? ":\n" : "");
    string_delete(&_s3);
    string_delete(&_s2);
  }
}
