/* Object class that is a helper class for KeyValueField. Each KeyValueByte
 * specifies how to extract a byte for a field in a KeyValuePair. Immutable
 * class. The hard work is done by KeyValueField.
 */
#ifndef KEY_VALUE_BYTE_H
#define KEY_VALUE_BYTE_H

#ifdef __cplusplus
extern "C" {
#endif

#include "jpc.h"

typedef struct {
  byte readByteMask;
  byte writeByteMask;
  sbyte shift;
  ulong valueMask;
} KeyValueByte;

/* Constructor. The values are actually computed in KeyValueField; this
 * constructor simply stores them.
 */
void key_value_byte_new(
    KeyValueByte **p_this,
    ulong value_mask,
    sbyte shift,
    byte byte_mask);

/* Copy.
 */
void key_value_byte_new_copy(KeyValueByte **p_this, KeyValueByte *other);

/* Load.
 */
void key_value_byte_new_load(KeyValueByte **p_this, File *f);

/* Destructor.
 */
void key_value_byte_delete(KeyValueByte **p_this);

/* Print.
 */
void key_value_byte_print(KeyValueByte *this, Printer *p);

/* Save.
 */
void key_value_byte_save(KeyValueByte *this, File *f);

/* Equals.
 */
bool key_value_byte_equals(KeyValueByte *this, KeyValueByte *other);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* _H */
