#include "KeyValueField.h"

void key_value_field_new(
    KeyValueField **p_this,
    ulong max_value,
    ulong *bit_offset) {
  static const byte m__[] = {
    0xff, 0x7f, 0x3f, 0x1f, 0x0f, 0x07, 0x03, 0x01, 0x00};

  byte bit_width;
  byte first_bit;
  byte last_bit;
  byte num_bytes;
  byte last_u;
  byte byte_mask;
  byte u;
  sbyte shift;
  ulong first_offset;
  ulong last_offset;
  ulong value_mask;
  KeyValueField *this;

  this = newo(p_this);

  this->maxValue = max_value;
  this->bitOffset = *bit_offset;

  this->bitWidth = bit_width = ulong_bit_width(max_value);

  if (bit_width == 0) {
    return;
  }

  this->firstOffset = first_offset = *bit_offset / 8;
  first_bit = *bit_offset % 8;

  *bit_offset += bit_width - 1;

  last_offset = *bit_offset / 8;
  last_bit = *bit_offset % 8;

  (*bit_offset)++;

  this->numBytes = num_bytes = last_offset - first_offset + 1;

  new(&this->_bytes, this->numBytes);

  last_u = num_bytes - 1;

  for (u = 0, shift = bit_width - 8 + first_bit;
      u < num_bytes;
      u++, shift -= 8) {
    byte_mask = m__[!u ? first_bit : 0];
    byte_mask ^= m__[u == last_u ? last_bit + 1 : 8];

    value_mask = (ulong)byte_mask;

    if (shift < 0) {
      value_mask >>= -shift;
    } else if (shift > 0) {
      value_mask <<= shift;
    }

    key_value_byte_new(&this->_bytes[u], value_mask, shift, byte_mask);
  }
}

void key_value_field_new_copy(KeyValueField **p_this, KeyValueField *other) {
  ulong bit_offset;

  bit_offset = other->bitOffset;
  key_value_field_new(p_this, other->maxValue, &bit_offset);
}

void key_value_field_new_load(KeyValueField **p_this, File *f) {
  ulong max_value;
  ulong bit_offset;

  load_label("[KeyValueField]", f);
  load_value(max_value, f);
  load_value(bit_offset, f);
  load_label("[/KeyValueField]", f);

  key_value_field_new(p_this, max_value, &bit_offset);
}

void key_value_field_delete(KeyValueField **p_this) {
  KeyValueField *this;

  if ((this = *p_this) == null) {
    return;
  }

  delete(&this->_bytes, this->numBytes, key_value_byte);

  deleteo(p_this);
}

void key_value_field_print(KeyValueField *this, Printer *p) {
  byte u;

  sprint(p, "KeyValueField");
  rprint(p, "Max value: %lu", this->maxValue);
  rprint(p, "Bit offset: %lu", this->bitOffset);
  rprint(p, "Bit width: %u", this->bitWidth);
  rprint(p, "Number of KeyValueBytes: %u", this->numBytes);
  rprint(p, "First offset: %lu", this->firstOffset);

  for (u = 0; u < this->numBytes; u++) {
    sprint(p, "Byte %u", u);
    key_value_byte_print(this->_bytes[u], p);
    fprint(p);
  }

  fprint(p);
}

void key_value_field_save(KeyValueField *this, File *f) {
  save_label("[KeyValueField]", f);
  save_value(this->maxValue, f);
  save_value(this->bitOffset, f);
  save_label("[/KeyValueField]", f);
}

bool key_value_field_equals(KeyValueField *this, KeyValueField *other) {
  return other->maxValue == this->maxValue
    && other->bitOffset == this->bitOffset;
}
