/* Object class that is a helper class for KeyValueInfo. Each KeyValueField
 * specifies the transformation between a field of a KeyValuePair and its
 * representation as an array of bytes. Immutable class. In turn, this class
 * sets up a collection of KeyValueByte objects for the individual bytes.
 */
#ifndef KEY_VALUE_FIELD_H
#define KEY_VALUE_FIELD_H

#ifdef __cplusplus
extern "C" {
#endif

#include "KeyValueByte.h"
#include "bits.h"

typedef struct {
  byte numBytes;
  byte bitWidth;
  ulong maxValue;
  ulong bitOffset;
  ulong firstOffset;
  KeyValueByte **_bytes;
} KeyValueField;

/* Constructor. The maximum value that will be stored in this field determines
 * how many bits it will require. Because this field will in general be one of
 * many, the pointer to bit_offset both tells it what its starting bit offset
 * is in the KeyValuePair, as well as allowing it to update it by adding on the
 * number of bits used by this field itself.
 */
void key_value_field_new(
    KeyValueField **p_this,
    ulong max_value,
    ulong *bit_offset);

/* Copy.
 */
void key_value_field_new_copy(KeyValueField **p_this, KeyValueField *other);

/* Load.
 */
void key_value_field_new_load(KeyValueField **p_this, File *f);

/* Destructor.
 */
void key_value_field_delete(KeyValueField **p_this);

/* Print.
 */
void key_value_field_print(KeyValueField *this, Printer *p);

/* Save.
 */
void key_value_field_save(KeyValueField *this, File *f);

/* Equals.
 */
bool key_value_field_equals(KeyValueField *this, KeyValueField *other);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* _H */
