#include "KeyValueNode.h"

static void merge_children_(KeyValueNode *this);


void key_value_node_new(KeyValueNode **p_this, KeyValueNode *parent) {
  KeyValueNode *this;

  this = newo(p_this);

  this->parent = parent;
}

void key_value_node_delete(KeyValueNode **p_this) {
  KeyValueNode *this;

  if ((this = *p_this) == null) {
    return;
  }

  key_value_node_delete(&this->_left);
  key_value_node_delete(&this->_right);
  key_value_array_delete(&this->_array);

  deleteo(p_this);
}

void key_value_node_print(KeyValueNode *this, Printer *p) {
  sprint(p, "KeyValueNode");

  if (this->_array != null) {
    sprint(p, "Array");
    key_value_array_print(this->_array, p);
    fprint(p);
  }

  if (this->_left != null) {
    sprint(p, "Left child");
    key_value_node_print(this->_left, p);
    fprint(p);
  }

  if (this->_right != null) {
    sprint(p, "Right child");
    key_value_node_print(this->_right, p);
    fprint(p);
  }

  fprint(p);
}

bool key_value_node_try_merge_pair(KeyValueNode *this, KeyValuePair *pair) {
  KeyValueArray *array;

  array = this->_array;

  if (array != null && key_value_array_try_merge_pair(array, pair)) {
    return true;
  }

  if (this->_left != null
      && key_value_node_try_merge_pair(this->_left, pair)) {
    return true;
  }

  if (this->_right != null
      && key_value_node_try_merge_pair(this->_right, pair)) {
    return true;
  }

  return false;
}

void key_value_node_add_chunk(
    KeyValueNode *this,
    KeyValueArray **p_chunk,
    byte level,
    ulong index) {
  bool right;
  KeyValueArray *chunk;

  if (this->_array != null) {
    throw("Attempt to add chunk to node that already has an array: index %lu",
        index);
  }

  if (level > 0) {
    right = (index & (1UL << (level - 1))) != 0;

    if (right && this->_right == null) {
      key_value_node_new(&this->_right, this);
    }

    if (!right && this->_left == null) {
      key_value_node_new(&this->_left, this);
    }

    key_value_node_add_chunk(
        right ? this->_right : this->_left,
        p_chunk,
        level - 1,
        index);

    return;
  }

  chunk = *p_chunk;

  if (!chunk->finalized) {
    throw("Chunk is not finalized");
  }

  donate(p_chunk, &this->_array);

  merge_children_(this->parent);
}

void key_value_node_force_merge_children(KeyValueNode *this) {
  if (this->_array != null) {
    if (this->_left != null || this->_right != null) {
      throw("Impossible: Node has both an array and at least one child");
    }

    return;
  }

  if (this->_left == null) {
    throw("Impossible: Node has no array and no left child");
  }

  key_value_node_force_merge_children(this->_left);

  if (this->_left->_array == null) {
    throw("Impossible: Force merge on left child did not create an array");
  }

  if (this->_right != null) {
    key_value_node_force_merge_children(this->_right);

    if (this->_right->_array == null) {
      throw("Impossible: Force merge on right child did not create an array");
    }

    if (!this->_left->_array->finalized) {
      throw("Impossible: Left array is not finalized (force)");
    }

    if (!this->_right->_array->finalized) {
      throw("Impossible: Right array is not finalized (force)");
    }

    key_value_array_merge(this->_left->_array, this->_right->_array);
    key_value_node_delete(&this->_right);
  }

  donate(&this->_left->_array, &this->_array);

  key_value_node_delete(&this->_left);
}


static void merge_children_(KeyValueNode *this) {
  if (this == null) {
    return;
  }

  if (this->_left != null
      && this->_left->_array != null
      && this->_right != null
      && this->_right->_array != null) {
    if (this->_array != null) {
      throw("Impossible: Node has array and both children have array");
    }

    if (!this->_left->_array->finalized) {
      throw("Impossible: Left array is not finalized");
    }

    if (!this->_right->_array->finalized) {
      throw("Impossible: Right array is not finalized");
    }

    key_value_array_merge(this->_left->_array, this->_right->_array);
    key_value_node_delete(&this->_right);

    donate(&this->_left->_array, &this->_array);
    key_value_node_delete(&this->_left);

    merge_children_(this->parent);
  }
}
