#define LOSS_FUNCTION_C
#include "LossFunction.h"
#include <float.h>
#include <math.h>

typedef struct {
  ulong numExamples;
  SwappableDoubles *labelsS;
  double *_savedLabels;
  Ranker *_ranker;
} Mae_;

typedef struct {
  double maxLabelPredictionOrEstimate;
  double minLabelPredictionOrEstimate;
} Log_;

typedef struct {
  double maxLabelEstimate;
  double maxLabelPrediction;
  double minLabelEstimate;
  double minLabelPrediction;
  double p;
} BalancedLog_;

static Lock lock_ = LOCK_INITIALIZER;
static LossFunction *_lfs_ = null;

static void create_params_mse_(
    void **p_params,
    ulong num_examples,
    SwappableDoubles *labels,
    SwappableUlongs *frequencies);

static void load_params_mse_(void **p_params, File *f);

static void save_params_mse_(void *params, File *f);

static double compute_label_estimate_mse_(
    double label_prediction,
    void *params);

static double compute_loss_mse_(
    double label,
    double label_estimate,
    void *params);

static void reset_params_mse_(void *params);

static void destroy_params_mse_(void **p_params);

static void create_params_mae_(
    void **p_params,
    ulong num_examples,
    SwappableDoubles *labels,
    SwappableUlongs *frequencies);

static void load_params_mae_(void **p_params, File *f);

static void save_params_mae_(void *params, File *f);

static double compute_label_estimate_mae_(
    double label_prediction,
    void *params);

static double compute_loss_mae_(
    double label,
    double label_estimate,
    void *params);

static void reset_params_mae_(void *params);

static void destroy_params_mae_(void **p_params);

static void create_params_log_(
    void **p_params,
    ulong num_examples,
    SwappableDoubles *labels,
    SwappableUlongs *frequencies);

static void load_params_log_(void **p_params, File *f);

static void save_params_log_(void *params, File *f);

static double compute_label_estimate_log_(
    double label_prediction,
    void *params);

static double compute_loss_log_(
    double label,
    double label_estimate,
    void *params);

static void reset_params_log_(void *params);

static void destroy_params_log_(void **p_params);

static void create_params_balanced_log_(
    void **p_params,
    ulong num_examples,
    SwappableDoubles *labels,
    SwappableUlongs *frequencies);

static void load_params_balanced_log_(void **p_params, File *f);

static void save_params_balanced_log_(void *params, File *f);

static double compute_label_estimate_balanced_log_(
    double label_prediction,
    void *params);

static double compute_loss_balanced_log_(
    double label,
    double label_estimate,
    void *params);

static void reset_params_balanced_log_(void *params);

static void destroy_params_balanced_log_(void **p_params);

static void create_params_log_balanced_log_(
    double *min_label_prediction,
    double *max_label_prediction,
    double *p,
    ulong num_examples,
    SwappableDoubles *labels,
    SwappableUlongs *frequencies);

static double compute_loss_log_balanced_log_(
    double label,
    double label_estimate,
    void *params,
    bool balanced);

static sbyte cmp_(LossFunction *a, LossFunction *b, void *x);


LossFunction *loss_function_get(char *name) {
  byte i;
  LossFunction needle;
  LossFunction *lf;

  if (_lfs_ == null) {
    lock(&lock_);

    if (_lfs_ == null) {
      new(&_lfs_, LOSS_FUNCTIONS_NUM);

      lf = &_lfs_[i = 0];
      lf->name = LOSS_FUNCTION_MSE;
      lf->createParams = create_params_mse_;
      lf->loadParams = load_params_mse_;
      lf->saveParams = save_params_mse_;
      lf->computeLabelEstimate = compute_label_estimate_mse_;
      lf->computeLoss = compute_loss_mse_;
      lf->resetParams = reset_params_mse_;
      lf->destroyParams = destroy_params_mse_;

      lf = &_lfs_[++i];
      lf->name = LOSS_FUNCTION_MAE;
      lf->createParams = create_params_mae_;
      lf->loadParams = load_params_mae_;
      lf->saveParams = save_params_mae_;
      lf->computeLabelEstimate = compute_label_estimate_mae_;
      lf->computeLoss = compute_loss_mae_;
      lf->resetParams = reset_params_mae_;
      lf->destroyParams = destroy_params_mae_;

      lf = &_lfs_[++i];
      lf->name = LOSS_FUNCTION_LOG;
      lf->createParams = create_params_log_;
      lf->loadParams = load_params_log_;
      lf->saveParams = save_params_log_;
      lf->computeLabelEstimate = compute_label_estimate_log_;
      lf->computeLoss = compute_loss_log_;
      lf->resetParams = reset_params_log_;
      lf->destroyParams = destroy_params_log_;

      lf = &_lfs_[++i];
      lf->name = LOSS_FUNCTION_BALANCED_LOG;
      lf->createParams = create_params_balanced_log_;
      lf->loadParams = load_params_balanced_log_;
      lf->saveParams = save_params_balanced_log_;
      lf->computeLabelEstimate = compute_label_estimate_balanced_log_;
      lf->computeLoss = compute_loss_balanced_log_;
      lf->resetParams = reset_params_balanced_log_;
      lf->destroyParams = destroy_params_balanced_log_;

      if (++i != LOSS_FUNCTIONS_NUM) {
        throw("You forgot to add details for %lu loss function%s",
            LOSS_FUNCTIONS_NUM - i,
            plural_s(LOSS_FUNCTIONS_NUM - i));
      }

      qsortx(_lfs_, LOSS_FUNCTIONS_NUM, cmp_, false, null);
    }

    for (i = 0; i < LOSS_FUNCTIONS_NUM; i++) {
      lf = loss_function_get(LOSS_FUNCTIONS[2 * i]);
      lf->description = LOSS_FUNCTIONS[2 * i + 1];
    }

    unlock(&lock_);
  }

  needle.name = name;

  if ((lf = bsearchx(&needle, _lfs_, LOSS_FUNCTIONS_NUM, cmp_, null))
      == null) {
    throw("Loss function '%s' is not defined", name);
  }

  return lf;
}


static void create_params_mse_(
    void **p_params,
    ulong num_examples,
    SwappableDoubles *labels,
    SwappableUlongs *frequencies) {
}

static void load_params_mse_(void **p_params, File *f) {
}

static void save_params_mse_(void *params, File *f) {
}

static double compute_label_estimate_mse_(
    double label_prediction,
    void *params) {
  return label_prediction;
}

static double compute_loss_mse_(
    double label,
    double label_estimate,
    void *params) {
  double error;

  if (double_pawized_is_infinite_or_nan(label)) {
    if (double_is_infinite_or_nan(label)) {
      throw("Invalid infinite or NaN label value %.16g", label);
    }

    throw("Label value %.16g s out of paw range", label);
  }

  if (double_pawized_is_infinite_or_nan(label_estimate)) {
    if (double_is_infinite_or_nan(label_estimate)) {
      throw("Invalid infinite or NaN label estimate %.16g", label_estimate);
    }

    throw("Label estimate %.16g s out of paw range", label_estimate);
  }

  error = label - label_estimate;

  return error * error;
}

static void reset_params_mse_(void *params) {
}

static void destroy_params_mse_(void **p_params) {
}

static void create_params_mae_(
    void **p_params,
    ulong num_examples,
    SwappableDoubles *labels,
    SwappableUlongs *frequencies) {
  ulong e;
  ulong *_frequencies = null;
  double label;
  double *saved_labels;
  double *_labels = null;
  Mae_ *mae;
  Ranker *ranker;

  mae = newo((Mae_ **)p_params);
  mae->numExamples = num_examples;
  mae->labelsS = labels;
  ranker_new(&mae->_ranker);
  ranker = mae->_ranker;
  swappable_values_access(labels, &_labels);
  swappable_values_access(frequencies, &_frequencies);
  saved_labels = new(&mae->_savedLabels, num_examples);

  for (e = 0; e < num_examples; e++) {
    ranker_add(ranker, label = _labels[e], _frequencies[e]);
    saved_labels[e] = label;
  }

  ranker_finalize(ranker);

  for (e = 0; e < num_examples; e++) {
    _labels[e] = ranker_crank_from_value(ranker, _labels[e]);
  }

  swappable_values_release(frequencies, &_frequencies);
  swappable_values_release(labels, &_labels);
}

static void load_params_mae_(void **p_params, File *f) {
  bool do_labels;
  Mae_ *mae;

  mae = newo((Mae_ **)p_params);
  load_value(do_labels, f);

  if (do_labels) {
    load_value(mae->numExamples, f);
    load_value(mae->labelsS, f);
    load_new_values(mae->_savedLabels, mae->numExamples, f);
  }

  load(mae->_ranker, ranker, f);
}

static void save_params_mae_(void *params, File *f) {
  bool do_labels;
  Mae_ *mae;

  mae = params;
  do_labels = mae->_savedLabels != null;
  save_value(do_labels, f);

  if (do_labels) {
    save_value(mae->numExamples, f);
    save_value(mae->labelsS, f);
    save_values(mae->_savedLabels, mae->numExamples, f);
  }

  save(mae->_ranker, ranker, f);
}

static double compute_label_estimate_mae_(
    double label_prediction,
    void *params) {
  Mae_ *mae;

  mae = params;

  return ranker_value_from_crank(mae->_ranker, label_prediction);
}

static double compute_loss_mae_(
    double label,
    double label_estimate,
    void *params) {
  double error;
  Mae_ *mae;

  if (double_pawized_is_infinite_or_nan(label)) {
    if (double_is_infinite_or_nan(label)) {
      throw("Invalid infinite or NaN label value %.16g", label);
    }

    throw("Label value %.16g s out of paw range", label);
  }

  if (double_pawized_is_infinite_or_nan(label_estimate)) {
    if (double_is_infinite_or_nan(label_estimate)) {
      throw("Invalid infinite or NaN label estimate %.16g", label_estimate);
    }

    throw("Label estimate %.16g s out of paw range", label_estimate);
  }

  mae = params;

  if (mae->_savedLabels != null) {
    label = compute_label_estimate_mae_(label, params);
  }

  error = label - label_estimate;

  return fabs(error);
}

static void reset_params_mae_(void *params) {
  double *_labels = null;
  Mae_ *mae;

  mae = params;

  swappable_values_access(mae->labelsS, &_labels);
  copy_values(mae->_savedLabels, _labels, mae->numExamples);
  swappable_values_release(mae->labelsS, &_labels);
  deletev(&mae->_savedLabels);
}

static void destroy_params_mae_(void **p_params) {
  Mae_ *mae;

  if ((mae = *(Mae_ **)p_params) == null) {
    return;
  }

  deletev(&mae->_savedLabels);
  ranker_delete(&mae->_ranker);
  deleteo((Mae_ **)p_params);
}

static void create_params_log_(
    void **p_params,
    ulong num_examples,
    SwappableDoubles *labels,
    SwappableUlongs *frequencies) {
  Log_ *params;

  params = newo((Log_ **)p_params);
  create_params_log_balanced_log_(
      &params->minLabelPredictionOrEstimate,
      &params->maxLabelPredictionOrEstimate,
      null,
      num_examples,
      labels,
      frequencies);
}

static void load_params_log_(void **p_params, File *f) {
  Log_ *params;

  params = newo((Log_ **)p_params);
  load_value(params->minLabelPredictionOrEstimate, f);
  load_value(params->maxLabelPredictionOrEstimate, f);
}

static void save_params_log_(void *params, File *f) {
  Log_ *pass;

  pass = params;
  save_value(pass->minLabelPredictionOrEstimate, f);
  save_value(pass->maxLabelPredictionOrEstimate, f);
}

static double compute_label_estimate_log_(
    double label_prediction,
    void *params) {
  Log_ *pass;

  pass = params;

  if (label_prediction < 0 || label_prediction > 1) {
    throw("Label prediction (%.16g) must be in [0, 1]", label_prediction);
  }

  return bound(label_prediction,
      pass->minLabelPredictionOrEstimate,
      pass->maxLabelPredictionOrEstimate);
}

static double compute_loss_log_(
    double label,
    double label_estimate,
    void *params) {
  return compute_loss_log_balanced_log_(
      label,
      label_estimate,
      params,
      false);
}

static void reset_params_log_(void *params) {
}

static void destroy_params_log_(void **p_params) {
  deleteo((Log_ **)p_params);
}

static void create_params_balanced_log_(
    void **p_params,
    ulong num_examples,
    SwappableDoubles *labels,
    SwappableUlongs *frequencies) {
  BalancedLog_ *params;

  params = newo((BalancedLog_ **)p_params);
  create_params_log_balanced_log_(
      &params->minLabelPrediction,
      &params->maxLabelPrediction,
      &params->p,
      num_examples,
      labels,
      frequencies);
  params->maxLabelEstimate = DBL_NAN;
  params->minLabelEstimate = compute_label_estimate_balanced_log_(
      params->minLabelPrediction,
      params);
  params->maxLabelEstimate = compute_label_estimate_balanced_log_(
      params->maxLabelPrediction,
      params);

  if (params->maxLabelEstimate < params->minLabelEstimate) {
    throw("Impossible");
  }
}

static void load_params_balanced_log_(void **p_params, File *f) {
  BalancedLog_ *params;

  params = newo((BalancedLog_ **)p_params);
  load_value(params->minLabelPrediction, f);
  load_value(params->maxLabelPrediction, f);
  load_value(params->p, f);
  params->maxLabelEstimate = DBL_NAN;
  params->minLabelEstimate = compute_label_estimate_balanced_log_(
      params->minLabelPrediction,
      params);
  params->maxLabelEstimate = compute_label_estimate_balanced_log_(
      params->maxLabelPrediction,
      params);
}

static void save_params_balanced_log_(void *params, File *f) {
  BalancedLog_ *pass;

  pass = params;
  save_value(pass->minLabelPrediction, f);
  save_value(pass->maxLabelPrediction, f);
  save_value(pass->p, f);
}

static double compute_label_estimate_balanced_log_(
    double label_prediction,
    void *params) {
  double bounded_label_prediction;
  double label_estimate;
  double p;
  double x;
  BalancedLog_ *pass;

  if (label_prediction < 0 || label_prediction > 1) {
    throw("Label prediction (%.16g) must be in [0, 1]", label_prediction);
  }

  pass = params;
  bounded_label_prediction = bound(label_prediction,
      pass->minLabelPrediction,
      pass->maxLabelPrediction);
  p = pass->p;
  x = (1 - p) * bounded_label_prediction;
  label_estimate = x / (p * (1 - bounded_label_prediction) + x);

  if (!double_is_nan(pass->maxLabelEstimate)
      && (label_estimate < pass->minLabelEstimate
        || label_estimate > pass->maxLabelEstimate)) {
    throw("Impossible: %.16g should be in [%.16g, %.16g]",
        label_estimate,
        pass->minLabelEstimate,
        pass->maxLabelEstimate);
  }

  return label_estimate;
}

static double compute_loss_balanced_log_(
    double label,
    double label_estimate,
    void *params) {
  return compute_loss_log_balanced_log_(label, label_estimate, params, true);
}

static void reset_params_balanced_log_(void *params) {
}

static void destroy_params_balanced_log_(void **p_params) {
  deleteo((BalancedLog_ **)p_params);
}

static void create_params_log_balanced_log_(
    double *min_label_prediction,
    double *max_label_prediction,
    double *p,
    ulong num_examples,
    SwappableDoubles *labels,
    SwappableUlongs *frequencies) {
  ulong e;
  ulong frequency;
  ulong total_frequency;
  ulong *_frequencies = null;
  double label;
  double sum;
  double *_labels = null;

  if (num_examples == 0) {
    throw("Number of examples cannot be zero");
  }

  swappable_values_access(labels, &_labels);
  swappable_values_access(frequencies, &_frequencies);

  for (e = 0, sum = 0, total_frequency = 0; e < num_examples; e++) {
    if ((label = _labels[e]) != 0 && label != 1) {
      swappable_values_release(frequencies, &_frequencies);
      swappable_values_release(labels, &_labels);

      throw("Invalid label value %.16g for example %lu (must be 0 or 1)",
          label,
          e);
    }

    if ((frequency = _frequencies[e]) == 0) {
      swappable_values_release(frequencies, &_frequencies);
      swappable_values_release(labels, &_labels);

      throw("Frequency cannnot be zero (example %lu)", e);
    }

    total_frequency += frequency;

    if (p != null) {
      sum += frequency * label;
    }
  }

  *min_label_prediction = 1.0 / (total_frequency + 1);
  *max_label_prediction = 1.0 - *min_label_prediction;

  if (p != null) {
    if (sum == 0 || sum == total_frequency) {
      swappable_values_release(frequencies, &_frequencies);
      swappable_values_release(labels, &_labels);

      throw("Cannot have no '%u' labels for Balanced Log Loss",
          sum == 0 ? 1 : 0);
    }

    *p = sum / total_frequency;
  }

  swappable_values_release(frequencies, &_frequencies);
  swappable_values_release(labels, &_labels);
}

static double compute_loss_log_balanced_log_(
    double label,
    double label_estimate,
    void *params,
    bool balanced) {
  double max_estimate;
  double min_estimate;
  double p;
  Log_ *log_params;
  BalancedLog_ *balanced_log_params;

  if (balanced) {
    balanced_log_params = params;
    min_estimate = balanced_log_params->minLabelEstimate;
    max_estimate = balanced_log_params->maxLabelEstimate;
    p = balanced_log_params->p;
  } else {
    log_params = params;
    min_estimate = log_params->minLabelPredictionOrEstimate;
    max_estimate = log_params->maxLabelPredictionOrEstimate;
  }

  if (label != 0 && label != 1) {
    throw("Invalid label value %.16g (must be 0 or 1)", label);
  }

  if (label_estimate < min_estimate || label_estimate > max_estimate) {
    throw("Invalid label estimate %.16g (must be in [%.16g, %.16g])",
        label_estimate,
        min_estimate,
        max_estimate);
  }

  if (balanced) {
    if (label == 0) {
      return -log(1 - label_estimate) / (2 * (1 - p));
    }

    return -log(label_estimate) / (2 * p);
  }

  if (label == 0) {
    return -log(1 - label_estimate);
  }

  return -log(label_estimate);
}

static sbyte cmp_(LossFunction *a, LossFunction *b, void *x) {
  char *na;
  char *nb;

  na = a->name;
  nb = b->name;

  return string_compare(na, nb, null);
}
