/* Object class for an object that contains all information needed for Bear for
 * a given loss function.
 */
#ifndef LOSS_FUNCTION_H
#define LOSS_FUNCTION_H

#ifdef __cplusplus
extern "C" {
#endif

#include "Ranker.h"
#include "paw.h"

/* The loss functions currently supported, and descriptions of them.
 */
#define LOSS_FUNCTION_MSE "MSE"
#define LOSS_FUNCTION_MAE "MAE"
#define LOSS_FUNCTION_LOG "LOG"
#define LOSS_FUNCTION_BALANCED_LOG "BALANCED_LOG"

#ifdef LOSS_FUNCTION_C
char *LOSS_FUNCTIONS[] = {
  LOSS_FUNCTION_MSE, "Mean squared error",
  LOSS_FUNCTION_MAE, "Mean absolute error",
  LOSS_FUNCTION_LOG, "Log",
  LOSS_FUNCTION_BALANCED_LOG, "Balanced log"
};
byte LOSS_FUNCTIONS_NUM = sizeof(LOSS_FUNCTIONS) / 16;
#else
extern char *LOSS_FUNCTIONS[];
extern byte LOSS_FUNCTIONS_NUM;
#endif

typedef struct {
  char *description;
  char *name;
  void (*createParams)(
      void **p_params,
      ulong num_examples,
      SwappableDoubles *labels,
      SwappableUlongs *frequencies);
  void (*loadParams)(void **p_params, File *f);
  void (*saveParams)(void *params, File *f);
  double (*computeLabelEstimate)(double label_prediction, void *params);
  double (*computeLoss)(double label, double label_estimate, void *params);
  void (*resetParams)(void *params);
  void (*destroyParams)(void **p_params);
} LossFunction;

/* Get a reference to the LossFunction structure for a given loss function.
 */
LossFunction *loss_function_get(char *name);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* _H */
