/* Object class that converts double values to ranks, for Bear with MAE loss.
 */
#ifndef RANKER_H
#define RANKER_H

#ifdef __cplusplus
extern "C" {
#endif

#include "KeyValueCollection.h"

typedef struct {
  ulong totalFrequency;
  double maxValue;
  double minValue;
  KeyValueCollection *_collection;
  KeyValueArray *_array;
} Ranker;

/* Constructor.
 */
void ranker_new(Ranker **p_this);

/* Copy constructor.
 */
void ranker_new_copy(Ranker **p_this, Ranker *other);

/* Load.
 */
void ranker_new_load(Ranker **p_this, File *f);

/* Destructor.
 */
void ranker_delete(Ranker **p_this);

/* Print.
 */
void ranker_print(Ranker *this, Printer *p);

/* Save.
 */
void ranker_save(Ranker *this, File *f);

/* Equals.
 */
bool ranker_equals(Ranker *this, Ranker *other);

/* Add a value with a given frequency. Throws if the value is infinite or not a
 * number, or if the frequency is zero. If the value is negative zero or
 * subnormal, it is taken to be zero.
 */
void ranker_add(Ranker *this, double value, ulong frequency);

/* Finalize. Cannot be called if the ranker is already finalized.
 */
void ranker_finalize(Ranker *this);

/* Whether the ranker is finalized.
 */
bool ranker_is_finalized(Ranker *this);

/* Compute the centered rank corresponding to a given value.
 */
long ranker_crank_from_value(Ranker *this, double value);

/* Compute the value corresponding to a given centered rank, which may be a
 * double.
 */
double ranker_value_from_crank(Ranker *this, double crank);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* _H */
