#include "SparseKeyFrequency.h"

static void merge_(KeyValuePair *acc, KeyValuePair *add, KeyValueInfo *info);

static bool get_invalid_(KeyValuePair *pair, KeyValueInfo *info);

static void set_invalid_(KeyValuePair *pair, KeyValueInfo *info);


void sparse_key_frequency_info_new(
    KeyValueInfo **p_this,
    ulong num_keys,
    ulong *max_keys,
    ulong max_frequency) {
  KeyValueInfo *this;

  key_value_info_new(
      p_this,
      num_keys,
      max_keys,
      1,
      &max_frequency);
  this = *p_this;
  this->merge = merge_;
  this->setInvalid = set_invalid_;
  this->getInvalid = get_invalid_;
}

ulong sparse_key_frequency_pair_get_frequency(
    KeyValuePair *pair,
    KeyValueInfo *info) {
  return key_value_pair_get_value(pair, info, 0);
}

void sparse_key_frequency_pair_set_frequency(
    KeyValuePair *pair,
    KeyValueInfo *info,
    ulong frequency) {
  if (frequency == 0) {
    throw("Invalid zero frequency");
  }

  key_value_pair_set_value(pair, info, 0, frequency);
}


static void merge_(KeyValuePair *acc, KeyValuePair *add, KeyValueInfo *info) {
  ulong f1;
  ulong f2;

  f1 = sparse_key_frequency_pair_get_frequency(acc, info);
  f2 = sparse_key_frequency_pair_get_frequency(add, info);
  sparse_key_frequency_pair_set_frequency(acc, info, f1 + f2);
  set_invalid_(add, info);
}

static bool get_invalid_(KeyValuePair *pair, KeyValueInfo *info) {
  return key_value_pair_get_value(pair, info, 0) == 0;
}

static void set_invalid_(KeyValuePair *pair, KeyValueInfo *info) {
  key_value_pair_set_value(pair, info, 0, 0);
}
