/* Special case of key-value pairs when there is a single value field that is
 * the frequency of the pair, but where arrays using such pairs can be "sparse"
 * by having empty (invalid) pairs for which the frequency is set to zero. This
 * means that the implementations of the methods in this convenience class are
 * slightly different than those of KeyFrequency, but the interface is the
 * same. Where there is no specific method in this class, that of KeyFrequency
 * can be used.
 */
#ifndef SPARSE_KEY_FREQUENCY_H
#define SPARSE_KEY_FREQUENCY_H

#ifdef __cplusplus
extern "C" {
#endif

#include "KeyValueArray.h"
#include "KeyValueInfo.h"

/* Constructor.
 */
void sparse_key_frequency_info_new(
    KeyValueInfo **p_this,
    ulong num_keys,
    ulong *max_keys,
    ulong max_frequency);

/* Get the frequency.
 */
ulong sparse_key_frequency_pair_get_frequency(
    KeyValuePair *pair,
    KeyValueInfo *info);

/* Set the frequency.
 */
void sparse_key_frequency_pair_set_frequency(
    KeyValuePair *pair,
    KeyValueInfo *info,
    ulong frequency);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* _H */
