#define DOUBLE_C
#include "double.h"
#include <math.h>

typedef union {
  double d;
  struct {
    ulong m : 52;
    ulong e : 11;
    ulong n : 1;
  } a;
} C_;

ulong g_DOUBLE_nan_ = 0x7ff0000000000001;
ulong g_DOUBLE_neginf_ = 0xfff0000000000000;
ulong g_DOUBLE_negzero_ = 0x8000000000000000;
ulong g_DOUBLE_posinf_ = 0x7ff0000000000000;
ulong g_DOUBLE_subnormal_ = 0x1;


double double_from_parts(bool negative, ushort exponent, ulong mantissa) {
  C_ c;

  c.a.n = negative;
  c.a.e = exponent;
  c.a.m = mantissa;
  return c.d;
}

bool double_is_negative_zero(double d) {
  C_ c;

  c.d = d;
  return c.a.e == 0x0 && c.a.n == 1 && c.a.m == 0x0;
}

bool double_is_positive_infinity(double d) {
  C_ c;

  c.d = d;
  return c.a.e == 0x7ff && c.a.n == 0 && c.a.m == 0x0;
}

bool double_is_negative_infinity(double d) {
  C_ c;

  c.d = d;
  return c.a.e == 0x7ff && c.a.n == 1 && c.a.m == 0x0;
}

bool double_is_nan(double d) {
  C_ c;

  c.d = d;
  return c.a.e == 0x7ff && c.a.m != 0x0;
}

bool double_is_subnormal(double d) {
  C_ c;

  c.d = d;
  return c.a.e == 0x0 && c.a.m != 0x0;
}

bool double_is_negative_zero_or_subnormal(double d) {
  C_ c;

  c.d = d;
  return c.a.e == 0x0 && (c.a.n == 1 || c.a.m != 0x0);
}

bool double_is_infinite_or_nan(double d) {
  C_ c;

  c.d = d;
  return c.a.e == 0x7ff;
}
