/* Static class of convenient methods for IEEE 754 doubles. Because some of
 * this isn't in ANSI C. Will fail if doubles are not stored in IEEE 754 format
 * at all, which should be rare; this is checked in the unit tests.
 */
#ifndef DOUBLE_H
#define DOUBLE_H

#ifdef __cplusplus
extern "C" {
#endif

#include "bits.h"
#include "jpc.h"
#include "numbers.h"

/* Useful constants.
 */
#define DBL_POSINF (*(double *)&g_DOUBLE_posinf_)
#define DBL_NEGINF (*(double *)&g_DOUBLE_neginf_)
#define DBL_NEGZERO (*(double *)&g_DOUBLE_negzero_)

/* Examples.
 */
#define DBL_NAN (*(double *)&g_DOUBLE_nan_)
#define DBL_SUBNORMAL (*(double *)&g_DOUBLE_subnormal_)

/* Get a double from explicit negative, exponent, and mantissa values.
 */
double double_from_parts(bool negative, ushort exponent, ulong mantissa);

/* Whether a double is negative zero.
 */
bool double_is_negative_zero(double d);

/* Whether a double is positive infinity.
 */
bool double_is_positive_infinity(double d);

/* Whether a double is negative infinity.
 */
bool double_is_negative_infinity(double d);

/* Whether a double is not a number.
 */
bool double_is_nan(double d);

/* Whether a double is subnormal.
 */
bool double_is_subnormal(double d);

/* Whether a double is negative zero or subnormal. This method is more
 * efficient than calling the two methods individually.
 */
bool double_is_negative_zero_or_subnormal(double d);

/* Whether a double is positive or negative infinity or not a number. This
 * method is more efficient than calling the three methods individually.
 */
bool double_is_infinite_or_nan(double d);

#ifndef DOUBLE_C
extern ulong g_DOUBLE_nan_;
extern ulong g_DOUBLE_neginf_;
extern ulong g_DOUBLE_negzero_;
extern ulong g_DOUBLE_posinf_;
extern ulong g_DOUBLE_subnormal_;
#endif

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* _H */
