/* Common functionality for Bear programs.
 */
#ifndef PROG_BEAR_H
#define PROG_BEAR_H

#ifdef __cplusplus
extern "C" {
#endif

#include "Bear.h"
#include "Json.h"
#include "Swappable.h"
#include "args.h"

#define PROG_BEAR_MEMORY "memory"
#define PROG_BEAR_PREDICT "predict"
#define PROG_BEAR_FEATURES "features"

void prog_bear_args_1(char *prog);

void prog_bear_args_2(void);

void prog_bear_version(void);

void prog_bear_parse_args(
    bool has_header_row,
    bool has_frequency_column,
    ulong *p_num_labels,
    ulong **p_label_columns,
    char ***p_label_column_names,
    ulong *p_num_ids,
    ulong **p_id_columns,
    char ***p_id_column_names,
    ulong *p_frequency_column,
    char **p_frequency_column_name,
    bool predict);

void prog_bear_feedback_1(
    bool has_header_row,
    bool has_frequency_column,
    char **label_column_names,
    ulong num_labels,
    ulong *label_columns,
    ulong frequency_column,
    char *frequency_column_name,
    char *missing_string,
    Logger *_l);

void prog_bear_read_header(
    bool has_frequency_column,
    bool confirm_header_row,
    ulong num_labels,
    ulong num_ids,
    char *missing_string,
    char ***p_label_column_names,
    char ***p_id_column_names,
    char *frequency_column_name,
    ulong *p_frequency_column,
    TextFile *in,
    Logger *_l,
    ulong *p_num_column_names,
    char ***p_column_names,
    ulong **p_label_columns,
    ulong **p_id_columns,
    bool print_column_names);

void prog_bear_write_header(
    bool predict,
    bool debug,
    bool no_features_out,
    ulong num_ids,
    ulong *id_columns,
    ulong num_features,
    ulong *feature_columns,
    ulong num_labels,
    ulong *label_columns,
    ulong num_columns,
    ulong *p_num_column_names,
    char ***p_column_names,
    TextFile *out);

void prog_bear_parse_features(
    ulong num_features,
    ulong *feature_columns,
    ulong num_fields,
    char **fields,
    char *missing_string,
    double *features);

void prog_bear_check_columns(
    ulong num_labels,
    ulong *label_columns,
    ulong num_ids,
    ulong *id_columns,
    bool has_frequency_column,
    ulong frequency_column);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* _H */
