#include "bear_model_details.h"

/* Program for extracting the details of a Bear model file and writing them to
 * a file or to stdout.
 */
int main(int argc, char *argv[]) {
  char *_bear_filename = null;
  char *_filetype = null;
  char *_filename = null;
  char *_s = null;
  Bear *_bear = null;

  /* Command-line arguments.
   */
  args_description("Print details of a Bear model file");
  args_string(
      "bear-filename",
      "Name of the Bear model file");
  args_optional_string(
      "output-filename",
      'o',
      "-",
      "Write the details to this filename, or '-' for stdout");
  args_flag("verbose", 'v', "Verbose: include a full debug print");
  args_parse(argc, argv);

  /* Get filenames.
   */
  string_new_copy(&_bear_filename, args_get_string("bear-filename"));
  string_renew_ensure_ends_with(&_bear_filename, ".bear.gz", true);

  string_new_copy(&_filename, args_get_string("output-filename"));

  if (!string_equals(_filename, "-")) {
    string_new_extension_from_path(&_filetype, _filename);

    if (!text_file_type_is_valid(_filetype)) {
      string_delete(&_filetype);
      string_new_copy(&_filetype, "csv");
    }

    if (!string_ends_with(_filename, _filetype, true)) {
      string_renew_append_until_null(&_filename, ".", _filetype, null);
    }

    string_delete(&_filetype);
  }

  load_storage(_bear, bear, _bear_filename);
  bear_write_model_summary(_bear, _filename);

  if (args_set("verbose")) {
    aprint(&_s, 0, bear, _bear);
    printf("\n%s", _s);
    string_delete(&_s);
  }

  bear_delete(&_bear);
  string_delete(&_filename);
  string_delete(&_bear_filename);

  return 0;
}
