#include "bear_model_features.h"

/* List the features (by index) actually used in a Bear model file.
 */
int main(int argc, char *argv[]) {
  char *_bear_filename = null;
  char *_filename = null;
  ulong num_used_features;
  ulong u;
  ulong *_used_features = null;
  TextFile *_out = null;
  Bear *_bear = null;

  /* Command-line arguments.
   */
  args_description(
      "List all the features used in a Bear model file, by index");
  args_string(
      "bear-filename",
      "Name of the Bear model file");
  args_optional_string(
      "output-filename",
      'o',
      "-",
      "Write the features to this filename, or '-' for stdout");
  args_parse(argc, argv);

  /* Get filenames.
   */
  string_new_copy(&_bear_filename, args_get_string("bear-filename"));
  string_renew_ensure_ends_with(&_bear_filename, ".bear.gz", true);

  string_new_copy(&_filename, args_get_string("output-filename"));

  /* Create the output file.
   */
  text_file_new_type(&_out, _filename, "w", "txt");

  /* Load the model.
   */
  load_storage(_bear, bear, _bear_filename);

  /* Get the array of used features.
   */
  num_used_features = bear_get_used_features(_bear, &_used_features);

  /* Write them out, one per line.
   */
  for (u = 0; u < num_used_features; u++) {
    text_file_write_single_ulong(_out, _used_features[u]);
  }

  /* Close the output file.
   */
  text_file_delete(&_out);

  /* Clean up.
   */
  deletev(&_used_features);
  bear_delete(&_bear);
  string_delete(&_filename);
  string_delete(&_bear_filename);

  return 0;
}
