#include "intermediate_bear_tutorial_reformat.h"
#include <math.h>

/* Reformat a trivariate input or output file to make it easily visualizable
 * using Excel.
 */
int main(int argc, char *argv[]) {
  char *in_filename;
  char *out_filename;
  char **fields;
  ulong i;
  ulong j;
  ulong num_fields;
  ulong samples_per_side;
  Logger *_l = null;
  TextFile *_in = null;
  TextFile *_out = null;

  /* Arguments.
   */
  args_description(
      "Reformat the data rows of a data file created by "
      "intermediate_bear_tutorial_data or the output file from memory_bear "
      "run on such a file to make it easily visualizable using Excel");
  args_string(
      "in-filename",
      "Filename of the trivariate data file to reformat");
  args_integer(
      "samples-per-side",
      "Number of samples on each side of the base, in [2, 255]");
  args_string(
      "out-filename",
      "Filename of the desired reformatted output file");
  args_parse(argc, argv);

  logger_new(&_l, stdout);
  in_filename = args_get_string("in-filename");
  out_filename = args_get_string("out-filename");
  samples_per_side = args_get_bounded_integer("samples-per-side", 2, 255);

  rlog(_l, "Input filename: '%s'", in_filename);
  rlog(_l, "Output filename: '%s'", out_filename);
  rlog(_l, "Samples per side: %lu", samples_per_side);

  slog(_l, "Reformatting");
  text_file_new(&_in, args_get_string("in-filename"), "r");
  text_file_new(&_out, args_get_string("out-filename"), "w");

  for (j = 0; j < samples_per_side; j++) {
    for (i = 0; i < samples_per_side; i++) {
      if ((num_fields = text_file_read(_in, &fields)) != 3) {
        throw("Expected to read 3 fields, but read %lu", num_fields);
      }

      text_file_write(_out, fields[2]);
    }

    text_file_end_record(_out);
  }

  text_file_delete(&_out);
  text_file_delete(&_in);
  flog(_l);
  logger_delete(&_l);

  return 0;
}
