#include "spiral_classification_data.h"
#include <math.h>

#define N_ 100000
#define S_ 255
#define NOISE_ 1
#define SIDE_ 50

/* Create a standard spiral classification dataset plus prediction rows
 * allowing an easy visualization using Excel.
 */
int main(int argc, char *argv[]) {
  bool red;
  char *filename;
  ulong e;
  ulong i;
  ulong j;
  ulong n;
  ulong samples_per_side;
  double noise;
  double r;
  double scale;
  double theta;
  double x;
  double y;
  Logger *_l = null;
  Random *_random = null;
  TextFile *_tf = null;

  /* Arguments. The defaults will produce a file like that supplied on the page
   * https://johncostella.com/bear/intermediate/.
   */
  args_description(
      "Create a spiral classification dataset");
  args_string("filename", "Name of the file to create");
  args_optional_integer("num", 'n', N_, "Number of examples");
  args_optional_real(
      "noise",
      's',
      NOISE_,
      "Standard deviation of the noise to be added to r");
  args_optional_integer("pred-side", 'p', S_, "Prediction samples per side");
  args_parse(argc, argv);

  logger_new(&_l, stdout);
  filename = args_get_string("filename");
  n = args_get_positive_integer("num");
  noise = args_get_positive_real("noise");
  samples_per_side = args_get_bounded_integer("pred-side", 2, 255);
  scale = (double)SIDE_ / samples_per_side;

  text_file_new(&_tf, filename, "w");
  random_new(&_random);

  for (e = 0; e < n; e++) {
    red = random_modulo(_random, 2) == 1;
    theta = 3.5 * JPC_PI *sqrt(random_double_unit_cc(_random));
    r = 2 * theta + noise * random_normal(_random);
    x = r * sin(theta);
    y = r * cos(theta);

    if (red) {
      x = -x;
      y = -y;
    }

    text_file_write_double(_tf, x);
    text_file_write_double(_tf, y);
    text_file_write_ulong(_tf, red ? 1 : 0);
    text_file_end_record(_tf);
  }

  for (j = 0; j < samples_per_side; j++) {
    y = ((double)j - 0.5 * (samples_per_side - 1)) * scale;

    for (i = 0; i < samples_per_side; i++) {
      x = ((double)i - 0.5 * (samples_per_side - 1)) * scale;

      text_file_write_double(_tf, x);
      text_file_write_double(_tf, y);
      text_file_end_record(_tf);
    }
  }
  text_file_delete(&_tf);
  random_delete(&_random);
  logger_delete(&_l);

  return 0;
}
