#include "BearAssemblyTest.h"

START

TEST(BearAssembly, NewEmpty) {
  BearAssembly *_ = null;

  bear_assembly_new_empty(&_);
  EXPECT_ULEQ(0, _->numParts);
  EXPECT_TRUE(bear_assembly_is_empty(_));
  bear_assembly_delete(&_);
}

TEST(BearAssembly, NewElementary) {
  BearPart *part;
  BearAssembly *_ = null;

  bear_assembly_new_elementary(&_, 27);
  EXPECT_ULEQ(1, _->numParts);
  part = _->_parts[0];
  EXPECT_ULEQ(1, part->numFeatures);
  EXPECT_ULEQ(27, part->_features[0]);
  part = bear_assembly_last_part(_);
  EXPECT_ULEQ(1, part->numFeatures);
  EXPECT_ULEQ(27, part->_features[0]);
  EXPECT_FALSE(bear_assembly_is_empty(_));
  test_save_load(BearAssembly, bear_assembly, _);
  bear_assembly_delete(&_);
}

TEST(BearAssembly, NewCopy) {
  char *_s2 = null;
  char *_s3 = null;
  BearPart *_p = null;
  BearAssembly *_1 = null;
  BearAssembly *_2 = null;
  BearAssembly *_3 = null;

  bear_assembly_new_elementary(&_1, 34);
  bear_part_new_elementary(&_p, 12);
  bear_assembly_new_melt(&_2, _1, _p);
  bear_part_delete(&_p);
  bear_part_new_elementary(&_p, 56);
  bear_assembly_new_glue(&_3, _2, _p);
  bear_part_delete(&_p);
  bear_assembly_delete(&_2);
  bear_assembly_delete(&_1);

  aprint(&_s3, 0, bear_assembly, _3);
  bear_assembly_new_copy(&_2, _3);
  EXPECT_TRUE(bear_assembly_equals(_2, _3));
  aprint(&_s2, 0, bear_assembly, _2);
  EXPECT_STREQ(_s2, _s3);
  string_delete(&_s3);
  string_delete(&_s2);

  bear_assembly_delete(&_3);
  bear_assembly_delete(&_2);
}

TEST(BearAssembly, NewParent) {
  BearPart *p;
  BearPart *_p = null;
  BearAssembly *_1 = null;
  BearAssembly *_2 = null;
  BearAssembly *_3 = null;

  bear_assembly_new_elementary(&_1, 34);
  bear_part_new_elementary(&_p, 12);
  bear_assembly_new_melt(&_2, _1, _p);
  bear_part_delete(&_p);
  bear_part_new_elementary(&_p, 56);
  bear_assembly_new_glue(&_3, _2, _p);
  bear_part_delete(&_p);
  bear_assembly_delete(&_2);
  bear_assembly_delete(&_1);

  bear_assembly_new_parent(&_2, _3);
  EXPECT_ULEQ(1, _2->numParts);
  p = _2->_parts[0];
  EXPECT_ULEQ(2, p->numFeatures);
  EXPECT_ULEQ(12, p->_features[0]);
  EXPECT_ULEQ(34, p->_features[1]);

  bear_assembly_delete(&_3);
  bear_assembly_delete(&_2);
}

TEST(BearAssembly, NewGlue) {
  BearPart *p;
  BearPart *_p = null;
  BearAssembly *_1 = null;
  BearAssembly *_2 = null;

  bear_assembly_new_elementary(&_1, 34);
  bear_part_new_elementary(&_p, 12);
  bear_assembly_new_glue(&_2, _1, _p);
  EXPECT_ULEQ(2, _2->numParts);
  p = _2->_parts[0];
  EXPECT_ULEQ(1, p->numFeatures);
  EXPECT_ULEQ(34, p->_features[0]);
  p = _2->_parts[1];
  EXPECT_ULEQ(1, p->numFeatures);
  EXPECT_ULEQ(12, p->_features[0]);
  bear_part_delete(&_p);
  bear_assembly_delete(&_2);
  bear_assembly_delete(&_1);
}

TEST(BearAssembly, NewMelt) {
  BearPart *p;
  BearPart *_p = null;
  BearAssembly *_1 = null;
  BearAssembly *_2 = null;

  bear_assembly_new_elementary(&_1, 34);
  bear_part_new_elementary(&_p, 12);
  bear_assembly_new_glue(&_2, _1, _p);
  bear_part_delete(&_p);
  bear_assembly_delete(&_1);
  bear_part_new_elementary(&_p, 56);
  bear_assembly_new_melt(&_1, _2, _p);
  EXPECT_ULEQ(2, _1->numParts);
  p = _1->_parts[0];
  EXPECT_ULEQ(1, p->numFeatures);
  EXPECT_ULEQ(34, p->_features[0]);
  p = _1->_parts[1];
  EXPECT_ULEQ(2, p->numFeatures);
  EXPECT_ULEQ(12, p->_features[0]);
  EXPECT_ULEQ(56, p->_features[1]);
  bear_part_delete(&_p);
  bear_assembly_delete(&_2);
  bear_assembly_delete(&_1);
}

TEST(BearAssembly, Equals) {
  BearPart *_p = null;
  BearAssembly *_1 = null;
  BearAssembly *_2 = null;

  bear_assembly_new_empty(&_1);
  EXPECT_TRUE(bear_assembly_equals(_1, _1));
  bear_assembly_new_elementary(&_2, 24);
  EXPECT_FALSE(bear_assembly_equals(_1, _2));
  EXPECT_FALSE(bear_assembly_equals(_2, _1));
  bear_assembly_delete(&_1);

  bear_part_new_elementary(&_p, 9);
  bear_assembly_new_melt(&_1, _2, _p);
  EXPECT_TRUE(bear_assembly_equals(_1, _1));
  EXPECT_TRUE(bear_assembly_equals(_2, _2));
  EXPECT_FALSE(bear_assembly_equals(_2, _1));
  EXPECT_FALSE(bear_assembly_equals(_1, _2));

  bear_part_delete(&_p);
  bear_assembly_delete(&_2);
  bear_assembly_delete(&_1);
}

TEST(BearAssembly, LastPartIsRepeat) {
  BearPart *_p = null;
  BearAssembly *_1 = null;
  BearAssembly *_2 = null;
  BearAssembly *_3 = null;

  bear_assembly_new_elementary(&_1, 24);
  EXPECT_FALSE(bear_assembly_last_part_is_repeat(_1));
  bear_part_new_elementary(&_p, 9);
  bear_assembly_new_glue(&_2, _1, _p);
  EXPECT_FALSE(bear_assembly_last_part_is_repeat(_2));
  bear_assembly_delete(&_2);
  bear_part_delete(&_p);
  bear_part_new_elementary(&_p, 24);
  bear_assembly_new_glue(&_2, _1, _p);
  EXPECT_TRUE(bear_assembly_last_part_is_repeat(_2));
  bear_assembly_delete(&_2);
  bear_part_delete(&_p);
  bear_part_new_elementary(&_p, 9);
  bear_assembly_new_glue(&_2, _1, _p);
  bear_assembly_new_glue(&_3, _2, _p);
  EXPECT_TRUE(bear_assembly_last_part_is_repeat(_3));
  bear_assembly_delete(&_3);
  bear_part_delete(&_p);
  bear_part_new_elementary(&_p, 24);
  bear_assembly_new_glue(&_3, _2, _p);
  EXPECT_TRUE(bear_assembly_last_part_is_repeat(_3));
  bear_part_delete(&_p);
  bear_assembly_delete(&_3);
  bear_assembly_delete(&_2);
  bear_assembly_delete(&_1);
}

TEST(BearAssembly, IsParentSibling) {
  BearPart *_p = null;
  BearAssembly *_1 = null;
  BearAssembly *_2 = null;
  BearAssembly *_3 = null;
  BearAssembly *_4 = null;

  bear_assembly_new_empty(&_1);
  EXPECT_FALSE(bear_assembly_is_parent(_1, _1));
  EXPECT_FALSE(bear_assembly_is_sibling(_1, _1));

  bear_assembly_new_elementary(&_2, 24);
  EXPECT_TRUE(bear_assembly_is_parent(_1, _2));
  EXPECT_FALSE(bear_assembly_is_parent(_2, _1));
  EXPECT_FALSE(bear_assembly_is_sibling(_2, _2));
  EXPECT_FALSE(bear_assembly_is_sibling(_1, _2));
  EXPECT_FALSE(bear_assembly_is_sibling(_2, _1));
  bear_assembly_delete(&_1);

  bear_assembly_new_elementary(&_1, 9);
  bear_part_new_elementary(&_p, 24);
  bear_assembly_new_glue(&_3, _1, _p);
  EXPECT_TRUE(bear_assembly_is_parent(_1, _3));
  EXPECT_FALSE(bear_assembly_is_parent(_3, _1));
  EXPECT_FALSE(bear_assembly_is_sibling(_3, _3));
  EXPECT_FALSE(bear_assembly_is_sibling(_1, _3));
  EXPECT_FALSE(bear_assembly_is_sibling(_3, _1));

  bear_part_delete(&_p);
  bear_part_new_elementary(&_p, 7);
  bear_assembly_new_glue(&_4, _1, _p);
  EXPECT_TRUE(bear_assembly_is_parent(_1, _4));
  EXPECT_FALSE(bear_assembly_is_parent(_4, _1));
  EXPECT_FALSE(bear_assembly_is_sibling(_4, _4));
  EXPECT_TRUE(bear_assembly_is_sibling(_4, _3));
  EXPECT_TRUE(bear_assembly_is_sibling(_3, _4));

  bear_part_delete(&_p);
  bear_assembly_delete(&_4);
  bear_assembly_delete(&_3);
  bear_assembly_delete(&_2);
  bear_assembly_delete(&_1);
}

TEST(BearAssembly, Compare) {
  BearPart *_p = null;
  BearAssembly *_1 = null;
  BearAssembly *_2 = null;
  BearAssembly *_3 = null;

  bear_assembly_new_empty(&_1);
  EXPECT_LEQ(0, bear_assembly_compare(_1, _1, null));

  bear_assembly_new_elementary(&_2, 24);
  EXPECT_LEQ(0, bear_assembly_compare(_2, _2, null));
  EXPECT_LEQ(-1, bear_assembly_compare(_1, _2, null));
  EXPECT_LEQ(+1, bear_assembly_compare(_2, _1, null));
  bear_assembly_delete(&_1);

  bear_assembly_new_elementary(&_1, 9);
  bear_part_new_elementary(&_p, 24);
  bear_assembly_new_melt(&_3, _1, _p);
  EXPECT_LEQ(0, bear_assembly_compare(_3, _3, null));
  EXPECT_LEQ(-1, bear_assembly_compare(_1, _3, null));
  EXPECT_LEQ(+1, bear_assembly_compare(_3, _1, null));
  EXPECT_LEQ(-1, bear_assembly_compare(_2, _3, null));
  EXPECT_LEQ(+1, bear_assembly_compare(_3, _2, null));
  bear_assembly_delete(&_3);

  bear_assembly_new_glue(&_3, _1, _p);
  EXPECT_LEQ(0, bear_assembly_compare(_3, _3, null));
  EXPECT_LEQ(-1, bear_assembly_compare(_1, _3, null));
  EXPECT_LEQ(+1, bear_assembly_compare(_3, _1, null));
  EXPECT_LEQ(-1, bear_assembly_compare(_2, _3, null));
  EXPECT_LEQ(+1, bear_assembly_compare(_3, _2, null));

  bear_part_delete(&_p);
  bear_assembly_delete(&_3);
  bear_assembly_delete(&_2);
  bear_assembly_delete(&_1);
}

FINISH
