#include "BearFieldTest.h"
#include <float.h>

START

TEST(BearField, Simple) {
  ushort *_bins_from_indexes = null;
  BearField *_ = null;

  bear_field_new(&_, true);
  bear_field_add(_, 3.14159, 7, -24.0966);
  EXPECT_ULEQ(7, _->totalFrequency);
  bear_field_finalize(_);
  EXPECT_ULEQ(1, _->cardinality);
  EXPECT_ULEQ(7, _->totalFrequency);
  EXPECT_PAWEQ(paw_make(3.14159), _->_values[0]);
  EXPECT_DEQ(3.14159, _->_meanValues[0]);
  EXPECT_ULEQ(7, _->_frequencies[0]);
  EXPECT_DEQ(-24.0966, _->_meanResiduals[0]);
  EXPECT_ULEQ(0, bear_field_lookup(_, -1));
  EXPECT_ULEQ(0, bear_field_lookup(_, 3.13));
  EXPECT_ULEQ(0, bear_field_lookup(_, 3.14));
  EXPECT_ULEQ(0, bear_field_lookup(_, 3.15));
  EXPECT_ULEQ(0, bear_field_lookup(_, 100));
  new(&_bins_from_indexes, 1);
  bear_field_bin(_, _bins_from_indexes);
  EXPECT_ULEQ(0, bear_field_lookup(_, -1));
  EXPECT_ULEQ(0, bear_field_lookup(_, 3.13));
  EXPECT_ULEQ(0, bear_field_lookup(_, 3.14));
  EXPECT_ULEQ(0, bear_field_lookup(_, 3.15));
  EXPECT_ULEQ(0, bear_field_lookup(_, 100));
  test_save_load(BearField, bear_field, _);
  deletev(&_bins_from_indexes);
  bear_field_delete(&_);

  bear_field_new(&_, true);
  bear_field_add(_, 3.14159, 2, 3.14159);
  bear_field_add(_, -1.23456, 3, -1.23456);
  bear_field_add(_, 3.142857, 4, 3.142857);
  bear_field_finalize(_);
  EXPECT_ULEQ(2, _->cardinality);
  EXPECT_ULEQ(9, _->totalFrequency);
  EXPECT_PAWEQ(paw_make(-1.23456), _->_values[0]);
  EXPECT_DEQ(-1.23456, _->_meanValues[0]);
  EXPECT_ULEQ(3, _->_frequencies[0]);
  EXPECT_PAWEQ(paw_make(3.142), _->_values[1]);
  EXPECT_DEQ((3.14159 * 2 + 3.142857 * 4) / 6, _->_meanValues[1]);
  EXPECT_ULEQ(6, _->_frequencies[1]);
  EXPECT_ULEQ(0, bear_field_lookup(_, -10));
  EXPECT_ULEQ(0, bear_field_lookup(_, -1.23));
  EXPECT_ULEQ(1, bear_field_lookup(_, 3.14));
  EXPECT_ULEQ(0, bear_field_lookup(_, 3.15));
  EXPECT_ULEQ(0, bear_field_lookup(_, 100));
  new(&_bins_from_indexes, 2);
  bear_field_bin(_, _bins_from_indexes);
  EXPECT_ULEQ(0, bear_field_lookup(_, -1));
  EXPECT_ULEQ(0, bear_field_lookup(_, 3.13));
  EXPECT_ULEQ(0, bear_field_lookup(_, 3.14));
  EXPECT_ULEQ(0, bear_field_lookup(_, 3.15));
  EXPECT_ULEQ(0, bear_field_lookup(_, 100));
  test_save_load(BearField, bear_field, _);
  deletev(&_bins_from_indexes);
  bear_field_delete(&_);

  bear_field_new(&_, true);
  bear_field_add(_, 3.14159, 1, -6.3);
  bear_field_add(_, -1.23456, 2, 0);
  bear_field_add(_, 2409.66, 3, 0.657);
  bear_field_add(_, 0, 4, -2.345);
  bear_field_add(_, -1.23456, 5, 1.23);
  bear_field_add(_, 3.142857, 6, 34.5);
  bear_field_finalize(_);
  EXPECT_ULEQ(4, _->cardinality);
  EXPECT_ULEQ(21, _->totalFrequency);
  EXPECT_PAWEQ(paw_make(-1.23456), _->_values[0]);
  EXPECT_DEQ(-1.23456, _->_meanValues[0]);
  EXPECT_ULEQ(7, _->_frequencies[0]);
  EXPECT_DEQ(1.23 * 5 / 7, _->_meanResiduals[0]);
  EXPECT_PAWEQ(PAW_ZERO, _->_values[1]);
  EXPECT_DEQ(0, _->_meanValues[1]);
  EXPECT_ULEQ(4, _->_frequencies[1]);
  EXPECT_DEQ(-2.345, _->_meanResiduals[1]);
  EXPECT_PAWEQ(paw_make(3.14), _->_values[2]);
  EXPECT_NEAR((3.14159 + 3.142857 * 6) / 7, _->_meanValues[2], 1e-15);
  EXPECT_ULEQ(7, _->_frequencies[2]);
  EXPECT_DEQ((-6.3 + 6 * 34.5) / 7, _->_meanResiduals[2]);
  EXPECT_PAWEQ(paw_make(2409), _->_values[3]);
  EXPECT_ULEQ(3, _->_frequencies[3]);
  EXPECT_DEQ(0.657, _->_meanResiduals[3]);
  EXPECT_ULEQ(0, bear_field_lookup(_, -10));
  EXPECT_ULEQ(0, bear_field_lookup(_, -1.23));
  EXPECT_ULEQ(0, bear_field_lookup(_, -0.62));
  EXPECT_ULEQ(0, bear_field_lookup(_, -0.60));
  EXPECT_ULEQ(1, bear_field_lookup(_, 0));
  EXPECT_ULEQ(0, bear_field_lookup(_, 1.56));
  EXPECT_ULEQ(0, bear_field_lookup(_, 1.58));
  EXPECT_ULEQ(2, bear_field_lookup(_, 3.14));
  EXPECT_ULEQ(0, bear_field_lookup(_, 1204));
  EXPECT_ULEQ(0, bear_field_lookup(_, 1206));
  EXPECT_ULEQ(3, bear_field_lookup(_, 2409));
  EXPECT_ULEQ(0, bear_field_lookup(_, 10000));
  new(&_bins_from_indexes, 4);
  _bins_from_indexes[1] = 1;
  _bins_from_indexes[2] = 2;
  _bins_from_indexes[3] = 3;
  bear_field_bin(_, _bins_from_indexes);
  EXPECT_ULEQ(0, bear_field_lookup(_, -10));
  EXPECT_ULEQ(0, bear_field_lookup(_, -1.23));
  EXPECT_ULEQ(0, bear_field_lookup(_, -0.62));
  EXPECT_ULEQ(1, bear_field_lookup(_, -0.60));
  EXPECT_ULEQ(1, bear_field_lookup(_, 0));
  EXPECT_ULEQ(1, bear_field_lookup(_, 1.56));
  EXPECT_ULEQ(2, bear_field_lookup(_, 1.58));
  EXPECT_ULEQ(2, bear_field_lookup(_, 3.14));
  EXPECT_ULEQ(2, bear_field_lookup(_, 1202));
  EXPECT_ULEQ(3, bear_field_lookup(_, 1206));
  EXPECT_ULEQ(3, bear_field_lookup(_, 2409));
  EXPECT_ULEQ(3, bear_field_lookup(_, 10000));
  test_save_load(BearField, bear_field, _);
  deletev(&_bins_from_indexes);
  bear_field_delete(&_);
}

TEST(BearField, Full) {
  ushort c;
  ushort i;
  ushort v;
  ushort *_bins_from_indexes = null;
  uint u;
  ulong frequency;
  ulong *_lookups = null;
  paw p;
  double d;
  BearField *_ = null;

  bear_field_new(&_, true);
  shuffle_make_lookups(&_lookups, 0xff00);

  for (v = 0, frequency = 1; v < 0xff00; v++) {
    u = _lookups[v];

    if (u == 0x0000 || ((u & 0x7f00) != 0x0000 && (u & 0x7f00) != 0x7f00)) {
      p.u = u;
      d = double_from_paw(p);
      bear_field_add(_, v, frequency++, v);
    }
  }

  deletev(&_lookups);
  shuffle_make_lookups(&_lookups, 0xff00);

  for (v = 0, frequency = 1; v < 0xff00; v++) {
    u = _lookups[v];

    if (u == 0x0000 || ((u & 0x7f00) != 0x0000 && (u & 0x7f00) != 0x7f00)) {
      p.u = u;
      d = double_from_paw(p);
      bear_field_add(_, d, frequency++, d);
    }
  }

  deletev(&_lookups);
  bear_field_finalize(_);
  EXPECT_ULEQ(c = 2 * 126 * 256 + 1, _->cardinality);
  EXPECT_ULEQ((ulong)c * (c + 1), _->totalFrequency);

  for (i = 0; i < c; i++) {
    EXPECT_ULEQ(i, bear_field_lookup(_, double_from_paw(_->_values[i])));
  }

  EXPECT_ULEQ(0, bear_field_lookup(_, DBL_POSINF));
  EXPECT_ULEQ(0, bear_field_lookup(_, DBL_NEGINF));
  EXPECT_ULEQ(0, bear_field_lookup(_, DBL_NAN));

  new(&_bins_from_indexes, c);

  for (i = 0; i < c; i++) {
    _bins_from_indexes[i] = i / 17;
  }

  bear_field_bin(_, _bins_from_indexes);
  EXPECT_ULEQ(
      (2 * 126 * 256 + 1 + 1) / 17,
      bear_field_lookup(_, double_from_paw(PAW_MAX)));
  EXPECT_ULEQ(0, _->cardinality);
  EXPECT_ULEQ(USHRT_MAX, bear_field_lookup(_, DBL_POSINF));
  EXPECT_ULEQ(USHRT_MAX, bear_field_lookup(_, DBL_NEGINF));
  EXPECT_ULEQ(USHRT_MAX, bear_field_lookup(_, DBL_NAN));
  test_save_load(BearField, bear_field, _);
  deletev(&_bins_from_indexes);
  bear_field_delete(&_);
}

FINISH
