#include "BearFieldTestDeath.h"

START_DEATH

TEST(BearField, Add) {
  BearField *_ = null;

  bear_field_new(&_, true);
  EXPECT_DEATH(
      bear_field_add(_, DBL_POSINF, 1, 0),
      "Cannot add the paw-infinite or NaN value inf");
  EXPECT_DEATH(
      bear_field_add(_, DBL_NEGINF, 2, 0),
      "Cannot add the paw-infinite or NaN value -inf");
  EXPECT_DEATH(
      bear_field_add(_, DBL_NAN, 3, 0),
      "Cannot add the paw-infinite or NaN value nan");
  EXPECT_DEATH(
      bear_field_add(_, 1e20, 3, 0),
      "Cannot add the paw-infinite or NaN value 1e+20");
  EXPECT_DEATH(
      bear_field_add(_, -1e20, 3, 0),
      "Cannot add the paw-infinite or NaN value -1e+20");
  EXPECT_DEATH(
      bear_field_add(_, 27, 0, -3.1),
      "Frequency cannot be zero");
  EXPECT_DEATH(
      bear_field_add(_, 0, 1, DBL_POSINF),
      "Cannot add the paw-infinite or NaN residual inf");
  EXPECT_DEATH(
      bear_field_add(_, 0, 1, DBL_NEGINF),
      "Cannot add the paw-infinite or NaN residual -inf");
  EXPECT_DEATH(
      bear_field_add(_, 0, 1, DBL_NAN),
      "Cannot add the paw-infinite or NaN residual nan");
  bear_field_add(_, 0.5, 2, 7.3);
  bear_field_finalize(_);
  EXPECT_DEATH(bear_field_add(_, 1, 2, 4), "Already finalized");
  EXPECT_DEATH(bear_field_finalize(_), "Already finalized");
  bear_field_delete(&_);
}

TEST(BearField, Bin) {
  ushort lookups;
  BearField *_ = null;

  bear_field_new(&_, false);
  EXPECT_DEATH(bear_field_bin(_, null), "Can only bin feature fields");
  bear_field_add(_, -0.5, 3, -0.5);
  EXPECT_DEATH(
      bear_field_add(_, -0.5, 3, -3),
      "Residual (-3) must equal value (-0.5) for a residual field");
  bear_field_finalize(_);
  EXPECT_DEATH(bear_field_bin(_, null), "Can only bin feature fields");
  bear_field_delete(&_);

  bear_field_new(&_, true);
  bear_field_add(_, -0.5, 3, -0.5);
  EXPECT_DEATH(bear_field_bin(_, null), "Not finalized");
  bear_field_finalize(_);
  lookups = 0;
  bear_field_bin(_, &lookups);
  EXPECT_DEATH(bear_field_bin(_, &lookups), "Already binned");
  bear_field_delete(&_);
}

TEST(BearField, Save) {
  BearField *_ = null;

  bear_field_new(&_, false);
  bear_field_add(_, -0.5, 1, -0.5);
  bear_field_finalize(_);
  EXPECT_DEATH(
      test_save_load(BearField, bear_field, _),
      "Can only save when binned");
  bear_field_delete(&_);
}

TEST(BearField, Finalize) {
  BearField *_ = null;

  bear_field_new(&_, true);
  EXPECT_DEATH(bear_field_finalize(_), "No values added");
  bear_field_delete(&_);
}

FINISH_DEATH
