#include "BearHypercubeTest.h"

START

TEST(BearHypercube, Simple) {
  ulong e;
  ulong num_examples;
  ulong num_features;
  ulong *_frequencies = null;
  double *_features = null;
  double **_values = null;
  Decider *_d = null;
  BearHypercube *_ = null;

  /* Test a manifestly trivial hypercube: one feature with one example.
   */
  num_examples = 1;
  num_features = 1;
  newm(&_values, num_examples, num_features + 1);
  new(&_frequencies, num_examples);
  new(&_features, num_features);
  e = 0;
  _values[e][0] = 24.0966;
  _values[e][1] = -7.071968;
  _frequencies[e] = 1;
  decider_new(&_d);
  bear_hypercube_new_memory(
      &_,
      num_features,
      _d,
      num_examples,
      _values,
      _frequencies);
  EXPECT_TRUE(bear_hypercube_is_trivial(_));
  decider_end_iterations(_d);
  decider_delete(&_d);
  deletev(&_features);
  deletev(&_frequencies);
  deletemv(&_values, num_examples);
  bear_hypercube_delete(&_);

  /* Test a manifestly trivial hypercube: one feature with two examples, but
   * the feature value is the same for both.
   */
  num_examples = 2;
  num_features = 1;
  newm(&_values, num_examples, num_features + 1);
  new(&_frequencies, num_examples);
  new(&_features, num_features);
  e = 0;
  _values[e][0] = 24.9;
  _values[e][1] = -7.768;
  _frequencies[e] = 1000;
  e++;
  _values[e][0] = 24.9;
  _values[e][1] = 1968;
  _frequencies[e] = 2000;
  decider_new(&_d);
  bear_hypercube_new_memory(
      &_,
      num_features,
      _d,
      num_examples,
      _values,
      _frequencies);
  EXPECT_TRUE(bear_hypercube_is_trivial(_));
  decider_end_iterations(_d);
  decider_delete(&_d);
  deletev(&_features);
  deletev(&_frequencies);
  deletemv(&_values, num_examples);
  bear_hypercube_delete(&_);

  /* Test a manifestly trivial hypercube: one feature with two examples, but
   * the label value is the same for both.
   */
  num_examples = 2;
  num_features = 1;
  newm(&_values, num_examples, num_features + 1);
  new(&_frequencies, num_examples);
  new(&_features, num_features);
  e = 0;
  _values[e][0] = 24.9;
  _values[e][1] = 1968;
  _frequencies[e] = 1000;
  e++;
  _values[e][0] = 7.7;
  _values[e][1] = 1968;
  _frequencies[e] = 2000;
  decider_new(&_d);
  bear_hypercube_new_memory(
      &_,
      num_features,
      _d,
      num_examples,
      _values,
      _frequencies);
  EXPECT_TRUE(bear_hypercube_is_trivial(_));
  decider_end_iterations(_d);
  decider_delete(&_d);
  deletev(&_features);
  deletev(&_frequencies);
  deletemv(&_values, num_examples);
  bear_hypercube_delete(&_);

  /* Test a trivial hypercube: one feature with two examples, with distinct
   * feature and label values, but insufficient frequency for statistical
   * significance.
   */
  num_examples = 2;
  num_features = 1;
  newm(&_values, num_examples, num_features + 1);
  new(&_frequencies, num_examples);
  new(&_features, num_features);
  e = 0;
  _values[e][0] = 24.9;
  _values[e][1] = 1968;
  _frequencies[e] = 3;
  e++;
  _values[e][0] = 7.7;
  _values[e][1] = -1966;
  _frequencies[e] = 6;
  decider_new(&_d);
  bear_hypercube_new_memory(
      &_,
      num_features,
      _d,
      num_examples,
      _values,
      _frequencies);
  EXPECT_TRUE(bear_hypercube_is_trivial(_));
  decider_end_iterations(_d);
  decider_delete(&_d);
  deletev(&_features);
  deletev(&_frequencies);
  deletemv(&_values, num_examples);
  bear_hypercube_delete(&_);

  /* Do the same, but now with enough frequency for statistical significance.
   */
  num_examples = 2;
  num_features = 1;
  newm(&_values, num_examples, num_features + 1);
  new(&_frequencies, num_examples);
  new(&_features, num_features);
  e = 0;
  _values[e][0] = 24.9;
  _values[e][1] = 1968;
  _frequencies[e] = 1000;
  e++;
  _values[e][0] = 7.7;
  _values[e][1] = -1966;
  _frequencies[e] = 2000;
  decider_new(&_d);
  bear_hypercube_new_memory(
      &_,
      num_features,
      _d,
      num_examples,
      _values,
      _frequencies);
  EXPECT_FALSE(bear_hypercube_is_trivial(_));
  _features[0] = -1000;
  EXPECT_DEQ(-1966, bear_hypercube_predict(_, _features));
  _features[0] = 7.7;
  EXPECT_DEQ(-1966, bear_hypercube_predict(_, _features));
  _features[0] = 16.2;
  EXPECT_DEQ(-1966, bear_hypercube_predict(_, _features));
  _features[0] = 16.4;
  EXPECT_DEQ(1968, bear_hypercube_predict(_, _features));
  _features[0] = 24.9;
  EXPECT_DEQ(1968, bear_hypercube_predict(_, _features));
  _features[0] = 1e10;
  EXPECT_DEQ(1968, bear_hypercube_predict(_, _features));

  decider_end_iterations(_d);
  decider_delete(&_d);
  deletev(&_features);
  deletev(&_frequencies);
  deletemv(&_values, num_examples);
  bear_hypercube_delete(&_);
}

TEST(BearHypercube, Nontrivial) {
  byte iteration;
  byte run;
  ulong e;
  ulong m;
  ulong multiplier;
  ulong num_examples;
  ulong num_features;
  ulong *_frequencies = null;
  double expected_left;
  double expected_right;
  double *_features = null;
  double **_values = null;
  Decider *_d = null;
  BearHypercube *_ = null;

  /* Do 10 examples that populate 8 of 16 cells in a 4 x 4 table of values. The
   * frequencies of the middle two values for both fields are higher, so that
   * the end values are merged into them.
   */
  decider_new(&_d);
  num_examples = 10;
  num_features = 1;
  newm(&_values, num_examples, num_features + 1);
  new(&_frequencies, num_examples);
  new(&_features, num_features);

  for (iteration = 0; iteration < 2; iteration++) {
    e = 0;
    _values[e][0] = 0;
    _values[e][1] = -1;
    _frequencies[e] = 6;
    e++;
    _values[e][0] = 11;
    _values[e][1] = 0;
    _frequencies[e] = 7;
    e++;
    _values[e][0] = -4.5;
    _values[e][1] = -12;
    _frequencies[e] = 1;
    e++;
    _values[e][0] = 24.5;
    _values[e][1] = 0;
    _frequencies[e] = 1;
    e++;
    _values[e][0] = 0;
    _values[e][1] = -1;
    _frequencies[e] = 4;
    e++;
    _values[e][0] = -4.5;
    _values[e][1] = 17;
    _frequencies[e] = 1;
    e++;
    _values[e][0] = 11;
    _values[e][1] = -1;
    _frequencies[e] = 2;
    e++;
    _values[e][0] = 11;
    _values[e][1] = 0;
    _frequencies[e] = 8;
    e++;
    _values[e][0] = 24.5;
    _values[e][1] = -1;
    _frequencies[e] = 1;
    e++;
    _values[e][0] = 0;
    _values[e][1] = 0;
    _frequencies[e] = 1;

    bear_hypercube_new_memory(
        &_,
        num_features,
        _d,
        num_examples,
        _values,
        _frequencies);

    /* Manually compute the predictions. Feature values less than (0 + 11) / 2
     * = 5.5 will map to the left prediction, and those greater to the right
     * prediction.
     */
    expected_left = (-6.0 - 12 - 4 + 17) / 13;
    expected_right = (-2.0 - 1) / 19;

    _features[0] = -1000;
    EXPECT_NEAR(expected_left, bear_hypercube_predict(_, _features), 1e-15);

    _features[0] = -4.5;
    EXPECT_NEAR(expected_left, bear_hypercube_predict(_, _features), 1e-15);

    _features[0] = -2.25;
    EXPECT_NEAR(expected_left, bear_hypercube_predict(_, _features), 1e-15);

    _features[0] = 0;
    EXPECT_NEAR(expected_left, bear_hypercube_predict(_, _features), 1e-15);

    _features[0] = 5.49;
    EXPECT_NEAR(expected_left, bear_hypercube_predict(_, _features), 1e-15);

    _features[0] = 5.51;
    EXPECT_NEAR(expected_right, bear_hypercube_predict(_, _features), 1e-15);

    _features[0] = 11;
    EXPECT_NEAR(expected_right, bear_hypercube_predict(_, _features), 1e-15);

    _features[0] = (11 + 24.5) / 2;
    EXPECT_NEAR(expected_right, bear_hypercube_predict(_, _features), 1e-15);

    _features[0] = 24.5;
    EXPECT_NEAR(expected_right, bear_hypercube_predict(_, _features), 1e-15);

    _features[0] = 1000;
    EXPECT_NEAR(expected_right, bear_hypercube_predict(_, _features), 1e-15);

    test_save_load(BearHypercube, bear_hypercube, _);
    save_load(bear_hypercube, _);

    _features[0] = -1000;
    EXPECT_NEAR(expected_left, bear_hypercube_predict(_, _features), 1e-15);

    _features[0] = -4.5;
    EXPECT_NEAR(expected_left, bear_hypercube_predict(_, _features), 1e-15);

    _features[0] = -2.25;
    EXPECT_NEAR(expected_left, bear_hypercube_predict(_, _features), 1e-15);

    _features[0] = 0;
    EXPECT_NEAR(expected_left, bear_hypercube_predict(_, _features), 1e-15);

    _features[0] = 5.49;
    EXPECT_NEAR(expected_left, bear_hypercube_predict(_, _features), 1e-15);

    _features[0] = 5.51;
    EXPECT_NEAR(expected_right, bear_hypercube_predict(_, _features), 1e-15);

    _features[0] = 11;
    EXPECT_NEAR(expected_right, bear_hypercube_predict(_, _features), 1e-15);

    _features[0] = (11 + 24.5) / 2;
    EXPECT_NEAR(expected_right, bear_hypercube_predict(_, _features), 1e-15);

    _features[0] = 24.5;
    EXPECT_NEAR(expected_right, bear_hypercube_predict(_, _features), 1e-15);

    _features[0] = 1000;
    EXPECT_NEAR(expected_right, bear_hypercube_predict(_, _features), 1e-15);

    bear_hypercube_delete(&_);
    decider_end_iteration(_d);
  }

  decider_delete(&_d);

  deletev(&_features);
  deletev(&_frequencies);
  deletemv(&_values, num_examples);

  /* Now do a simple test of two features. We only populate the (0, 0, 0) and
   * (1, 1, 1) cells in the cube. This also tests that the predictions for
   * feature bins with no data in them default to the empty-model label
   * predictions.
   *
   * We do two overall runs. On the first there are only two examples. On the
   * second there are two million, to force the parallel methods.
   */
  for (run = 0; run < 2; run++) {
    multiplier = run == 0 ? 1 : 1000000;
    num_features = 2;
    num_examples = 2 * multiplier;
    newm(&_values, num_examples, num_features + 1);
    new(&_frequencies, num_examples);
    new(&_features, num_features);

    for (m = 0, e = 0; m < multiplier; m++) {
      _values[e][0] = 7;
      _values[e][1] = 9;
      _values[e][2] = 16;
      _frequencies[e] = 1000;
      e++;
      _values[e][0] = 1966;
      _values[e][1] = 1968;
      _values[e][2] = 24;
      _frequencies[e] = 3000;
      e++;
    }

    if (e != num_examples) {
      throw("Impossible");
    }

    decider_new(&_d);

    for (iteration = 0; iteration < 2; iteration++) {
      bear_hypercube_new_memory(
          &_,
          num_features,
          _d,
          num_examples,
          _values,
          _frequencies);

      _features[0] = -1e12;
      _features[1] = -2e14;
      EXPECT_NEAR(16, bear_hypercube_predict(_, _features), 1e-16);

      _features[0] = 7;
      _features[1] = 9;
      EXPECT_NEAR(16, bear_hypercube_predict(_, _features), 1e-16);

      _features[0] = 100;
      _features[1] = 1400;
      EXPECT_NEAR(22, bear_hypercube_predict(_, _features), 1e-16);

      _features[0] = 1900;
      _features[1] = -83.4;
      EXPECT_NEAR(22, bear_hypercube_predict(_, _features), 1e-16);

      _features[0] = 1.23e14;
      _features[1] = 5e16;
      EXPECT_NEAR(24, bear_hypercube_predict(_, _features), 1e-16);

      test_save_load(BearHypercube, bear_hypercube, _);
      save_load(bear_hypercube, _);

      _features[0] = -1e12;
      _features[1] = -2e14;
      EXPECT_NEAR(16, bear_hypercube_predict(_, _features), 1e-16);

      _features[0] = 7;
      _features[1] = 9;
      EXPECT_NEAR(16, bear_hypercube_predict(_, _features), 1e-16);

      _features[0] = 100;
      _features[1] = 1400;
      EXPECT_NEAR(22, bear_hypercube_predict(_, _features), 1e-16);

      _features[0] = 1900;
      _features[1] = -83.4;
      EXPECT_NEAR(22, bear_hypercube_predict(_, _features), 1e-16);

      _features[0] = 1.23e14;
      _features[1] = 5e16;
      EXPECT_NEAR(24, bear_hypercube_predict(_, _features), 1e-16);

      bear_hypercube_delete(&_);
      decider_end_iteration(_d);
    }

    decider_delete(&_d);

    deletev(&_features);
    deletev(&_frequencies);
    deletemv(&_values, num_examples);
  }
}

FINISH
