#include "BearHypercubeTestDeath.h"

START_DEATH

TEST(BearHypercube, New) {
  Decider *_d = null;
  BearHypercube *_ = null;

  decider_new(&_d);
  EXPECT_DEATH(bear_hypercube_new(&_, 0, _d),
      "Must have at least one feature");
  bear_hypercube_delete(&_);
  decider_end_iterations(_d);
  decider_delete(&_d);
}

TEST(BearHypercube, Add) {
  double *_values = null;
  Decider *_d = null;
  BearHypercube *_ = null;

  decider_new(&_d);
  bear_hypercube_new(&_, 1, _d);
  new(&_values, 2);
  EXPECT_DEATH(bear_hypercube_add(_, _values, 0), "Cannot add zero frequency");
  deletev(&_values);
  bear_hypercube_delete(&_);
  decider_end_iterations(_d);
  decider_delete(&_d);

  decider_new(&_d);
  bear_hypercube_new(&_, 1, _d),
  new(&_values, 2);
  bear_hypercube_add(_, _values, 1);
  _values[0] = _values[1] = 1;
  bear_hypercube_add(_, _values, 1);
  bear_hypercube_end_iteration(_);
  bear_hypercube_add(_, _values, 1);
  _values[0] = _values[1] = 0;
  bear_hypercube_add(_, _values, 1);
  bear_hypercube_end_iteration(_);
  EXPECT_DEATH(
      bear_hypercube_add(_, _values, 1),
      "Hypercube was already determined to be trivial");
  deletev(&_values);
  bear_hypercube_delete(&_);
  decider_end_iterations(_d);
  decider_delete(&_d);

  decider_new(&_d);
  bear_hypercube_new(&_, 1, _d),
  new(&_values, 2);
  bear_hypercube_add(_, _values, 10);
  _values[0] = _values[1] = 1;
  bear_hypercube_add(_, _values, 10);
  bear_hypercube_end_iteration(_);
  bear_hypercube_add(_, _values, 10);
  _values[0] = _values[1] = 0;
  bear_hypercube_add(_, _values, 10);
  bear_hypercube_end_iteration(_);
  bear_hypercube_add(_, _values, 10);
  _values[0] = _values[1] = 1;
  bear_hypercube_add(_, _values, 10);
  bear_hypercube_end_iteration(_);
  EXPECT_DEATH(
      bear_hypercube_end_iteration(_),
      "Have already ended the third iteration");
  deletev(&_values);
  bear_hypercube_delete(&_);
  decider_end_iterations(_d);
  decider_delete(&_d);
}

TEST(BearHypercube, EndIteration) {
  double *_values = null;
  Decider *_d = null;
  BearHypercube *_ = null;

  decider_new(&_d);
  bear_hypercube_new(&_, 1, _d),
  new(&_values, 2);
  bear_hypercube_add(_, _values, 1);
  _values[0] = _values[1] = 1;
  bear_hypercube_add(_, _values, 1);
  bear_hypercube_end_iteration(_);
  EXPECT_DEATH(
      bear_hypercube_end_iteration(_),
      "A different total frequency was added on the second iteration (0) "
      "than on the first (2)");
  deletev(&_values);
  bear_hypercube_delete(&_);
  decider_end_iterations(_d);
  decider_delete(&_d);

  decider_new(&_d);
  bear_hypercube_new(&_, 1, _d),
  new(&_values, 2);
  bear_hypercube_add(_, _values, 10);
  _values[0] = _values[1] = 1;
  bear_hypercube_add(_, _values, 10);
  bear_hypercube_end_iteration(_);
  bear_hypercube_add(_, _values, 10);
  _values[0] = _values[1] = 0;
  bear_hypercube_add(_, _values, 10);
  bear_hypercube_end_iteration(_);
  bear_hypercube_add(_, _values, 17);
  EXPECT_DEATH(
      bear_hypercube_end_iteration(_),
      "A different total frequency was added on the third iteration (17) "
      "than on the first two (20)");
  deletev(&_values);
  bear_hypercube_delete(&_);
  decider_end_iterations(_d);
  decider_delete(&_d);

  decider_new(&_d);
  bear_hypercube_new(&_, 1, _d),
  new(&_values, 2);
  bear_hypercube_add(_, _values, 1);
  bear_hypercube_end_iteration(_);
  EXPECT_DEATH(
      bear_hypercube_end_iteration(_),
      "Hypercube was already determined to be trivial");
  deletev(&_values);
  bear_hypercube_delete(&_);
  decider_end_iterations(_d);
  decider_delete(&_d);

  decider_new(&_d);
  bear_hypercube_new(&_, 1, _d),
  new(&_values, 2);
  bear_hypercube_add(_, _values, 1);
  _values[0] = _values[1] = 1;
  bear_hypercube_add(_, _values, 1);
  bear_hypercube_end_iteration(_);
  bear_hypercube_add(_, _values, 1);
  _values[0] = _values[1] = 0;
  bear_hypercube_add(_, _values, 1);
  bear_hypercube_end_iteration(_);
  EXPECT_DEATH(
      bear_hypercube_end_iteration(_),
      "Hypercube was already determined to be trivial");
  deletev(&_values);
  bear_hypercube_delete(&_);
  decider_end_iterations(_d);
  decider_delete(&_d);

  decider_new(&_d);
  bear_hypercube_new(&_, 1, _d),
  new(&_values, 2);
  bear_hypercube_add(_, _values, 10);
  _values[0] = _values[1] = 1;
  bear_hypercube_add(_, _values, 10);
  bear_hypercube_end_iteration(_);
  bear_hypercube_add(_, _values, 10);
  _values[0] = _values[1] = 0;
  bear_hypercube_add(_, _values, 10);
  bear_hypercube_end_iteration(_);
  bear_hypercube_add(_, _values, 10);
  _values[0] = _values[1] = 1;
  bear_hypercube_add(_, _values, 10);
  bear_hypercube_end_iteration(_);
  EXPECT_DEATH(
      bear_hypercube_end_iteration(_),
      "Have already ended the third iteration");
  deletev(&_values);
  bear_hypercube_delete(&_);
  decider_end_iterations(_d);
  decider_delete(&_d);
}

FINISH_DEATH
