#include "BearPartTest.h"

START

TEST(BearPart, NewElementary) {
  BearPart *_ = null;

  bear_part_new_elementary(&_, 27);
  EXPECT_ULEQ(1, _->numFeatures);
  EXPECT_ULEQ(27, _->_features[0]);
  bear_part_delete(&_);
}

TEST(BearPart, NewCopy) {
  BearPart *_1 = null;
  BearPart *_2 = null;
  BearPart *_3 = null;

  bear_part_new_elementary(&_1, 24);
  bear_part_new_copy(&_2, _1);
  bear_part_delete(&_1);
  EXPECT_ULEQ(1, _2->numFeatures);
  EXPECT_ULEQ(24, _2->_features[0]);

  bear_part_new_elementary(&_1, 9);
  bear_part_new_melt(&_3, _1, _2);
  bear_part_delete(&_2);
  bear_part_delete(&_1);
  bear_part_new_copy(&_1, _3);
  bear_part_delete(&_3);
  EXPECT_ULEQ(2, _1->numFeatures);
  EXPECT_ULEQ(9, _1->_features[0]);
  EXPECT_ULEQ(24, _1->_features[1]);
  test_save_load(BearPart, bear_part, _1);
  bear_part_delete(&_1);
}

TEST(BearPart, NewMelt) {
  BearPart *_1 = null;
  BearPart *_2 = null;
  BearPart *_3 = null;
  BearPart *_4 = null;

  bear_part_new_elementary(&_1, 24);
  bear_part_new_elementary(&_2, 9);
  bear_part_new_melt(&_3, _1, _2);
  bear_part_delete(&_2);
  bear_part_delete(&_1);
  EXPECT_ULEQ(2, _3->numFeatures);
  EXPECT_ULEQ(9, _3->_features[0]);
  EXPECT_ULEQ(24, _3->_features[1]);

  bear_part_new_elementary(&_1, 1966);
  bear_part_new_melt(&_4, _1, _3);
  EXPECT_ULEQ(3, _4->numFeatures);
  EXPECT_ULEQ(9, _4->_features[0]);
  EXPECT_ULEQ(24, _4->_features[1]);
  EXPECT_ULEQ(1966, _4->_features[2]);
  test_save_load(BearPart, bear_part, _4);
  bear_part_delete(&_4);

  bear_part_new_elementary(&_2, 0);
  bear_part_new_melt(&_4, _1, _2);
  bear_part_delete(&_1);
  bear_part_delete(&_2);
  bear_part_new_melt(&_1, _3, _4);
  bear_part_delete(&_3);
  bear_part_delete(&_4);
  EXPECT_ULEQ(4, _1->numFeatures);
  EXPECT_ULEQ(0, _1->_features[0]);
  EXPECT_ULEQ(9, _1->_features[1]);
  EXPECT_ULEQ(24, _1->_features[2]);
  EXPECT_ULEQ(1966, _1->_features[3]);
  test_save_load(BearPart, bear_part, _1);
  bear_part_delete(&_1);
}

TEST(BearPart, CanMelt) {
  BearPart *_1 = null;
  BearPart *_2 = null;
  BearPart *_3 = null;
  BearPart *_4 = null;

  bear_part_new_elementary(&_1, 24);
  bear_part_new_elementary(&_2, 9);
  EXPECT_TRUE(bear_part_can_melt(_1, _2));
  EXPECT_TRUE(bear_part_can_melt(_2, _1));
  bear_part_delete(&_2);
  bear_part_new_elementary(&_2, 24);
  EXPECT_FALSE(bear_part_can_melt(_1, _2));
  EXPECT_FALSE(bear_part_can_melt(_2, _1));
  bear_part_delete(&_2);
  bear_part_delete(&_1);

  bear_part_new_elementary(&_1, 24);
  bear_part_new_elementary(&_2, 9);
  bear_part_new_melt(&_3, _1, _2);
  EXPECT_FALSE(bear_part_can_melt(_1, _3));
  EXPECT_FALSE(bear_part_can_melt(_3, _1));
  EXPECT_FALSE(bear_part_can_melt(_2, _3));
  EXPECT_FALSE(bear_part_can_melt(_3, _2));
  bear_part_delete(&_2);
  bear_part_delete(&_1);
  bear_part_new_elementary(&_1, 7);
  bear_part_new_elementary(&_2, 1968);
  bear_part_new_melt(&_4, _1, _2);
  bear_part_delete(&_2);
  bear_part_delete(&_1);
  EXPECT_TRUE(bear_part_can_melt(_3, _4));
  EXPECT_TRUE(bear_part_can_melt(_4, _3));
  bear_part_delete(&_4);
  bear_part_delete(&_3);
}

TEST(BearPart, Equals) {
  BearPart *_1 = null;
  BearPart *_2 = null;
  BearPart *_3 = null;
  BearPart *_4 = null;
  BearPart *_5 = null;

  bear_part_new_elementary(&_1, 24);
  EXPECT_TRUE(bear_part_equals(_1, _1));
  bear_part_new_elementary(&_2, 66);
  EXPECT_FALSE(bear_part_equals(_1, _2));
  EXPECT_FALSE(bear_part_equals(_2, _1));
  bear_part_delete(&_2);
  bear_part_new_elementary(&_2, 9);
  bear_part_new_elementary(&_3, 99);
  bear_part_new_melt(&_4, _2, _3);
  EXPECT_TRUE(bear_part_equals(_4, _4));
  EXPECT_FALSE(bear_part_equals(_1, _4));
  EXPECT_FALSE(bear_part_equals(_4, _1));
  bear_part_new_melt(&_5, _1, _3);
  EXPECT_FALSE(bear_part_equals(_5, _4));
  EXPECT_FALSE(bear_part_equals(_4, _5));
  bear_part_delete(&_5);
  bear_part_new_melt(&_5, _1, _2);
  EXPECT_FALSE(bear_part_equals(_5, _4));
  EXPECT_FALSE(bear_part_equals(_4, _5));
  bear_part_delete(&_5);
  bear_part_delete(&_4);
  bear_part_delete(&_3);
  bear_part_delete(&_2);
  bear_part_delete(&_1);
}

TEST(BearPart, Compare) {
  BearPart *_1 = null;
  BearPart *_2 = null;
  BearPart *_3 = null;
  BearPart *_4 = null;
  BearPart *_5 = null;

  bear_part_new_elementary(&_1, 24);
  EXPECT_LEQ(0, bear_part_compare(_1, _1, null));
  bear_part_new_elementary(&_2, 66);
  EXPECT_LEQ(-1, bear_part_compare(_1, _2, null));
  EXPECT_LEQ(+1, bear_part_compare(_2, _1, null));
  bear_part_delete(&_2);
  bear_part_new_elementary(&_2, 9);
  bear_part_new_elementary(&_3, 99);
  bear_part_new_melt(&_4, _2, _3);
  EXPECT_LEQ(-1, bear_part_compare(_1, _4, null));
  EXPECT_LEQ(+1, bear_part_compare(_4, _1, null));
  bear_part_new_melt(&_5, _1, _3);
  EXPECT_LEQ(+1, bear_part_compare(_5, _4, null));
  EXPECT_LEQ(-1, bear_part_compare(_4, _5, null));
  bear_part_delete(&_5);
  bear_part_new_melt(&_5, _1, _2);
  EXPECT_LEQ(-1, bear_part_compare(_5, _4, null));
  EXPECT_LEQ(+1, bear_part_compare(_4, _5, null));
  bear_part_delete(&_5);
  bear_part_delete(&_4);
  bear_part_delete(&_3);
  bear_part_delete(&_2);
  bear_part_delete(&_1);
}

FINISH
