#include "DictionaryTest.h"

static void new_(Dictionary **p_this);

static void donate_(Dictionary *d, char *key, char *value);

static void print_(void *this, Printer *p);


START

TEST(Dictionary, Comprehensive) {
  Dictionary *_d = null;
  DictionaryPairs *_pairs = null;

  /* Empty dictionary.
   */
  new_(&_d);
  EXPECT_FALSE(dictionary_is_finalized(_d));

  EXPECT_FALSE(dictionary_is_finalized(_d));
  dictionary_finalize(_d);
  EXPECT_TRUE(dictionary_is_finalized(_d));
  EXPECT_ULEQ(0, dictionary_cardinality(_d));
  EXPECT_NULL(dictionary_find(_d, "dog"));

  dictionary_pairs_new(&_pairs, _d, false);
  dictionary_pairs_delete(&_pairs);

  dictionary_pairs_new(&_pairs, _d, true);
  dictionary_pairs_delete(&_pairs);

  dictionary_delete(&_d);

  /* One pair.
   */
  new_(&_d);
  donate_(_d, "cat", "miaow");
  EXPECT_FALSE(dictionary_is_finalized(_d));

  EXPECT_FALSE(dictionary_is_finalized(_d));
  dictionary_finalize(_d);
  EXPECT_TRUE(dictionary_is_finalized(_d));
  EXPECT_ULEQ(1, dictionary_cardinality(_d));
  EXPECT_NULL(dictionary_find(_d, "dog"));
  EXPECT_STREQ("miaow", dictionary_find(_d, "cat"));

  dictionary_pairs_new(&_pairs, _d, false);
  dictionary_pairs_delete(&_pairs);

  dictionary_pairs_new(&_pairs, _d, true);
  dictionary_pairs_delete(&_pairs);

  dictionary_delete(&_d);

  /* Repeated key.
   */
  new_(&_d);
  donate_(_d, "cat", "miaow");
  donate_(_d, "cat", "mew");

  EXPECT_FALSE(dictionary_is_finalized(_d));
  dictionary_finalize(_d);
  EXPECT_TRUE(dictionary_is_finalized(_d));
  EXPECT_ULEQ(1, dictionary_cardinality(_d));
  EXPECT_NULL(dictionary_find(_d, "dog"));
  EXPECT_STREQ("mew", dictionary_find(_d, "cat"));

  dictionary_pairs_new(&_pairs, _d, false);
  dictionary_pairs_delete(&_pairs);

  dictionary_pairs_new(&_pairs, _d, true);
  dictionary_pairs_delete(&_pairs);

  dictionary_delete(&_d);

  /* Two different keys, one with a repeated key, ultimately added in reverse
   * key order.
   */
  new_(&_d);
  donate_(_d, "cat", "miaow");
  donate_(_d, "dog", "woof");
  donate_(_d, "cat", "mew");
  EXPECT_FALSE(dictionary_is_finalized(_d));
  dictionary_finalize(_d);
  EXPECT_TRUE(dictionary_is_finalized(_d));
  EXPECT_ULEQ(2, dictionary_cardinality(_d));
  EXPECT_NULL(dictionary_find(_d, "pig"));
  EXPECT_STREQ("mew", dictionary_find(_d, "cat"));
  EXPECT_STREQ("woof", dictionary_find(_d, "dog"));

  dictionary_pairs_new(&_pairs, _d, false);
  dictionary_pairs_delete(&_pairs);

  dictionary_pairs_new(&_pairs, _d, true);
  dictionary_pairs_delete(&_pairs);

  EXPECT_TRUE(dictionary_is_finalized(_d));
  dictionary_delete(&_d);

  /* One pair, empty key.
   */
  new_(&_d);
  donate_(_d, "", "miaow");
  EXPECT_FALSE(dictionary_is_finalized(_d));
  dictionary_finalize(_d);
  EXPECT_TRUE(dictionary_is_finalized(_d));
  EXPECT_ULEQ(1, dictionary_cardinality(_d));
  EXPECT_NULL(dictionary_find(_d, "pig"));
  EXPECT_STREQ("miaow", dictionary_find(_d, ""));

  dictionary_pairs_new(&_pairs, _d, false);
  dictionary_pairs_delete(&_pairs);

  dictionary_pairs_new(&_pairs, _d, true);
  dictionary_pairs_delete(&_pairs);

  EXPECT_TRUE(dictionary_is_finalized(_d));

  /* Test reopen.
   */
  dictionary_reopen(_d);
  donate_(_d, "dog", "woof");
  dictionary_finalize(_d);
  dictionary_pairs_new(&_pairs, _d, true);
  dictionary_pairs_delete(&_pairs);

  EXPECT_TRUE(dictionary_is_finalized(_d));
  dictionary_delete(&_d);
}

FINISH


static void new_(Dictionary **p_this) {
  dictionary_new(p_this, string_delete, print_);
}

static void donate_(Dictionary *d, char *key, char *value) {
  char *_key = null;
  char *_value = null;

  string_new_copy(&_key, key);
  string_new_copy(&_value, value);
  dictionary_donate(d, &_key, &_value);
}

static void print_(void *this, Printer *p) {
  char *s;

  s = this;
  rprint(p, "'%s'", s);
}
