#include "DynamicSetTest.h"

#define K_ 300
#define N_ 65536

static void short_delete_(short **p_s);

static void short_print_(short *s, Printer *p);

static void ushort_delete_(ushort **p_s);

static void ushort_print_(ushort *s, Printer *p);

static void callback_(short *s, short *sum);


START

TEST(DynamicSet, Simple) {
  short s;
  short *p;
  short *_s = null;
  short **_a = null;
  DynamicSet *_ = null;

  dynamic_set_new(&_, short_delete_, short_compare, short_print_);
  EXPECT_TRUE(dynamic_set_is_empty(_));

  newo(&_s);
  *_s = s = 24;
  EXPECT_FALSE(dynamic_set_contains(_, _s));
  EXPECT_NULL(dynamic_set_find(_, &s));
  dynamic_set_insert(_, &_s);
  EXPECT_FALSE(dynamic_set_is_empty(_));
  EXPECT_NULL(_s);
  EXPECT_TRUE(dynamic_set_contains(_, &s));
  p = dynamic_set_find(_, &s);
  EXPECT_ULEQ(s, *p);
  dynamic_set_extract(_, &s, &_s);
  EXPECT_LEQ(s, *_s);
  EXPECT_TRUE(dynamic_set_is_empty(_));
  dynamic_set_insert(_, &_s);
  EXPECT_FALSE(dynamic_set_is_empty(_));
  EXPECT_NULL(_s);
  EXPECT_TRUE(dynamic_set_contains(_, &s));
  p = dynamic_set_find(_, &s);
  EXPECT_ULEQ(s, *p);
  dynamic_set_remove(_, &s);
  EXPECT_TRUE(dynamic_set_is_empty(_));
  newo(&_s);
  *_s = s = 24;
  dynamic_set_insert(_, &_s);

  newo(&_s);
  *_s = s = 9;
  EXPECT_FALSE(dynamic_set_contains(_, _s));
  dynamic_set_insert(_, &_s);
  EXPECT_NULL(_s);
  EXPECT_TRUE(dynamic_set_contains(_, &s));
  p = dynamic_set_find(_, &s);

  newo(&_s);
  *_s = s = 1966;
  EXPECT_FALSE(dynamic_set_contains(_, _s));
  EXPECT_NULL(dynamic_set_find(_, &s));
  dynamic_set_insert(_, &_s);
  EXPECT_NULL(_s);
  EXPECT_TRUE(dynamic_set_contains(_, &s));
  p = dynamic_set_find(_, &s);
  EXPECT_ULEQ(1966, *p);

  s = -24;
  EXPECT_FALSE(dynamic_set_contains(_, &s));
  EXPECT_NULL(dynamic_set_find(_, &s));

  s = 9;
  dynamic_set_remove(_, &s);
  EXPECT_FALSE(dynamic_set_contains(_, &s));

  s = 1966;
  dynamic_set_remove(_, &s);
  EXPECT_FALSE(dynamic_set_contains(_, &s));

  s = 24;
  dynamic_set_remove(_, &s);
  EXPECT_FALSE(dynamic_set_contains(_, &s));

  EXPECT_TRUE(dynamic_set_is_empty(_));

  newo(&_s);
  *_s = 1966;
  dynamic_set_insert(_, &_s);

  newo(&_s);
  *_s = -9;
  dynamic_set_insert(_, &_s);

  newo(&_s);
  *_s = 24;
  dynamic_set_insert(_, &_s);

  EXPECT_ULEQ(3, dynamic_set_extract_all(_, &_a));
  EXPECT_LEQ(-9, *_a[0]);
  EXPECT_LEQ(24, *_a[1]);
  EXPECT_LEQ(1966, *_a[2]);

  delete_(&_a, 3, short);
  dynamic_set_delete(&_);
}

TEST(DynamicSet, ForEach) {
  short sum;
  short *_s = null;
  DynamicSet *_ = null;

  dynamic_set_new(&_, short_delete_, short_compare, short_print_);
  newo(&_s);
  *_s = 24;
  dynamic_set_insert(_, &_s);
  newo(&_s);
  *_s = 9;
  dynamic_set_insert(_, &_s);
  newo(&_s);
  *_s = -1966;
  dynamic_set_insert(_, &_s);
  sum = 0;
  dynamic_set_foreach(_, callback_, &sum);
  EXPECT_LEQ(-1933, sum);

  dynamic_set_delete(&_);
}

TEST(DynamicSet, Comprehensive) {
  bool *_added = null;
  ushort u;
  ushort *_u = null;
  ulong i;
  uint t;
  Random *_random = null;
  DynamicSet *_ = null;

  dynamic_set_new(&_, ushort_delete_, ushort_compare, ushort_print_);
  new(&_added, N_);
  random_new(&_random);

  for (i = 0; i < K_; i++) {
    while (_added[u = random_modulo(_random, N_)]) {
    }

    _added[u] = true;
    newo(&_u);
    *_u = u;
    dynamic_set_insert(_, &_u);

    for (t = 0; t < N_; t++) {
      EXPECT_TRUE(dynamic_set_contains(_, &t) == _added[t]);
    }
  }

  random_delete(&_random);
  deletev(&_added);
  dynamic_set_delete(&_);
}

FINISH


static void short_delete_(short **p_s) {
  deleteo(p_s);
}

static void short_print_(short *s, Printer *p) {
  rprint(p, "%d", *s);
}

static void ushort_delete_(ushort **p_s) {
  deleteo(p_s);
}

static void ushort_print_(ushort *u, Printer *p) {
  rprint(p, "%u", *u);
}

static void callback_(short *s, short *sum) {
  *sum += *s;
}
