#include "HeapTest.h"
#include <limits.h>
#include <math.h>

#define N_ 3000
#define P_ 1000
#define Q_ 100

static void ulong_print_(HeapObjectPrintInfo *hopi, Printer *p);

static void check_(Heap *h, ulong *data, ulong n, char *mode);


START

TEST(Heap, Comprehensive) {
  Random *_random = null;
  ulong i;
  ulong j;
  ulong max;
  ulong n_add;
  ulong n_insert;
  ulong n_total;
  ulong u;
  ulong *_data = null;
  Heap *_h = null;

  random_new(&_random);
  max = ULONG_MAX;

  for (i = 0; i < N_; i++) {
    heap_new(&_h, &max, ulong_compare, ulong_print_, null);
    heap_delete(&_h);
    heap_new(&_h, &max, ulong_compare, ulong_print_, null);

    n_add = (ulong)floor(exp(log(P_) * random_double_unit_cc(_random)) - 1);
    n_insert = (ulong)floor(exp(log(P_) * random_double_unit_cc(_random)) - 1);
    n_total = n_add + n_insert;

    new(&_data, n_total);

    for (j = 0; j < n_add; j++) {
      u = random_modulo(_random, Q_);
      _data[j] = u;
      heap_add(_h, &u);
    }

    heap_build(_h);
    qsortx(_data, n_add, ulong_compare, false, null);
    check_(_h, _data, n_add, "add");

    for (j = 0; j < n_insert; j++) {
      u = random_modulo(_random, Q_);
      _data[j] = u;
      heap_insert(_h, &u);
    }

    qsortx(_data, n_insert, ulong_compare, false, null);
    check_(_h, _data, n_insert, "insert");

    for (j = 0; j < n_add; j++) {
      ulong u = random_modulo(_random, Q_);
      _data[j] = u;
      heap_add(_h, &u);
    }

    heap_build(_h);

    for (j = 0; j < n_insert; j++) {
      u = random_modulo(_random, Q_);
      _data[n_add + j] = u;
      heap_insert(_h, &u);
    }

    qsortx(_data, n_total, ulong_compare, false, 0);
    check_(_h, _data, n_total, "add + insert");

    deletev(&_data);
    heap_delete(&_h);
  }

  random_delete(&_random);
}

TEST(Heap, Print) {
  char *_s = null;
  ulong max;
  ulong u;
  Heap *_ = null;

  max = ULONG_MAX;

  heap_new(&_, &max, ulong_compare, ulong_print_, null);
  aprint(&_s, 0, heap, _);
  EXPECT_STREQ(
      "[Heap]\n. Element size: 8\n. Maximum number of elements: 1\n"
      ". Current number of elements: 0\n. Heapified: true\n[/Heap]\n",
      _s);
  string_delete(&_s);
  heap_delete(&_);

  heap_new(&_, &max, ulong_compare, ulong_print_, null);
  u = 24;
  heap_add(_, &u);
  u = 9;
  heap_add(_, &u);
  u = 1966;
  heap_add(_, &u);
  aprint(&_s, 0, heap, _);
  EXPECT_STREQ(
      "[Heap]\n. Element size: 8\n. Maximum number of elements: 4\n"
      ". Current number of elements: 3\n. Heapified: false\n. [Elements]\n"
      ". . [Index 0]\n. . . 24\n. . . [Index 1]\n. . . . 9\n. . . [/Index 1]\n"
      ". . . [Index 2]\n. . . . 1966\n. . . [/Index 2]\n. . [/Index 0]\n"
      ". [/Elements]\n[/Heap]\n",
      _s);
  string_delete(&_s);

  heap_build(_);
  aprint(&_s, 0, heap, _);
  EXPECT_STREQ(
      "[Heap]\n. Element size: 8\n. Maximum number of elements: 4\n"
      ". Current number of elements: 3\n. Heapified: true\n. [Elements]\n"
      ". . [Index 0]\n. . . 9\n. . . [Index 1]\n. . . . 24\n. . . [/Index 1]\n"
      ". . . [Index 2]\n. . . . 1966\n. . . [/Index 2]\n. . [/Index 0]\n"
      ". [/Elements]\n[/Heap]\n",
      _s);
  string_delete(&_s);

  heap_delete(&_);
}

FINISH


static void ulong_print_(HeapObjectPrintInfo *hopi, Printer *p) {
  rprint(p, "%lu", *(ulong *)hopi->this);
}

static void check_(Heap *h, ulong *data, ulong n, char *mode) {
  ulong i;
  ulong expected;
  ulong got;
  ulong extracted;

  for (i = 0; i < n; i++) {
    expected = data[i];

    got = *(ulong *)heap_get_min(h);

    if (expected != got) {
      throw("Expected %lu but got %lu (%s):\n", expected, got, mode);
    }

    heap_extract_min(h, &extracted);

    if (expected != extracted) {
      throw("Expected %lu but extracted %lu:\n", expected, extracted);
    }
  }
}
