#include "HypercubeTestDeath.h"

static void callback_(Hypercube *this,
    bool *walk_dimensions,
    ulong *indexes,
    ulong o,
    void *x);


START_DEATH

TEST(Hypercube, ZeroDimensional) {
  double d;
  Hypercube *_ = null;

  d = -24.0966;
  hypercube_new(&_, ulong, 0, null);
  EXPECT_DEATH(
      hypercube_set(_, 1, &d),
      "Overall element index 1 is invalid (max 0)");
  EXPECT_DEATH(
      hypercube_get(_, 1, &d),
      "Overall element index 1 is invalid (max 0)");
  hypercube_delete(&_);
}

TEST(Hypercube, New) {
  ulong _dimensions[2];
  Hypercube *_ = null;

  _dimensions[0] = 0;
  EXPECT_DEATH(
      hypercube_new(&_, paw, 1, _dimensions),
      "Dimension 0 is zero, which is not allowed");
  hypercube_delete(&_);

  _dimensions[1] = 1;
  EXPECT_DEATH(
      hypercube_new(&_, double, 2, _dimensions),
      "Dimension 0 is zero, which is not allowed");
  hypercube_delete(&_);

  _dimensions[0] = 1;
  _dimensions[1] = 0;
  EXPECT_DEATH(
      hypercube_new(&_, ulong, 2, _dimensions),
      "Dimension 1 is zero, which is not allowed");
  hypercube_delete(&_);
}

TEST(Hypercube, IFromO) {
  ulong _dimensions[] = {2, 3};
  Hypercube *_ = null;

  hypercube_new(&_, paw, 1, _dimensions),
  EXPECT_DEATH(
      hypercube_i_from_o(_, 1, 1),
      "Dimension index 1 is invalid (max 0)");
  EXPECT_DEATH(
      hypercube_i_from_o(_, 0, 2),
      "Overall element index 2 is invalid (max 1)");
  hypercube_delete(&_);

  hypercube_new(&_, double, 2, _dimensions),
  EXPECT_DEATH(
      hypercube_i_from_o(_, 2, 1),
      "Dimension index 2 is invalid (max 1)");
  EXPECT_DEATH(
      hypercube_i_from_o(_, 1, 6),
      "Overall element index 6 is invalid (max 5)");
  hypercube_delete(&_);
}

TEST(Hypercube, SetGet) {
  ulong u;
  paw p;
  ulong _dimensions[] = {2, 3};
  ulong _indexes[2];
  Hypercube *_ = null;

  u = 24091966;
  hypercube_new(&_, ulong, 1, _dimensions),
  EXPECT_DEATH(
      hypercube_set(_, 2, &u),
      "Overall element index 2 is invalid (max 1)");
  _indexes[0] = 2;
  EXPECT_DEATH(
      hypercube_get_from_indexes(_, _indexes, &u),
      "Index 0 (2) is invalid (max 1)");
  hypercube_delete(&_);

  hypercube_new(&_, paw, 2, _dimensions);
  p = paw_make(1.2);
  EXPECT_DEATH(
      hypercube_set(_, 6, &p),
      "Overall element index 6 is invalid (max 5)");
  EXPECT_DEATH(
      hypercube_get(_, 6, &p),
      "Overall element index 6 is invalid (max 5)");
  _indexes[0] = 2;
  EXPECT_DEATH(
      hypercube_get_from_indexes(_, _indexes, &p),
      "Index 0 (2) is invalid (max 1)");
  _indexes[0] = 1;
  _indexes[1] = 3;
  EXPECT_DEATH(
      hypercube_get_from_indexes(_, _indexes, &p),
      "Index 1 (3) is invalid (max 2)");
  hypercube_delete(&_);
}

TEST(Hypercube, ForEach) {
  bool _walks[4];
  ulong _dimensions[] = {2, 3, 4, 5};
  ulong _is[4];
  Hypercube *_ = null;

  hypercube_new(&_, ulong, 4, _dimensions);
  _walks[0] = false;
  _walks[1] = true;
  _walks[2] = false;
  _walks[3] = true;
  _is[0] = 1;
  _is[1] = 0;
  _is[2] = 2;
  _is[3] = 0;

  _is[1] = 1;
  EXPECT_DEATH(
      hypercube_foreach(_, _walks, _is, callback_, null, 1),
      "specified_indexes[1] set to 1 but this is a walk dimension, "
      "so must be 0");
  _is[1] = 0;
  _is[3] = 4;
  EXPECT_DEATH(
      hypercube_foreach(_, _walks, _is, callback_, null, 1),
      "specified_indexes[3] set to 4 but this is a walk dimension, "
      "so must be 0");
  hypercube_delete(&_);
}

TEST(UlongHypercube, ZeroDimensional) {
  ulong u;
  UlongHypercube *_ = null;

  ulong_hypercube_new(&_, 0, null);
  u = 240966;
  EXPECT_DEATH(
      hypercube_set(_, 1, &u),
      "Overall element index 1 is invalid (max 0)");
  EXPECT_DEATH(
      hypercube_get(_, 1, &u),
      "Overall element index 1 is invalid (max 0)");
  hypercube_delete(&_);
}

TEST(UlongHypercube, New) {
  ulong _dimensions[2];
  UlongHypercube *_ = null;

  _dimensions[0] = 0;
  EXPECT_DEATH(
      ulong_hypercube_new(&_, 1, _dimensions),
      "Dimension 0 is zero, which is not allowed");
  hypercube_delete(&_);

  _dimensions[1] = 1;
  EXPECT_DEATH(
      ulong_hypercube_new(&_, 2, _dimensions),
      "Dimension 0 is zero, which is not allowed");
  hypercube_delete(&_);

  _dimensions[0] = 1;
  _dimensions[1] = 0;
  EXPECT_DEATH(
      ulong_hypercube_new(&_, 2, _dimensions),
      "Dimension 1 is zero, which is not allowed");
  hypercube_delete(&_);
}

TEST(UlongHypercube, IFromO) {
  ulong _dimensions[] = {2, 3};
  UlongHypercube *_ = null;

  ulong_hypercube_new(&_, 1, _dimensions),
  EXPECT_DEATH(
      hypercube_i_from_o(_, 1, 1),
      "Dimension index 1 is invalid (max 0)");
  EXPECT_DEATH(
      hypercube_i_from_o(_, 0, 2),
      "Overall element index 2 is invalid (max 1)");
  hypercube_delete(&_);

  ulong_hypercube_new(&_, 2, _dimensions),
  EXPECT_DEATH(
      hypercube_i_from_o(_, 2, 1),
      "Dimension index 2 is invalid (max 1)");
  EXPECT_DEATH(
      hypercube_i_from_o(_, 1, 6),
      "Overall element index 6 is invalid (max 5)");
  hypercube_delete(&_);
}

TEST(UlongHypercube, SetGet) {
  ulong u;
  ulong _dimensions[] = {2, 3};
  ulong _indexes[2];
  UlongHypercube *_ = null;

  ulong_hypercube_new(&_, 1, _dimensions);
  u = 12;
  EXPECT_DEATH(
      hypercube_set(_, 2, &u),
      "Overall element index 2 is invalid (max 1)");
  EXPECT_DEATH(
      hypercube_get(_, 2, &u),
      "Overall element index 2 is invalid (max 1)");
  _indexes[0] = 2;
  EXPECT_DEATH(
      hypercube_get_from_indexes(_, _indexes, &u),
      "Index 0 (2) is invalid (max 1)");
  hypercube_delete(&_);

  ulong_hypercube_new(&_, 2, _dimensions),
  EXPECT_DEATH(
      hypercube_set(_, 6, &u),
      "Overall element index 6 is invalid (max 5)");
  EXPECT_DEATH(
      hypercube_get(_, 6, &u),
      "Overall element index 6 is invalid (max 5)");
  _indexes[0] = 2;
  EXPECT_DEATH(
      hypercube_get_from_indexes(_, _indexes, &u),
      "Index 0 (2) is invalid (max 1)");
  _indexes[0] = 1;
  _indexes[1] = 3;
  EXPECT_DEATH(
      hypercube_get_from_indexes(_, _indexes, &u),
      "Index 1 (3) is invalid (max 2)");
  hypercube_delete(&_);
}

TEST(UlongHypercube, NewFromKeySparseFrequencyArray) {
  ulong _maxes[] = {0, 0};
  ulong _dimensions[] = {1, 1};
  KeyValueInfo *_info = null;
  KeyValueArray *_array = null;
  UlongHypercube *_ = null;

  key_value_info_new(&_info, 1, _maxes, 2, _maxes);
  key_value_array_new(&_array, _info, 1);
  EXPECT_DEATH(
      ulong_hypercube_new_from_sparse_key_frequency_array(
        &_,
        &_array,
        _dimensions),
      "Number of values (2) must be 1");
  hypercube_delete(&_);
  key_value_info_delete(&_info);
}

TEST(UlongHypercube, ForEach) {
  bool _walks[4];
  ulong _dimensions[] = {2, 3, 4, 5};
  ulong _is[4];
  UlongHypercube *_ = null;

  ulong_hypercube_new(&_, 4, _dimensions);
  _walks[0] = false;
  _walks[1] = true;
  _walks[2] = false;
  _walks[3] = true;
  _is[0] = 1;
  _is[1] = 0;
  _is[2] = 2;
  _is[3] = 0;

  _is[1] = 1;
  EXPECT_DEATH(
      hypercube_foreach(_, _walks, _is, callback_, null, 1),
      "specified_indexes[1] set to 1 but this is a walk dimension, "
      "so must be 0");
  _is[1] = 0;
  _is[3] = 4;
  EXPECT_DEATH(
      hypercube_foreach(_, _walks, _is, callback_, null, 1),
      "specified_indexes[3] set to 4 but this is a walk dimension, "
      "so must be 0");
  hypercube_delete(&_);
}
FINISH_DEATH


static void callback_(Hypercube *this,
    bool *walk_dimensions,
    ulong *indexes,
    ulong o,
    void *x) {
}
