#include "JsonTest.h"

static void check_(char *s, char *e, char *pretty);


START

TEST(Json, NewEmpty) {
  Json *_j = null;

  json_new_empty(&_j);
  EXPECT_ULEQ(JSON_EMPTY, _j->type);
  EXPECT_STREQ("JsonEmpty", json_get_type_string(_j));
  EXPECT_TRUE(json_is_empty(_j));
  EXPECT_FALSE(json_is_unknown(_j));
  EXPECT_FALSE(json_is_null(_j));
  EXPECT_FALSE(json_is_scalar(_j));
  EXPECT_FALSE(json_is_value(_j));
  EXPECT_FALSE(json_is_array(_j));
  EXPECT_FALSE(json_is_object(_j));
  json_delete(&_j);
}

TEST(Json, Unknown) {
  bool b;
  char *s;
  ulong u;
  long l;
  double d;
  char *p;
  Json *_j = null;

  p = "pig";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_UNKNOWN, _j->type);
  EXPECT_STREQ("JsonUnknown", json_get_type_string(_j));
  EXPECT_STREQ("Json: Unknown first character 'p' in 'pig'", _j->_e);
  EXPECT_FALSE(json_is_empty(_j));
  EXPECT_TRUE(json_is_unknown(_j));
  EXPECT_FALSE(json_is_null(_j));
  EXPECT_FALSE(json_is_scalar(_j));
  EXPECT_FALSE(json_is_value(_j));
  EXPECT_FALSE(json_is_array(_j));
  EXPECT_FALSE(json_is_object(_j));
  EXPECT_FALSE(json_try_get_bool(_j, &b));
  EXPECT_FALSE(json_try_get_ulong(_j, &u));
  EXPECT_FALSE(json_try_get_long(_j, &l));
  EXPECT_FALSE(json_try_get_double(_j, &d));
  EXPECT_FALSE(json_try_get_string(_j, &s));
  json_delete(&_j);

  p = "  \xab""dog\n";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_UNKNOWN, _j->type);
  EXPECT_STREQ(
      "Json: Unknown first character '\\253' in '\\253dog\\n'",
      _j->_e);
  json_delete(&_j);
}

TEST(Json, Empty) {
  char *p;
  Json *_j = null;

  p = "";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_EMPTY, _j->type);
  EXPECT_STREQ("JsonEmpty", json_get_type_string(_j));
  EXPECT_TRUE(json_is_empty(_j));
  EXPECT_FALSE(json_is_unknown(_j));
  EXPECT_FALSE(json_is_null(_j));
  EXPECT_FALSE(json_is_scalar(_j));
  EXPECT_FALSE(json_is_value(_j));
  EXPECT_FALSE(json_is_array(_j));
  EXPECT_FALSE(json_is_object(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  p = " ";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_EMPTY, _j->type);
  EXPECT_TRUE(json_is_empty(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  p = "  ";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_EMPTY, _j->type);
  EXPECT_TRUE(json_is_empty(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  p = "\n \t\r\f\v \n";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_EMPTY, _j->type);
  EXPECT_TRUE(json_is_empty(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);
}

TEST(Json, Null) {
  char *p;
  Json *_j = null;

  p = " null  ";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NULL, _j->type);
  EXPECT_STREQ("JsonNull", json_get_type_string(_j));
  EXPECT_FALSE(json_is_empty(_j));
  EXPECT_FALSE(json_is_unknown(_j));
  EXPECT_TRUE(json_is_null(_j));
  EXPECT_TRUE(json_is_scalar(_j));
  EXPECT_TRUE(json_is_value(_j));
  EXPECT_FALSE(json_is_array(_j));
  EXPECT_FALSE(json_is_object(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  p = " noll  ";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NULL, _j->type);
  EXPECT_TRUE(json_is_null(_j));
  EXPECT_STREQ(
      "Json: Read 'n' and expected it to be 'null':\n"
      ". Expected 'null' but got 'noll  ': error at offset 1",
      _j->_e);
  json_delete(&_j);

  p = " nuil  ";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NULL, _j->type);
  EXPECT_TRUE(json_is_null(_j));
  EXPECT_STREQ(
      "Json: Read 'n' and expected it to be 'null':\n"
      ". Expected 'null' but got 'nuil  ': error at offset 2",
      _j->_e);
  json_delete(&_j);

  p = " nulm  ";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NULL, _j->type);
  EXPECT_TRUE(json_is_null(_j));
  EXPECT_STREQ(
      "Json: Read 'n' and expected it to be 'null':\n"
      ". Expected 'null' but got 'nulm  ': error at offset 3",
      _j->_e);
  json_delete(&_j);

  p = "  n";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NULL, _j->type);
  EXPECT_TRUE(json_is_null(_j));
  EXPECT_STREQ(
      "Json: Read 'n' and expected it to be 'null':\n"
      ". Expected 'null' but got 'n': error at offset 1",
      _j->_e);
  json_delete(&_j);

  p = "  nu";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NULL, _j->type);
  EXPECT_TRUE(json_is_null(_j));
  EXPECT_STREQ(
      "Json: Read 'n' and expected it to be 'null':\n"
      ". Expected 'null' but got 'nu': error at offset 2",
      _j->_e);
  json_delete(&_j);

  p = "  nul";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NULL, _j->type);
  EXPECT_TRUE(json_is_null(_j));
  EXPECT_STREQ(
      "Json: Read 'n' and expected it to be 'null':\n"
      ". Expected 'null' but got 'nul': error at offset 3",
      _j->_e);
  json_delete(&_j);

  p = "  nulll";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NULL, _j->type);
  EXPECT_TRUE(json_is_null(_j));
  EXPECT_STREQ(
      "Json: Read 'n' and expected it to be 'null':\n"
      ". Expected white space or punctuation but got 'l'",
      _j->_e);
  json_delete(&_j);

  p = "  null\x80";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NULL, _j->type);
  EXPECT_TRUE(json_is_null(_j));
  EXPECT_STREQ(
      "Json: Read 'n' and expected it to be 'null':\n"
      ". Expected white space or punctuation but got '\\200'",
      _j->_e);
  json_delete(&_j);
}

TEST(Json, True) {
  char *p;
  char *op;
  Json *_j = null;

  p = "true  ";
  op = p;
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_TRUE, _j->type);
  EXPECT_STREQ("JsonTrue", json_get_type_string(_j));
  EXPECT_FALSE(json_is_empty(_j));
  EXPECT_FALSE(json_is_unknown(_j));
  EXPECT_FALSE(json_is_null(_j));
  EXPECT_TRUE(json_is_scalar(_j));
  EXPECT_TRUE(json_is_value(_j));
  EXPECT_FALSE(json_is_array(_j));
  EXPECT_FALSE(json_is_object(_j));
  EXPECT_TRUE(json_get_bool(_j));
  EXPECT_NULL(_j->_e);
  EXPECT_PEQ(op + 4, p);
  json_delete(&_j);

  p = " tru  \n";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_TRUE, _j->type);
  EXPECT_STREQ(
      "Json: Read 't' and expected it to be 'true':\n"
      ". Expected 'true' but got 'tru  \\n': error at offset 3",
      _j->_e);
  EXPECT_TRUE(json_get_bool(_j));
  json_delete(&_j);

  p = "\n tr";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_TRUE, _j->type);
  EXPECT_STREQ(
      "Json: Read 't' and expected it to be 'true':\n"
      ". Expected 'true' but got 'tr': error at offset 2",
      _j->_e);
  EXPECT_TRUE(json_get_bool(_j));
  json_delete(&_j);
}

TEST(Json, False) {
  char *p;
  char *op;
  Json *_j = null;

  p = "\vfalse  ";
  op = p;
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_FALSE, _j->type);
  EXPECT_STREQ("JsonFalse", json_get_type_string(_j));
  EXPECT_FALSE(json_is_empty(_j));
  EXPECT_FALSE(json_is_unknown(_j));
  EXPECT_FALSE(json_is_null(_j));
  EXPECT_TRUE(json_is_scalar(_j));
  EXPECT_TRUE(json_is_value(_j));
  EXPECT_FALSE(json_is_array(_j));
  EXPECT_FALSE(json_is_object(_j));
  EXPECT_NULL(_j->_e);
  EXPECT_PEQ(op + 6, p);
  EXPECT_FALSE(json_get_bool(_j));
  json_delete(&_j);

  p = " fals  \n";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_FALSE, _j->type);
  EXPECT_STREQ(
      "Json: Read 'f' and expected it to be 'false':\n"
      ". Expected 'false' but got 'fals  \\n': error at offset 4",
      _j->_e);
  EXPECT_FALSE(json_get_bool(_j));
  json_delete(&_j);

  p = "\n fa";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_FALSE, _j->type);
  EXPECT_STREQ(
      "Json: Read 'f' and expected it to be 'false':\n"
      ". Expected 'false' but got 'fa': error at offset 2",
      _j->_e);
  EXPECT_FALSE(json_get_bool(_j));
  json_delete(&_j);
}

TEST(Json, Entire) {
  Json *_j = null;

  json_new_entire(&_j, " true  ");
  EXPECT_ULEQ(_j->type, JSON_TRUE);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_entire(&_j, "1,2");
  EXPECT_STREQ(
      "Json: Expected to use entire string '1,2' but didn't use ',2'",
      _j->_e);
  json_delete(&_j);

  json_new_entire(&_j, "{\"a\":");
  EXPECT_STREQ(
      "JsonObject: String ended before object complete in '{\"a\":'",
      _j->_e);
  json_delete(&_j);

  json_new_entire(&_j, "[1,,2]");
  EXPECT_STREQ(
      "Json: Expected to use entire string '[1,,2]' but didn't use ',2]':\n"
      ". JsonArray: Got ',' after another ',' at ',2]' in '[1,,2]'",
      _j->_e);
  json_delete(&_j);
}

TEST(Json, Chain) {
  char *p;
  Json *_j = null;

  p = "\vnull  true\n\rfalse  pig";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NULL, _j->type);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_TRUE, _j->type);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_FALSE, _j->type);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_UNKNOWN, _j->type);
  EXPECT_STREQ("Json: Unknown first character 'p' in 'pig'", _j->_e);
  json_delete(&_j);
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_UNKNOWN, _j->type);
  EXPECT_STREQ("Json: Unknown first character 'i' in 'ig'", _j->_e);
  json_delete(&_j);

  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_UNKNOWN, _j->type);
  EXPECT_STREQ("Json: Unknown first character 'g' in 'g'", _j->_e);
  json_delete(&_j);

  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_EMPTY, _j->type);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);
}

TEST(Json, String) {
  char *e;
  char *p;
  char *s;
  char *_err = null;
  Json *_j = null;
  JsonString *_js = null;
  JsonString *_js2 = null;

  p = "  \"\"";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_STRING, _j->type);
  EXPECT_STREQ("", json_get_json_string(_j)->_s);
  s = json_get_string(_j);
  EXPECT_STREQ("", s);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = "  \"";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_STRING, _j->type);
  EXPECT_STREQ(
      "JsonString: String ended without closing '\"': '\"'",
      _j->_e);
  EXPECT_STREQ("", p);
  EXPECT_STREQ("", json_get_json_string(_j)->_s);
  s = json_get_string(_j);
  EXPECT_STREQ("", s);
  json_delete(&_j);

  p = "  \"\xfe";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_STRING, _j->type);
  EXPECT_STREQ(
      "JsonString: String ended without closing '\"': '\"\\376'",
      _j->_e);
  EXPECT_STREQ("", p);
  EXPECT_STREQ("\xfe", json_get_json_string(_j)->_s);
  s = json_get_string(_j);
  EXPECT_STREQ("\xfe", s);
  json_delete(&_j);

  p = "\"dog";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_STRING, _j->type);
  EXPECT_STREQ(
      "JsonString: String ended without closing '\"': '\"dog'",
      _j->_e);
  EXPECT_STREQ("", p);
  EXPECT_STREQ("dog", json_get_json_string(_j)->_s);
  s = json_get_string(_j);
  EXPECT_STREQ("dog", s);
  json_delete(&_j);

  p = "\"a\"";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_STRING, _j->type);
  EXPECT_STREQ("a", json_get_json_string(_j)->_s);
  s = json_get_string(_j);
  EXPECT_STREQ("a", s);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " \"ab\"";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_STRING, _j->type);
  EXPECT_STREQ("ab", json_get_json_string(_j)->_s);
  s = json_get_string(_j);
  EXPECT_STREQ("ab", s);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " \"ab cd\"";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_STRING, _j->type);
  EXPECT_STREQ("ab cd", json_get_json_string(_j)->_s);
  s = json_get_string(_j);
  EXPECT_STREQ("ab cd", s);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " \" \\\\ \\\" \\/\\b\\f\\n\\r\\t\"";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_STRING, _j->type);
  e = " \\\\ \\\" \\/\\b\\f\\n\\r\\t";
  EXPECT_STREQ(e, json_get_json_string(_j)->_s);
  s = json_get_string(_j);
  EXPECT_STREQ(e, s);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = "\"\\g\"";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_STRING, _j->type);
  EXPECT_STREQ(
      "JsonString: Illegal escape sequence '\\\\g' in '\"\\\\g\"' "
      "at '\\\\g\"'",
      _j->_e);
  EXPECT_STREQ("", json_get_json_string(_j)->_s);
  s = json_get_string(_j);
  EXPECT_STREQ("", s);
  EXPECT_STREQ("\\g\"", p);
  json_delete(&_j);

  p = "\" \\\x9f \"";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_STRING, _j->type);
  EXPECT_STREQ(
      "JsonString: Illegal escape sequence '\\\\\\237' "
      "in '\" \\\\\\237 \"' "
      "at '\\\\\\237 \"'",
      _j->_e);
  EXPECT_STREQ(" ", json_get_json_string(_j)->_s);
  s = json_get_string(_j);
  EXPECT_STREQ(" ", s);
  EXPECT_STREQ("\\\x9f \"", p);
  json_delete(&_j);

  p = "\"\\u0123\",null";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_STRING, _j->type);
  EXPECT_STREQ("\\u0123", json_get_json_string(_j)->_s);
  s = json_get_string(_j);
  EXPECT_STREQ("\\u0123", s);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ(",null", p);
  json_delete(&_j);

  p = "\"a\\u0123&\\ucdEF!\"";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_STRING, _j->type);
  e = "a\\u0123&\\ucdef!";
  EXPECT_STREQ(e, json_get_json_string(_j)->_s);
  s = json_get_string(_j);
  EXPECT_STREQ(e, s);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = "\"a\\u0123&\\ucdEF!\"";
  json_string_new(&_js, &p, &_err);
  e = "a\\u0123&\\ucdef!";
  EXPECT_STREQ(e, _js->_s);
  EXPECT_NULL(_err);
  EXPECT_STREQ("", p);
  json_string_new_copy(&_js2, _js);
  json_string_delete(&_js);
  EXPECT_STREQ(e, _js2->_s);
  json_string_delete(&_js2);

  p = "\"\\u";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_STRING, _j->type);
  EXPECT_STREQ(
      "JsonString: Illegal Unicode escape sequence in '\"\\\\u' at '\\\\u'",
      _j->_e);
  EXPECT_STREQ("", json_get_json_string(_j)->_s);
  EXPECT_STREQ("\\u", p);
  s = json_get_string(_j);
  EXPECT_STREQ("", s);
  json_delete(&_j);

  p = "\"\\ug000";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_STRING, _j->type);
  EXPECT_STREQ(
      "JsonString: Illegal Unicode escape sequence "
      "in '\"\\\\ug000' at '\\\\ug000'",
      _j->_e);
  EXPECT_STREQ("", json_get_json_string(_j)->_s);
  s = json_get_string(_j);
  EXPECT_STREQ("", s);
  EXPECT_STREQ("\\ug000", p);
  json_delete(&_j);

  p = "\"\\u012";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_STRING, _j->type);
  EXPECT_STREQ(
      "JsonString: Illegal Unicode escape sequence "
      "in '\"\\\\u012' at '\\\\u012'",
      _j->_e);
  EXPECT_STREQ("", json_get_json_string(_j)->_s);
  s = json_get_string(_j);
  EXPECT_STREQ("", s);
  EXPECT_STREQ("\\u012", p);
  json_delete(&_j);
}

TEST(Json, Number) {
  char *p;
  char *_err = null;
  JsonNumber *n;
  JsonNumber *_jn = null;
  JsonNumber *_jn2 = null;
  Json *_j = null;

  p = "  0";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  EXPECT_STREQ("JsonNumber", json_get_type_string(_j));
  EXPECT_FALSE(json_is_empty(_j));
  EXPECT_FALSE(json_is_unknown(_j));
  EXPECT_FALSE(json_is_null(_j));
  EXPECT_TRUE(json_is_scalar(_j));
  EXPECT_TRUE(json_is_value(_j));
  EXPECT_FALSE(json_is_array(_j));
  EXPECT_FALSE(json_is_object(_j));
  n = json_get_json_number(_j);
  EXPECT_STREQ("0", n->_s);
  EXPECT_ULEQ(0, json_get_ulong(_j));
  EXPECT_ULEQ(0, json_get_long(_j));
  EXPECT_ULEQ(0, json_get_double(_j));
  EXPECT_FALSE(n->negative);
  EXPECT_TRUE(n->integer);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " -0";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  n = json_get_json_number(_j);
  EXPECT_STREQ("-0", n->_s);
  EXPECT_ULEQ(0, json_get_long(_j));
  EXPECT_ULEQ(0, json_get_double(_j));
  EXPECT_TRUE(n->negative);
  EXPECT_TRUE(n->integer);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " 1";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  n = json_get_json_number(_j);
  EXPECT_STREQ("1", n->_s);
  EXPECT_ULEQ(1, json_get_ulong(_j));
  EXPECT_ULEQ(1, json_get_long(_j));
  EXPECT_ULEQ(1, json_get_double(_j));
  EXPECT_FALSE(n->negative);
  EXPECT_TRUE(n->integer);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " 12";
  json_number_new(&_jn, &p, &_err);
  EXPECT_STREQ("12", _jn->_s);
  EXPECT_ULEQ(12, json_number_get_ulong(_jn));
  EXPECT_ULEQ(12, json_number_get_long(_jn));
  EXPECT_ULEQ(12, json_number_get_double(_jn));
  EXPECT_FALSE(_jn->negative);
  EXPECT_TRUE(_jn->integer);
  EXPECT_NULL(_err);
  EXPECT_STREQ("", p);
  json_number_new_copy(&_jn2, _jn);
  json_number_delete(&_jn);
  EXPECT_STREQ("12", _jn2->_s);
  json_number_delete(&_jn2);

  p = "\n-2";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  n = json_get_json_number(_j);
  EXPECT_STREQ("-2", n->_s);
  EXPECT_LEQ(-2, json_get_long(_j));
  EXPECT_DEQ(-2, json_get_double(_j));
  EXPECT_TRUE(n->negative);
  EXPECT_TRUE(n->integer);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " 01";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  n = json_get_json_number(_j);
  EXPECT_NULL(n->_s);
  EXPECT_STREQ(
      "JsonNumber: Cannot have digit after '0': got '1' in '01'",
      _j->_e);
  EXPECT_STREQ("01", p);
  json_delete(&_j);

  p = " 10";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  n = json_get_json_number(_j);
  EXPECT_STREQ("10", n->_s);
  EXPECT_ULEQ(10, json_get_ulong(_j));
  EXPECT_ULEQ(10, json_get_long(_j));
  EXPECT_ULEQ(10, json_get_double(_j));
  EXPECT_FALSE(n->negative);
  EXPECT_TRUE(n->integer);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " -100";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  n = json_get_json_number(_j);
  EXPECT_STREQ("-100", n->_s);
  EXPECT_LEQ(-100, json_get_long(_j));
  EXPECT_DEQ(-100, json_get_double(_j));
  EXPECT_TRUE(n->negative);
  EXPECT_TRUE(n->integer);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " \f 1009";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  n = json_get_json_number(_j);
  EXPECT_STREQ("1009", n->_s);
  EXPECT_ULEQ(1009, json_get_ulong(_j));
  EXPECT_ULEQ(1009, json_get_long(_j));
  EXPECT_ULEQ(1009, json_get_double(_j));
  EXPECT_FALSE(n->negative);
  EXPECT_TRUE(n->integer);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " 0.";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  n = json_get_json_number(_j);
  EXPECT_NULL(n->_s);
  EXPECT_STREQ(
      "JsonNumber: Expected digit after '.' but got '' in '0.'",
      _j->_e);
  EXPECT_STREQ("0.", p);
  json_delete(&_j);

  p = " \f 0.0";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  n = json_get_json_number(_j);
  EXPECT_STREQ("0.0", n->_s);
  EXPECT_ULEQ(0, json_get_double(_j));
  EXPECT_FALSE(n->negative);
  EXPECT_FALSE(n->integer);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " -0.0";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  n = json_get_json_number(_j);
  EXPECT_STREQ("-0.0", n->_s);
  EXPECT_ULEQ(0, json_get_double(_j));
  EXPECT_TRUE(n->negative);
  EXPECT_FALSE(n->integer);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " -3.14";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  n = json_get_json_number(_j);
  EXPECT_STREQ("-3.14", n->_s);
  EXPECT_DEQ(-3.14, json_get_double(_j));
  EXPECT_TRUE(n->negative);
  EXPECT_FALSE(n->integer);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " -3.14,12";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  n = json_get_json_number(_j);
  EXPECT_STREQ("-3.14", n->_s);
  EXPECT_DEQ(-3.14, json_get_double(_j));
  EXPECT_TRUE(n->negative);
  EXPECT_FALSE(n->integer);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ(",12", p);
  json_delete(&_j);

  p = " 0e0";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  n = json_get_json_number(_j);
  EXPECT_STREQ("0e0", n->_s);
  EXPECT_ULEQ(0, json_get_double(_j));
  EXPECT_FALSE(n->negative);
  EXPECT_FALSE(n->integer);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " 0e";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  n = json_get_json_number(_j);
  EXPECT_NULL(n->_s);
  EXPECT_STREQ(
      "JsonNumber: Expected digit in exponent but got '' in '0e'",
      _j->_e);
  EXPECT_STREQ("0e", p);
  json_delete(&_j);

  p = " 1E4";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  n = json_get_json_number(_j);
  EXPECT_STREQ("1e4", n->_s);
  EXPECT_ULEQ(1e4, json_get_double(_j));
  EXPECT_FALSE(n->negative);
  EXPECT_FALSE(n->integer);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " 1.2E4";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  n = json_get_json_number(_j);
  EXPECT_STREQ("1.2e4", n->_s);
  EXPECT_ULEQ(1.2e4, json_get_double(_j));
  EXPECT_FALSE(n->negative);
  EXPECT_FALSE(n->integer);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " -1.2e4";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  n = json_get_json_number(_j);
  EXPECT_STREQ("-1.2e4", n->_s);
  EXPECT_DEQ(-1.2e4, json_get_double(_j));
  EXPECT_TRUE(n->negative);
  EXPECT_FALSE(n->integer);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " -1.23e-4";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  n = json_get_json_number(_j);
  EXPECT_STREQ("-1.23e-4", n->_s);
  EXPECT_DEQ(-1.23e-4, json_get_double(_j));
  EXPECT_TRUE(n->negative);
  EXPECT_FALSE(n->integer);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " -1.23e+004";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  n = json_get_json_number(_j);
  EXPECT_STREQ("-1.23e+004", n->_s);
  EXPECT_DEQ(-1.23e4, json_get_double(_j));
  EXPECT_TRUE(n->negative);
  EXPECT_FALSE(n->integer);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " -0.23e+0047";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  n = json_get_json_number(_j);
  EXPECT_STREQ("-0.23e+0047", n->_s);
  EXPECT_DEQ(-0.23e47, json_get_double(_j));
  EXPECT_TRUE(n->negative);
  EXPECT_FALSE(n->integer);
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " 1dog";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  n = json_get_json_number(_j);
  EXPECT_NULL(n->_s);
  EXPECT_STREQ("1dog", p);
  json_delete(&_j);

  p = " -1.2e-4e-5";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  n = json_get_json_number(_j);
  EXPECT_NULL(n->_s);
  EXPECT_STREQ(
      "JsonNumber: Number not terminated properly:\n"
      ". Expected white space or punctuation but got 'e-5'",
      _j->_e);
  EXPECT_STREQ("-1.2e-4e-5", p);
  json_delete(&_j);
}

TEST(Json, Array) {
  char *_err = null;
  char *p;
  JsonArray *a;
  JsonArray *aa;
  JsonArray *_ja = null;
  Json *e;
  Json *_j = null;
  Json *_j2 = null;

  json_new_empty_array(&_j);
  EXPECT_ULEQ(JSON_ARRAY, _j->type);
  EXPECT_STREQ("JsonArray", json_get_type_string(_j));
  EXPECT_FALSE(json_is_empty(_j));
  EXPECT_FALSE(json_is_unknown(_j));
  EXPECT_FALSE(json_is_null(_j));
  EXPECT_FALSE(json_is_scalar(_j));
  EXPECT_TRUE(json_is_value(_j));
  EXPECT_TRUE(json_is_array(_j));
  EXPECT_FALSE(json_is_object(_j));
  a = json_get_json_array(_j);
  EXPECT_ULEQ(0, json_array_length(a));
  EXPECT_NULL(json_array_get(a, 0));
  EXPECT_NULL(json_array_get(a, 1));
  EXPECT_NULL(json_array_get(a, 2));
  EXPECT_NULL(json_array_get(a, -1));
  EXPECT_NULL(json_array_get(a, -2));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  p = "  []";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_ARRAY, _j->type);
  a = json_get_json_array(_j);
  EXPECT_ULEQ(0, json_array_length(a));
  EXPECT_NULL(json_array_get(a, 0));
  EXPECT_NULL(json_array_get(a, 1));
  EXPECT_NULL(json_array_get(a, 2));
  EXPECT_NULL(json_array_get(a, -1));
  EXPECT_NULL(json_array_get(a, -2));
  EXPECT_NULL(_j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " [null],dog";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_ARRAY, _j->type);
  EXPECT_NULL(_j->_e);
  a = json_get_json_array(_j);
  EXPECT_ULEQ(1, json_array_length(a));
  e = json_array_get(a, 0);
  EXPECT_ULEQ(JSON_NULL, e->type);
  e = json_array_get(a, -1);
  EXPECT_ULEQ(JSON_NULL, e->type);
  EXPECT_NULL(json_array_get(a, 1));
  EXPECT_NULL(json_array_get(a, -2));
  EXPECT_STREQ(",dog", p);
  json_new_entire(&_j2, "\"cat\"");
  json_append_array(_j, &_j2);
  EXPECT_ULEQ(2, json_array_length(a));
  e = json_array_get(a, 1);
  EXPECT_STREQ("cat", json_get_json_string(e)->_s);
  json_new_entire(&_j2, "[9,24]");
  json_append_array_with_array(_j, &_j2);
  EXPECT_ULEQ(4, json_array_length(a));
  e = json_array_get(a, 2);
  EXPECT_STREQ("9", json_get_json_number(e)->_s);
  e = json_array_get(a, 3);
  EXPECT_STREQ("24", json_get_json_number(e)->_s);
  json_delete(&_j);

  p = " [  true  ,  false \n, null]  ,dog";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_ARRAY, _j->type);
  EXPECT_NULL(_j->_e);
  a = json_get_json_array(_j);
  EXPECT_ULEQ(3, json_array_length(a));
  e = json_array_get(a, 0);
  EXPECT_ULEQ(JSON_TRUE, e->type);
  e = json_array_get(a, -3);
  EXPECT_ULEQ(JSON_TRUE, e->type);
  e = json_array_get(a, 1);
  EXPECT_ULEQ(JSON_FALSE, e->type);
  e = json_array_get(a, -2);
  EXPECT_ULEQ(JSON_FALSE, e->type);
  e = json_array_get(a, 2);
  EXPECT_ULEQ(JSON_NULL, e->type);
  e = json_array_get(a, -1);
  EXPECT_ULEQ(JSON_NULL, e->type);
  EXPECT_NULL(json_array_get(a, 3));
  EXPECT_NULL(json_array_get(a, -4));
  EXPECT_STREQ("  ,dog", p);
  json_delete(&_j);

  p = "[null,true]";
  json_array_new(&_ja, &p, &_err);
  EXPECT_NULL(_err);
  EXPECT_ULEQ(2, json_array_length(_ja));
  e = json_array_get(_ja, 0);
  EXPECT_ULEQ(JSON_NULL, e->type);
  e = json_array_get(_ja, 1);
  EXPECT_ULEQ(JSON_TRUE, e->type);
  json_array_delete(&_ja);

  json_array_new_empty(&_ja);
  EXPECT_ULEQ(0, json_array_length(_ja));
  json_array_delete(&_ja);

  p = "[null]";
  json_array_new_entire(&_ja, p);
  EXPECT_ULEQ(1, json_array_length(_ja));
  e = json_array_get(_ja, 0);
  EXPECT_ULEQ(JSON_NULL, e->type);
  json_new_entire(&_j, "false");
  json_array_append(_ja, &_j);
  EXPECT_ULEQ(2, json_array_length(_ja));
  e = json_array_get(_ja, 0);
  EXPECT_ULEQ(JSON_NULL, e->type);
  e = json_array_get(_ja, 1);
  EXPECT_ULEQ(JSON_FALSE, e->type);
  json_new_entire(&_j, "[1,\"a\"]");
  json_array_append_array(_ja, &_j);
  EXPECT_ULEQ(4, json_array_length(_ja));
  e = json_array_get(_ja, 2);
  EXPECT_ULEQ(JSON_NUMBER, e->type);
  e = json_array_get(_ja, 3);
  EXPECT_ULEQ(JSON_STRING, e->type);
  json_array_delete(&_ja);

  p = " [1,0,-1,[1.2,-1.3e4],\"dog\"]";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_ARRAY, _j->type);
  EXPECT_NULL(_j->_e);
  a = json_get_json_array(_j);
  EXPECT_ULEQ(5, json_array_length(a));
  EXPECT_NULL(json_array_get(a, 5));
  EXPECT_NULL(json_array_get(a, 6));
  EXPECT_NULL(json_array_get(a, -7));
  e = json_array_get(a, 0);
  EXPECT_ULEQ(JSON_NUMBER, e->type);
  EXPECT_STREQ("1", json_get_json_number(e)->_s);
  e = json_array_get(a, -5);
  EXPECT_ULEQ(JSON_NUMBER, e->type);
  EXPECT_STREQ("1", json_get_json_number(e)->_s);
  e = json_array_get(a, 1);
  EXPECT_ULEQ(JSON_NUMBER, e->type);
  EXPECT_STREQ("0", json_get_json_number(e)->_s);
  e = json_array_get(a, -4);
  EXPECT_ULEQ(JSON_NUMBER, e->type);
  EXPECT_STREQ("0", json_get_json_number(e)->_s);
  e = json_array_get(a, 2);
  EXPECT_ULEQ(JSON_NUMBER, e->type);
  EXPECT_STREQ("-1", json_get_json_number(e)->_s);
  e = json_array_get(a, -3);
  EXPECT_ULEQ(JSON_NUMBER, e->type);
  EXPECT_STREQ("-1", json_get_json_number(e)->_s);
  e = json_array_get(a, 3);
  EXPECT_ULEQ(JSON_ARRAY, e->type);
  e = json_array_get(a, -2);
  EXPECT_ULEQ(JSON_ARRAY, e->type);
  aa = json_get_json_array(e);
  EXPECT_ULEQ(2, json_array_length(aa));
  e = json_array_get(aa, 0);
  EXPECT_ULEQ(JSON_NUMBER, e->type);
  EXPECT_STREQ("1.2", json_get_json_number(e)->_s);
  e = json_array_get(aa, -2);
  EXPECT_ULEQ(JSON_NUMBER, e->type);
  EXPECT_STREQ("1.2", json_get_json_number(e)->_s);
  e = json_array_get(aa, 1);
  EXPECT_ULEQ(JSON_NUMBER, e->type);
  EXPECT_STREQ("-1.3e4", json_get_json_number(e)->_s);
  e = json_array_get(aa, -1);
  EXPECT_ULEQ(JSON_NUMBER, e->type);
  EXPECT_STREQ("-1.3e4", json_get_json_number(e)->_s);
  e = json_array_get(a, 4);
  EXPECT_ULEQ(JSON_STRING, e->type);
  EXPECT_STREQ("dog", ((JsonString *)e->_o)->_s);
  e = json_array_get(a, -1);
  EXPECT_ULEQ(JSON_STRING, e->type);
  EXPECT_STREQ("dog", ((JsonString *)e->_o)->_s);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " [12";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_ARRAY, _j->type);
  EXPECT_STREQ(
      "JsonArray: String ended before array complete in '[12'",
      _j->_e);
  a = json_get_json_array(_j);
  EXPECT_ULEQ(1, json_array_length(a));
  e = json_array_get(a, 0);
  EXPECT_ULEQ(JSON_NUMBER, e->type);
  EXPECT_STREQ("12", json_get_json_number(e)->_s);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " [,1]";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_ARRAY, _j->type);
  a = json_get_json_array(_j);
  EXPECT_ULEQ(0, json_array_length(a));
  EXPECT_STREQ(
      "JsonArray: Got ',' before any elements at ',1]' in '[,1]'",
      _j->_e);
  EXPECT_STREQ(",1]", p);
  json_delete(&_j);

  p = " [-1,]";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_ARRAY, _j->type);
  a = json_get_json_array(_j);
  EXPECT_ULEQ(1, json_array_length(a));
  e = json_array_get(a, 0);
  EXPECT_ULEQ(JSON_NUMBER, e->type);
  EXPECT_STREQ("-1", json_get_json_number(e)->_s);
  EXPECT_STREQ(
      "JsonArray: Got ']' but expecting element after ',' at ']' in '[-1,]'",
      _j->_e);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " [3.1,,4]";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_ARRAY, _j->type);
  a = json_get_json_array(_j);
  EXPECT_ULEQ(1, json_array_length(a));
  e = json_array_get(a, 0);
  EXPECT_ULEQ(JSON_NUMBER, e->type);
  EXPECT_STREQ("3.1", json_get_json_number(e)->_s);
  EXPECT_STREQ(
      "JsonArray: Got ',' after another ',' at ',4]' in '[3.1,,4]'",
      _j->_e);
  EXPECT_STREQ(",4]", p);
  json_delete(&_j);

  p = " [\"a\":null]";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_ARRAY, _j->type);
  a = json_get_json_array(_j);
  EXPECT_ULEQ(1, json_array_length(a));
  e = json_array_get(a, 0);
  EXPECT_ULEQ(JSON_STRING, e->type);
  EXPECT_STREQ(
      "JsonArray: Got element after another element without separating ',' "
      "at ':null]' in '[\"a\":null]'",
      _j->_e);
  EXPECT_STREQ(":null]", p);
  json_delete(&_j);

  p = " [true false]";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_ARRAY, _j->type);
  a = json_get_json_array(_j);
  EXPECT_ULEQ(1, json_array_length(a));
  e = json_array_get(a, 0);
  EXPECT_ULEQ(JSON_TRUE, e->type);
  EXPECT_STREQ(
      "JsonArray: Got element after another element "
      "without separating ',' at 'false]' in '[true false]'",
      _j->_e);
  EXPECT_STREQ("false]", p);
  json_delete(&_j);

  p = " [true, \"\\g\"]";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_ARRAY, _j->type);
  a = json_get_json_array(_j);
  EXPECT_ULEQ(2, json_array_length(a));
  e = json_array_get(a, 0);
  EXPECT_ULEQ(JSON_TRUE, e->type);
  e = json_array_get(a, 1);
  EXPECT_ULEQ(JSON_STRING, e->type);
  EXPECT_STREQ(
      "JsonArray: Error in element 1:\n"
      ". JsonString: Illegal escape sequence '\\\\g' "
      "in '\"\\\\g\"]' at '\\\\g\"]'",
      _j->_e);
  EXPECT_STREQ(
      "JsonString: Illegal escape sequence '\\\\g' "
      "in '\"\\\\g\"]' at '\\\\g\"]'",
      e->_e);
  EXPECT_STREQ("\\g\"]", p);
  json_delete(&_j);

  p = " [[[a]]]";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_ARRAY, _j->type);
  EXPECT_STREQ(
      "JsonArray: Error in element 0:\n"
      ". JsonArray: Error in element 0:\n"
      ". . JsonArray: Got non-value element of type JsonUnknown "
      "at 'a]]]' in '[a]]]'",
      _j->_e);
  a = json_get_json_array(_j);
  EXPECT_ULEQ(1, json_array_length(a));
  e = json_array_get(a, 0);
  EXPECT_ULEQ(JSON_ARRAY, e->type);
  EXPECT_STREQ(
      "JsonArray: Error in element 0:\n"
      ". JsonArray: Got non-value element of type JsonUnknown "
      "at 'a]]]' in '[a]]]'",
      e->_e);
  a = json_get_json_array(e);
  EXPECT_ULEQ(JSON_ARRAY, e->type);
  EXPECT_ULEQ(1, json_array_length(a));
  e = json_array_get(a, 0);
  EXPECT_STREQ(
      "JsonArray: Got non-value element of type JsonUnknown "
      "at 'a]]]' in '[a]]]'",
      e->_e);
  EXPECT_STREQ("]]]", p);
  json_delete(&_j);

  p = " [[[false]]],true";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_ARRAY, _j->type);
  EXPECT_NULL(_j->_e);
  a = json_get_json_array(_j);
  EXPECT_ULEQ(1, json_array_length(a));
  e = json_array_get(a, 0);
  EXPECT_ULEQ(JSON_ARRAY, e->type);
  EXPECT_NULL(e->_e);
  a = json_get_json_array(e);
  EXPECT_ULEQ(1, json_array_length(a));
  e = json_array_get(a, 0);
  EXPECT_ULEQ(JSON_ARRAY, e->type);
  EXPECT_NULL(e->_e);
  a = json_get_json_array(e);
  EXPECT_ULEQ(1, json_array_length(a));
  e = json_array_get(a, 0);
  EXPECT_ULEQ(JSON_FALSE, e->type);
  EXPECT_STREQ(",true", p);
  json_delete(&_j);
}

TEST(Json, NativeArrayNewFromEntire) {
  char **_ss = null;
  ulong *_us = null;
  long *_ls = null;
  ulong n;

  long_array_new_from_json_entire(&_ls, &n, "[-45,12,-3]");
  EXPECT_ULEQ(3, n);
  EXPECT_LEQ(-45, _ls[0]);
  EXPECT_LEQ(12, _ls[1]);
  EXPECT_LEQ(-3, _ls[2]);
  deletev(&_ls);

  ulong_array_new_from_json_entire(&_us, &n, "[45,12,3]");
  EXPECT_ULEQ(3, n);
  EXPECT_LEQ(45, _us[0]);
  EXPECT_LEQ(12, _us[1]);
  EXPECT_LEQ(3, _us[2]);
  deletev(&_us);

  string_array_new_from_json_entire(&_ss, &n, "[\"a\",\"dog\"]");
  EXPECT_ULEQ(2, n);
  EXPECT_STREQ("a", _ss[0]);
  EXPECT_STREQ("dog", _ss[1]);
  delete(&_ss, n, string);
}

TEST(Json, Object) {
  char *p;
  char *_err = null;
  char *_key = null;
  char *_s = null;
  Dictionary *d;
  DictionaryPair *pair;
  DictionaryPairs *_pairs = null;
  JsonNumber *n;
  JsonObject *o;
  JsonObject *_jo = null;
  JsonObject *_jo2 = null;
  JsonString *s;
  Json *_j = null;
  Json *_j2 = null;
  Json *k;

  p = "  {}";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_OBJECT, _j->type);
  EXPECT_STREQ("JsonObject", json_get_type_string(_j));
  EXPECT_FALSE(json_is_empty(_j));
  EXPECT_FALSE(json_is_unknown(_j));
  EXPECT_FALSE(json_is_null(_j));
  EXPECT_FALSE(json_is_scalar(_j));
  EXPECT_TRUE(json_is_value(_j));
  EXPECT_FALSE(json_is_array(_j));
  EXPECT_TRUE(json_is_object(_j));
  EXPECT_NULL(_j->_e);
  o = json_get_json_object(_j);
  d = o->_d;
  dictionary_pairs_new(&_pairs, d, true);
  EXPECT_ULEQ(0, dictionary_cardinality(d));
  EXPECT_ULEQ(0, _pairs->cardinality);
  dictionary_pairs_delete(&_pairs);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = "{\"\":null}";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_OBJECT, _j->type);
  EXPECT_NULL(_j->_e);
  o = json_get_json_object(_j);
  d = o->_d;
  k = dictionary_find(d, "");
  EXPECT_ULEQ(JSON_NULL, k->type);
  EXPECT_ULEQ(1, dictionary_cardinality(d));
  dictionary_pairs_new(&_pairs, d, true);
  EXPECT_ULEQ(1, _pairs->cardinality);
  pair = _pairs->_pairs[0];
  EXPECT_STREQ("", pair->key);
  k = pair->object;
  EXPECT_ULEQ(JSON_NULL, k->type);
  dictionary_pairs_delete(&_pairs);
  EXPECT_STREQ("", p);
  json_delete(&_j);

  p = " {  \"dog\" :  true, \"cat\":false\n}";
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_OBJECT, _j->type);
  EXPECT_NULL(_j->_e);
  o = json_get_json_object(_j);
  d = o->_d;
  k = dictionary_find(d, "cat");
  EXPECT_ULEQ(JSON_FALSE, k->type);
  k = dictionary_find(d, "dog");
  EXPECT_ULEQ(JSON_TRUE, k->type);
  EXPECT_NULL(dictionary_find(d, "pig"));
  EXPECT_ULEQ(2, dictionary_cardinality(d));
  dictionary_pairs_new(&_pairs, d, true);
  EXPECT_ULEQ(2, _pairs->cardinality);
  pair = _pairs->_pairs[0];
  EXPECT_STREQ("dog", pair->key);
  k = pair->object;
  EXPECT_ULEQ(JSON_TRUE, k->type);
  pair = _pairs->_pairs[1];
  EXPECT_STREQ("cat", pair->key);
  k = pair->object;
  EXPECT_ULEQ(JSON_FALSE, k->type);
  dictionary_pairs_delete(&_pairs);
  EXPECT_STREQ("", p);
  string_new_copy(&_key, "pig");
  json_new_entire(&_j2, "[9,24]");
  json_donate_to_object(_j, &_key, &_j2);
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("{\"dog\":true,\"cat\":false,\"pig\":[9,24]}", _s);
  string_delete(&_s);
  json_delete(&_j);

  p = " {  \"dog\" :  true, \"cat\":false\n}";
  json_object_new(&_jo, &p, &_err);
  EXPECT_NULL(_err);
  d = _jo->_d;
  k = dictionary_find(d, "cat");
  EXPECT_ULEQ(JSON_FALSE, k->type);
  k = dictionary_find(d, "dog");
  EXPECT_ULEQ(JSON_TRUE, k->type);
  EXPECT_NULL(dictionary_find(d, "pig"));
  EXPECT_ULEQ(2, dictionary_cardinality(d));
  dictionary_pairs_new(&_pairs, d, true);
  EXPECT_ULEQ(2, _pairs->cardinality);
  pair = _pairs->_pairs[0];
  EXPECT_STREQ("dog", pair->key);
  k = pair->object;
  EXPECT_ULEQ(JSON_TRUE, k->type);
  pair = _pairs->_pairs[1];
  EXPECT_STREQ("cat", pair->key);
  k = pair->object;
  EXPECT_ULEQ(JSON_FALSE, k->type);
  dictionary_pairs_delete(&_pairs);
  json_object_new_copy(&_jo2, _jo);
  json_object_delete(&_jo);
  d = _jo2->_d;
  dictionary_pairs_new(&_pairs, d, true);
  EXPECT_ULEQ(2, _pairs->cardinality);
  pair = _pairs->_pairs[0];
  EXPECT_STREQ("dog", pair->key);
  k = pair->object;
  EXPECT_ULEQ(JSON_TRUE, k->type);
  pair = _pairs->_pairs[1];
  EXPECT_STREQ("cat", pair->key);
  k = pair->object;
  EXPECT_ULEQ(JSON_FALSE, k->type);
  dictionary_pairs_delete(&_pairs);
  json_object_delete(&_jo2);

  json_object_new_empty(&_jo);
  d = _jo->_d;
  dictionary_pairs_new(&_pairs, d, true);
  EXPECT_ULEQ(0, _pairs->cardinality);
  dictionary_pairs_delete(&_pairs);
  string_new_copy(&_key, "woof");
  json_new_entire(&_j, "[45]");
  json_object_donate(_jo, &_key, &_j);
  EXPECT_ULEQ(1, json_object_cardinality(_jo));
  k = json_object_get(_jo, "woof");
  EXPECT_ULEQ(JSON_ARRAY, k->type);
  json_object_delete(&_jo);

  p = "{\"c\":1,\"b\":{\"\":true,\"E\":[1,2]},\"a\":\"woof\"}";
  CALL(check_(
      p,
      "{\"c\":1,\"b\":{\"\":true,\"E\":[1,2]},\"a\":\"woof\"}",
      "{\n"
      "  \"c\": 1,\n"
      "  \"b\": {\n"
      "    \"\": true,\n"
      "    \"E\": [\n"
      "      1,\n"
      "      2\n"
      "    ]\n"
      "  },\n"
      "  \"a\": \"woof\"\n"
      "}"));
  json_new(&_j, &p);
  EXPECT_ULEQ(JSON_OBJECT, _j->type);
  EXPECT_NULL(_j->_e);
  o = json_get_json_object(_j);
  d = o->_d;
  k = dictionary_find(d, "c");
  EXPECT_ULEQ(JSON_NUMBER, k->type);
  n = json_get_json_number(k);
  EXPECT_STREQ("1", n->_s);
  k = dictionary_find(d, "b");
  EXPECT_ULEQ(JSON_OBJECT, k->type);
  k = dictionary_find(d, "a");
  EXPECT_ULEQ(JSON_STRING, k->type);
  s = json_get_json_string(k);
  EXPECT_STREQ("woof", s->_s);
  EXPECT_NULL(dictionary_find(d, "pig"));
  EXPECT_ULEQ(3, dictionary_cardinality(d));
  dictionary_pairs_new(&_pairs, d, true);
  EXPECT_ULEQ(3, _pairs->cardinality);
  pair = _pairs->_pairs[0];
  EXPECT_STREQ("c", pair->key);
  k = pair->object;
  EXPECT_ULEQ(JSON_NUMBER, k->type);
  n = json_get_json_number(k);
  EXPECT_STREQ("1", n->_s);
  EXPECT_STREQ("", p);
  pair = _pairs->_pairs[1];
  EXPECT_STREQ("b", pair->key);
  k = pair->object;
  EXPECT_ULEQ(JSON_OBJECT, k->type);
  pair = _pairs->_pairs[2];
  EXPECT_STREQ("a", pair->key);
  k = pair->object;
  EXPECT_ULEQ(JSON_STRING, k->type);
  s = (JsonString *)k->_o;
  EXPECT_STREQ("woof", s->_s);
  dictionary_pairs_delete(&_pairs);
  json_delete(&_j);
}

TEST(Json, Copy) {
  char *_s = null;
  Json *_j = null;
  Json *_j2 = null;

  json_new_empty(&_j);
  EXPECT_ULEQ(JSON_EMPTY, _j->type);
  json_new_copy(&_j2, _j);
  EXPECT_ULEQ(JSON_EMPTY, _j2->type);
  json_delete(&_j);
  EXPECT_ULEQ(JSON_EMPTY, _j2->type);
  json_delete(&_j2);

  json_new_entire(&_j, "null");
  EXPECT_ULEQ(JSON_NULL, _j->type);
  json_new_copy(&_j2, _j);
  json_delete(&_j);
  EXPECT_ULEQ(JSON_NULL, _j2->type);
  json_delete(&_j2);

  json_new_entire(&_j, "true");
  EXPECT_ULEQ(JSON_TRUE, _j->type);
  json_new_copy(&_j2, _j);
  json_delete(&_j);
  EXPECT_ULEQ(JSON_TRUE, _j2->type);
  json_delete(&_j2);

  json_new_entire(&_j, "false");
  EXPECT_ULEQ(JSON_FALSE, _j->type);
  json_new_copy(&_j2, _j);
  json_delete(&_j);
  EXPECT_ULEQ(JSON_FALSE, _j2->type);
  json_delete(&_j2);

  json_new_entire(&_j, "\"dog\"");
  EXPECT_ULEQ(JSON_STRING, _j->type);
  json_new_copy(&_j2, _j);
  json_delete(&_j);
  EXPECT_ULEQ(JSON_STRING, _j2->type);
  string_new_json(&_s, _j2, false, 0);
  EXPECT_STREQ("\"dog\"", _s);
  string_delete(&_s);
  json_delete(&_j2);

  json_new_entire(&_j, "-12.34e-56");
  EXPECT_ULEQ(JSON_NUMBER, _j->type);
  json_new_copy(&_j2, _j);
  json_delete(&_j);
  EXPECT_ULEQ(JSON_NUMBER, _j2->type);
  string_new_json(&_s, _j2, false, 0);
  EXPECT_STREQ("-12.34e-56", _s);
  string_delete(&_s);
  json_delete(&_j2);

  json_new_entire(&_j, "[1, \"dog\"]");
  EXPECT_ULEQ(JSON_ARRAY, _j->type);
  json_new_copy(&_j2, _j);
  json_delete(&_j);
  EXPECT_ULEQ(JSON_ARRAY, _j2->type);
  string_new_json(&_s, _j2, false, 0);
  EXPECT_STREQ("[1,\"dog\"]", _s);
  string_delete(&_s);
  json_delete(&_j2);

  json_new_entire(&_j, "{\"cat\": \"dog\"}");
  EXPECT_ULEQ(JSON_OBJECT, _j->type);
  json_new_copy(&_j2, _j);
  json_delete(&_j);
  EXPECT_ULEQ(JSON_OBJECT, _j2->type);
  string_new_json(&_s, _j2, false, 0);
  EXPECT_STREQ("{\"cat\":\"dog\"}", _s);
  string_delete(&_s);
  json_delete(&_j2);

  json_new_entire(&_j, "{\"cat\": [1, null, \"dog\", {\"a\": false}]}");
  EXPECT_ULEQ(JSON_OBJECT, _j->type);
  json_new_copy(&_j2, _j);
  json_delete(&_j);
  EXPECT_ULEQ(JSON_OBJECT, _j2->type);
  string_new_json(&_s, _j2, false, 0);
  EXPECT_STREQ("{\"cat\":[1,null,\"dog\",{\"a\":false}]}", _s);
  string_delete(&_s);
  json_delete(&_j2);
}

TEST(Json, Write) {
  CALL(check_("\n", "", ""));
  CALL(check_(" null", "null", "null"));
  CALL(check_("true\r", "true", "true"));
  CALL(check_("false  ", "false", "false"));
  CALL(check_("\t\"cat\"", "\"cat\"", "\"cat\""));
  CALL(check_("-12.3E-0046", "-12.3e-0046", "-12.3e-0046"));
  CALL(check_(
      "[1,2,null,\"cat\",false,{\"a\":5}]",
      "[1,2,null,\"cat\",false,{\"a\":5}]",
      "[\n"
      "  1,\n"
      "  2,\n"
      "  null,\n"
      "  \"cat\",\n"
      "  false,\n"
      "  {\n"
      "    \"a\": 5\n"
      "  }\n"
      "]"));
  CALL(check_(
      "{\"\":1,\"dog\":null,\"cat\":[1,2,3]}",
      "{\"\":1,\"dog\":null,\"cat\":[1,2,3]}",
      "{\n"
      "  \"\": 1,\n"
      "  \"dog\": null,\n"
      "  \"cat\": [\n"
      "    1,\n"
      "    2,\n"
      "    3\n"
      "  ]\n"
      "}"));
}

TEST(Json, Extract) {
  char *_s = null;
  Json *_j = null;
  Json *_m = null;

  /* Extracting from empty should be empty, regardless of path.
   */
  json_new_entire(&_m, "");
  json_new_extract(&_j, _m, "");
  EXPECT_TRUE(json_is_empty(_j));
  json_delete(&_j);
  json_delete(&_m);

  /* Simple extraction, testing path syntax errors.
   */
  json_new_entire(&_m, "null");
  json_new_extract(&_j, _m, "");
  EXPECT_STREQ(
      "Json: Expected path to begin with '$', but got '':\n"
      ". Expected '$' but got '': error at offset 0",
      _j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$");
  EXPECT_TRUE(json_is_null(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$.");
  EXPECT_TRUE(json_is_unknown(_j));
  EXPECT_STREQ(
      "Json: Expected key string after '$.' in path but got '' in '$.'",
      _j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$.0day");
  EXPECT_TRUE(json_is_unknown(_j));
  EXPECT_STREQ(
      "Json: Key string after '$.' in path cannot start with digit: "
      "got '0day' in '$.0day'",
      _j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[");
  EXPECT_TRUE(json_is_unknown(_j));
  EXPECT_STREQ(
      "Json: Expected quote (') or integer after '[' in path "
      "but got '' in '$['",
      _j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$['");
  EXPECT_TRUE(json_is_unknown(_j));
  EXPECT_STREQ(
      "Json: String ended before quoted key string in path was complete: "
      "got '' in '$[''",
      _j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$['a");
  EXPECT_TRUE(json_is_unknown(_j));
  EXPECT_STREQ(
      "Json: String ended before quoted key string in path was complete: "
      "got 'a' in '$['a'",
      _j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$['abc'");
  EXPECT_TRUE(json_is_unknown(_j));
  EXPECT_STREQ(
      "Json: Didn't get closing ']' after key string 'abc' in path: "
      "got '' in '$['abc''",
      _j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[-");
  EXPECT_TRUE(json_is_unknown(_j));
  EXPECT_STREQ(
      "Json: String ended before integer key in path was complete: "
      "got '-' in '$[-'",
      _j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[-1");
  EXPECT_TRUE(json_is_unknown(_j));
  EXPECT_STREQ(
      "Json: String ended before integer key in path was complete: "
      "got '-1' in '$[-1'",
      _j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[01]");
  EXPECT_TRUE(json_is_unknown(_j));
  EXPECT_STREQ(
      "Json: Invalid digit after leading '0' in integer key in path: "
      "got '01]' in '$[01]'",
      _j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[-01]");
  EXPECT_TRUE(json_is_unknown(_j));
  EXPECT_STREQ(
      "Json: Invalid digit after leading '0' in integer key in path: "
      "got '-01]' in '$[-01]'",
      _j->_e);
  json_delete(&_j);

  /* Extracting from a scalar should return an empty result.
   */
  json_new_extract(&_j, _m, "$.dog");
  EXPECT_TRUE(json_is_empty(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$['dog']");
  EXPECT_TRUE(json_is_empty(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[0]");
  EXPECT_TRUE(json_is_empty(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[12]");
  EXPECT_TRUE(json_is_empty(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[-5]");
  EXPECT_TRUE(json_is_empty(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);
  json_delete(&_m);

  /* Extract from a simple object, testing many empty result sets.
   */
  json_new_entire(&_m, "{\"dog\":7}");
  json_new_extract(&_j, _m, "$.cat");
  EXPECT_TRUE(json_is_empty(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$.dog");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("7", _s);
  string_delete(&_s);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$.dog.cat");
  EXPECT_TRUE(json_is_empty(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$['dog']");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("7", _s);
  string_delete(&_s);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$['dog']['cat']");
  EXPECT_TRUE(json_is_empty(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$['dog'].cat");
  EXPECT_TRUE(json_is_empty(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$.dog['cat']");
  EXPECT_TRUE(json_is_empty(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);
  json_delete(&_m);

  /* Extract from a two-level object.
   */
  json_new_entire(&_m, "{\"dog\":{\"cat\":true}}");
  json_new_extract(&_j, _m, "$.cat");
  EXPECT_TRUE(json_is_empty(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$.dog");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("{\"cat\":true}", _s);
  string_delete(&_s);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$.dog.cat");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("true", _s);
  string_delete(&_s);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$['dog']");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("{\"cat\":true}", _s);
  string_delete(&_s);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$['dog']['cat']");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("true", _s);
  string_delete(&_s);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$['dog'].cat");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("true", _s);
  string_delete(&_s);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$.dog['cat']");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("true", _s);
  string_delete(&_s);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);
  json_delete(&_m);

  /* Extract from a simple array.
   */
  json_new_entire(&_m, "[\"cat\",7,true,null]");
  json_new_extract(&_j, _m, "$[0]");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("\"cat\"", _s);
  string_delete(&_s);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[1]");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("7", _s);
  string_delete(&_s);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[2]");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("true", _s);
  string_delete(&_s);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[3]");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("null", _s);
  string_delete(&_s);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[-1]");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("null", _s);
  string_delete(&_s);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[-2]");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("true", _s);
  string_delete(&_s);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[-4]");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("\"cat\"", _s);
  string_delete(&_s);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[4]");
  EXPECT_TRUE(json_is_empty(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[5]");
  EXPECT_TRUE(json_is_empty(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[7]");
  EXPECT_TRUE(json_is_empty(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[8]");
  EXPECT_TRUE(json_is_empty(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[-5]");
  EXPECT_TRUE(json_is_empty(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[-8]");
  EXPECT_TRUE(json_is_empty(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[-9]");
  EXPECT_TRUE(json_is_empty(_j));
  EXPECT_NULL(_j->_e);
  json_delete(&_j);
  json_delete(&_m);

  /* Extract from a complex object.
   */
  json_new_entire(&_m, "[1,\"cat\",{\"dog\":[1,null,{\"a\":-1.3e-4}]},0]");
  json_new_extract(&_j, _m, "$[-2].dog[2]['a']");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("-1.3e-4", _s);
  string_delete(&_s);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[-2].dog[2]");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("{\"a\":-1.3e-4}", _s);
  string_delete(&_s);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[-2].dog");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("[1,null,{\"a\":-1.3e-4}]", _s);
  string_delete(&_s);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$[-2]");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("{\"dog\":[1,null,{\"a\":-1.3e-4}]}", _s);
  string_delete(&_s);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("[1,\"cat\",{\"dog\":[1,null,{\"a\":-1.3e-4}]},0]", _s);
  string_delete(&_s);
  EXPECT_NULL(_j->_e);
  json_delete(&_j);

  /* Check that path errors bubble up.
   */
  json_new_extract(&_j, _m, "$[-2].dog[2]['a'");
  EXPECT_TRUE(json_is_unknown(_j));
  EXPECT_STREQ(
      "Json: Error in sub-extract for path '$[-2].dog[2]['a'':\n"
      ". Json: Error in sub-extract for path '$.dog[2]['a'':\n"
      ". . Json: Error in sub-extract for path '$[2]['a'':\n"
      ". . . Json: Didn't get closing ']' after key string 'a' in path: "
      "got '' in '$['a''",
      _j->_e);
  json_delete(&_j);
  json_delete(&_m);

  /* Test recursive extract.
   */
  json_new_entire(
      &_m,
      "{\"cat\":{\"dog\":1,\"cat\":2,\"pig\":{\"dog\":7}},"
      "\"pig\":{\"cat\":8,\"dog\":9}}");
  json_new_extract(&_j, _m, "$..dog");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("[1,7,9]", _s);
  string_delete(&_s);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$..cat");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("[{\"dog\":1,\"cat\":2,\"pig\":{\"dog\":7}},8]", _s);
  string_delete(&_s);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$..cat..cat");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("[[2]]", _s);
  string_delete(&_s);
  json_delete(&_j);

  /* Check errors in recursive spec.
   */
  json_new_extract(&_j, _m, "$..");
  EXPECT_STREQ(
      "Json: Expected key string after '$..' in path but got '' in '$..'",
      _j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$..[");
  EXPECT_STREQ(
      "Json: Expected key string after '$..' in path but got '[' in '$..['",
      _j->_e);
  json_delete(&_j);

  /* Check that errors after recursive bubble up.
   */
  json_new_extract(&_j, _m, "$..dog.");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("[]", _s);
  string_delete(&_s);
  EXPECT_STREQ(
      "Json: Error in sub-extract for path '$..dog.':\n"
      ". Json: Error in sub-extract for path '$..dog.':\n"
      ". . Json: Expected key string after '$.' in path but got '' in '$.'",
      _j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$..dog[");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("[]", _s);
  string_delete(&_s);
  EXPECT_STREQ(
      "Json: Error in sub-extract for path '$..dog[':\n"
      ". Json: Error in sub-extract for path '$..dog[':\n"
      ". . Json: Expected quote (') or integer after '[' in path "
      "but got '' in '$['",
      _j->_e);
  json_delete(&_j);

  json_new_extract(&_j, _m, "$..dog[-1");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("[]", _s);
  string_delete(&_s);
  EXPECT_STREQ(
      "Json: Error in sub-extract for path '$..dog[-1':\n"
      ". Json: Error in sub-extract for path '$..dog[-1':\n"
      ". . Json: String ended before integer key in path was complete: "
      "got '-1' in '$[-1'",
      _j->_e);
  json_delete(&_j);
  json_delete(&_m);

  /* Check further path after recursive.
   */
  json_new_entire(
      &_m,
      "{\"cat\":{\"dog\":{\"score\":56}},"
      "\"a\":{\"b\":{\"dog\":{\"score\":12}},\"dog\":null},"
      "\"dog\":{\"big\":true,\"score\":3}}");
  json_new_extract(&_j, _m, "$..dog.score");
  string_new_json(&_s, _j, false, 0);
  EXPECT_STREQ("[56,12,3]", _s);
  string_delete(&_s);
  json_delete(&_j);
  json_delete(&_m);
}

FINISH


static void check_(char *s, char *e, char *pretty) {
  char *_s2 = null;
  char *_s3 = null;
  char *_s4 = null;
  char *_s5 = null;
  char *_s6 = null;
  Json *_j = null;
  Json *_j2 = null;
  Json *_j3 = null;
  File *_f = null;

  json_new_entire(&_j, s);
  EXPECT_NULL(_j->_e);
  string_new_json(&_s2, _j, false, 0);
  EXPECT_STREQ(e, _s2);
  file_new_memory(&_f);
  json_write_file(_j, _f, false, 0);
  string_new_memory_file_contents(&_s5, _f->_mf);
  file_delete(&_f);
  EXPECT_STREQ(e, _s5);
  file_new_memory(&_f);
  json_write_file(_j, _f, true, 0);
  string_new_memory_file_contents(&_s6, _f->_mf);
  file_delete(&_f);
  EXPECT_STREQ(pretty, _s6);
  json_new_entire(&_j2, _s2);
  string_new_json(&_s3, _j2, false, 0);
  EXPECT_STREQ(_s2, _s3);
  EXPECT_STREQ(e, _s3);
  string_delete(&_s3);
  string_new_json(&_s3, _j2, true, 0);
  EXPECT_STREQ(pretty, _s3);
  json_new_entire(&_j3, _s3);
  string_new_json(&_s4, _j2, false, 0);
  EXPECT_STREQ(_s2, _s4);
  string_delete(&_s6);
  string_delete(&_s5);
  string_delete(&_s4);
  string_delete(&_s3);
  string_delete(&_s2);
  json_delete(&_j3);
  json_delete(&_j2);
  json_delete(&_j);
}
