#include "JsonTestDeath.h"

START_DEATH

TEST(Json, Get) {
  Json *_ = null;

  json_new_empty(&_);
  EXPECT_DEATH(
      json_get_json_string(_),
      "Json object is a JsonEmpty, not a JsonString");
  EXPECT_DEATH(
      json_get_json_number(_),
      "Json object is a JsonEmpty, not a JsonNumber");
  EXPECT_DEATH(
      json_get_json_array(_),
      "Json object is a JsonEmpty, not a JsonArray");
  EXPECT_DEATH(
      json_get_json_object(_),
      "Json object is a JsonEmpty, not a JsonObject");
  json_delete(&_);
}

TEST(Json, GetFromUnknown) {
  char *p;
  Json *_ = null;

  p = "pig";
  json_new(&_, &p);
  EXPECT_DEATH(json_get_bool(_), "Could not get bool value from Json ''");
  EXPECT_DEATH(json_get_ulong(_), "Could not get ulong value from Json ''");
  EXPECT_DEATH(json_get_long(_), "Could not get long value from Json ''");
  EXPECT_DEATH(json_get_double(_), "Could not get double value from Json ''");
  EXPECT_DEATH(json_get_string(_), "Could not get string value from Json ''");
  json_delete(&_);
}

TEST(Json, GetUlongLong) {
  char *p;
  Json *_ = null;

  p = " -0";
  json_new(&_, &p);
  EXPECT_DEATH(json_get_ulong(_), "Could not get ulong value from Json '-0'");
  json_delete(&_);

  p = "\n-2";
  json_new(&_, &p);
  EXPECT_DEATH(json_get_ulong(_), "Could not get ulong value from Json '-2'");
  json_delete(&_);

  p = " 01";
  json_new(&_, &p);
  EXPECT_DEATH(json_get_ulong(_), "Could not get ulong value from Json ''");
  EXPECT_DEATH(json_get_long(_), "Could not get long value from Json ''");
  EXPECT_DEATH(json_get_double(_), "Could not get double value from Json ''");
  json_delete(&_);

  p = " -100";
  json_new(&_, &p);
  EXPECT_DEATH(json_get_ulong(_),
      "Could not get ulong value from Json '-100'");
  json_delete(&_);

  p = " \f 0.0";
  json_new(&_, &p);
  EXPECT_DEATH(json_get_ulong(_), "Could not get ulong value from Json '0.0'");
  EXPECT_DEATH(json_get_long(_), "Could not get long value from Json '0.0'");
  json_delete(&_);

  p = " -0.0";
  json_new(&_, &p);
  EXPECT_DEATH(json_get_ulong(_),
      "Could not get ulong value from Json '-0.0'");
  EXPECT_DEATH(json_get_long(_), "Could not get long value from Json '-0.0'");
  json_delete(&_);

  p = " -3.14";
  json_new(&_, &p);
  EXPECT_DEATH(
      json_get_ulong(_),
      "Could not get ulong value from Json '-3.14'");
  EXPECT_DEATH(json_get_long(_), "Could not get long value from Json '-3.14'");
  json_delete(&_);

  p = " -3.14,12";
  json_new(&_, &p);
  EXPECT_DEATH(
      json_get_ulong(_),
      "Could not get ulong value from Json '-3.14'");
  EXPECT_DEATH(json_get_long(_), "Could not get long value from Json '-3.14'");
  json_delete(&_);

  p = " 0e0";
  json_new(&_, &p);
  EXPECT_DEATH(json_get_ulong(_), "Could not get ulong value from Json '0e0'");
  EXPECT_DEATH(json_get_long(_), "Could not get long value from Json '0e0'");
  json_delete(&_);

  p = " 1E4";
  json_new(&_, &p);
  EXPECT_DEATH(json_get_ulong(_), "Could not get ulong value from Json '1e4'");
  EXPECT_DEATH(json_get_long(_), "Could not get long value from Json '1e4'");
  json_delete(&_);

  p = " 1.2E4";
  json_new(&_, &p);
  EXPECT_DEATH(
      json_get_ulong(_),
      "Could not get ulong value from Json '1.2e4'");
  EXPECT_DEATH(
      json_get_long(_),
      "Could not get long value from Json '1.2e4'");
  json_delete(&_);

  p = " -1.2e4";
  json_new(&_, &p);
  EXPECT_DEATH(
      json_get_ulong(_),
      "Could not get ulong value from Json '-1.2e4'");
  EXPECT_DEATH(
      json_get_long(_),
      "Could not get long value from Json '-1.2e4'");
  json_delete(&_);

  p = " -1.23e-4";
  json_new(&_, &p);
  EXPECT_DEATH(
      json_get_ulong(_),
      "Could not get ulong value from Json '-1.23e-4'");
  EXPECT_DEATH(
      json_get_long(_),
      "Could not get long value from Json '-1.23e-4'");
  json_delete(&_);

  p = " -1.23e+004";
  json_new(&_, &p);
  EXPECT_DEATH(
      json_get_ulong(_),
      "Could not get ulong value from Json '-1.23e+004'");
  EXPECT_DEATH(
      json_get_long(_),
      "Could not get long value from Json '-1.23e+004'");
  json_delete(&_);

  p = " -0.23e+0047";
  json_new(&_, &p);
  EXPECT_DEATH(
      json_get_ulong(_),
      "Could not get ulong value from Json '-0.23e+0047'");
  EXPECT_DEATH(
      json_get_long(_),
      "Could not get long value from Json '-0.23e+0047'");
  json_delete(&_);

  p = " 1dog";
  json_new(&_, &p);
  EXPECT_DEATH(json_get_ulong(_), "Could not get ulong value from Json ''");
  EXPECT_DEATH(json_get_long(_), "Could not get long value from Json ''");
  EXPECT_DEATH(json_get_double(_), "Could not get double value from Json ''");
  json_delete(&_);

  p = " -1.2e-4e-5";
  json_new(&_, &p);
  EXPECT_DEATH(json_get_ulong(_), "Could not get ulong value from Json ''");
  EXPECT_DEATH(json_get_long(_), "Could not get long value from Json ''");
  EXPECT_DEATH(json_get_double(_), "Could not get double value from Json ''");
  json_delete(&_);
}

TEST(Json, AppendArray) {
  Json *_j1 = null;
  Json *_j2 = null;

  json_new_entire(&_j1, "dog");
  json_new_entire(&_j2, "cat");
  EXPECT_DEATH(
      json_append_array(_j1, &_j2),
      "Can only append to an array");
  EXPECT_DEATH(
      json_append_array_with_array(_j1, &_j2),
      "Can only append to an array");
  json_delete(&_j1);

  json_new_entire(&_j1, "[1]");
  EXPECT_DEATH(
      json_append_array_with_array(_j1, &_j2),
      "Can only append an array to an array");
  json_delete(&_j1);

  json_delete(&_j2);
}

TEST(Json, AddToObject) {
  char *_key = null;
  Json *_j1 = null;
  Json *_j2 = null;

  json_new_entire(&_j1, "dog");
  json_new_entire(&_j2, "cat");
  string_new_copy(&_key, "woof");
  EXPECT_DEATH(
      json_donate_to_object(_j1, &_key, &_j2),
      "Can only donate to an object");
  json_delete(&_j2);
  json_delete(&_j1);
}

TEST(JsonArray, NewEntire) {
  JsonArray *_ = null;

  EXPECT_DEATH(
      json_array_new_entire(&_, ""),
      "JsonArray: Expected to start with '[':\n"
      ". Expected '[' but got '': error at offset 0");

  json_array_delete(&_);
  EXPECT_DEATH(
      json_array_new_entire(&_, "dog"),
      "Json: Expected to use entire string 'dog' but didn't use 'dog':\n"
      ". JsonArray: Expected to start with '[':\n"
      ". . Expected '[' but got 'dog': error at offset 0");

  json_array_delete(&_);
}

FINISH_DEATH
