#include "KeyValueArrayTest.h"
#include <math.h>

#define NUM_PAIRS_ 10
#define DUMMY_ 24
#define SCALE_1_ 3
#define SCALE_2_ 2
#define TR_SCALE_1_ 2
#define SCALE_2_2_ 5
#define TRANSFORM_SCALE_ 2

#define N_ 100
#define M_ 10000
#define P_ 1000000

typedef struct {
  byte scale_1;
  byte scale_2;
} Pass_;

typedef struct {
  byte scale;
} Pass_1_;

typedef struct {
  ulong dummy;
  byte scale;
} Pass_2_;

typedef struct {
  char *s1;
  char *s2;
  ushort dummy;
} Object_;

typedef struct {
  ushort dummy1;
  char *s1;
  byte dummy2;
  char *s2;
} Object_1_;

typedef struct {
  uint dummy;
  char *s;
} Object_2_1_;

typedef struct {
  byte dummy;
  char *s;
} Object_2_2_;

static void construct_(
    KeyValuePair *pair,
    KeyValueInfo *info,
    byte k0,
    byte k1,
    byte v0);

static void string_(char **p_this, byte len);

static void destroy_(KeyValuePair *pair, KeyValueInfo *info);

static void print_(KeyValuePairPrintInfo *kvppi, Printer *p);

static void merge_(KeyValuePair *acc,KeyValuePair *add, KeyValueInfo *info);

static sbyte compare_(KeyValuePair *a, KeyValuePair *b, KeyValueInfo *info);

static bool get_invalid_(KeyValuePair *pair, KeyValueInfo *info);

static void set_invalid_(KeyValuePair *pair, KeyValueInfo *info);

static void construct_1_(
    KeyValuePair *pair,
    KeyValueInfo *info,
    byte k0,
    byte k1,
    byte v0);

static void destroy_1_(KeyValuePair *pair, KeyValueInfo *info);

static void print_1_(KeyValuePairPrintInfo *kvppi, Printer *p);

static void merge_1_(KeyValuePair *acc, KeyValuePair *add, KeyValueInfo *info);

static sbyte compare_1_(KeyValuePair *a, KeyValuePair *b, KeyValueInfo *info);

static void construct_2_(
    KeyValuePair *pair,
    KeyValueInfo *info,
    byte k0,
    byte k1,
    byte k2,
    byte v0,
    byte v2);

static void destroy_2_(KeyValuePair *pair, KeyValueInfo *info);

static void print_2_(KeyValuePairPrintInfo *kvppi, Printer *p);

static void merge_2_(KeyValuePair *acc, KeyValuePair *add, KeyValueInfo *info);

static sbyte compare_2_(KeyValuePair *a, KeyValuePair *b, KeyValueInfo *info);

static void transform_(
    KeyValuePair *pair_old,
    KeyValueInfo *info_old,
    KeyValuePair *pair_new,
    KeyValueInfo *info_new,
    void *x);

static void transform_2_(
    KeyValuePair *pair_old,
    KeyValuePair *pair_new,
    KeyValueInfo *info,
    void *x);


START

TEST(KeyValueArray, Objects) {
  byte k0;
  byte k1;
  byte v1;
  ulong f;
  ulong max_keys[] = {7, 9, ULONG_MAX};
  ulong max_values[] = {ULONG_MAX, 12, 1};
  Pass_ pass;
  KeyValueInfo *_info = null;
  KeyValuePair *pair;
  KeyValuePair *_pair = null;
  KeyValueArray *_array = null;

  /* Test storing objects in the pairs, making them arbitrarily intricate just
   * for testing purposes. Key 2 contains a pointer to an Object_, in which is
   * created two strings, equal in length to keys 0 and 1 respectively. Value 0
   * contains a string pointer, whose length is scale_1 (from the passthrough
   * object) times value 1, which is supplied. Value 2 is an invalid flag.
   */
  key_value_info_new(&_info, 3, max_keys, 3, max_values);
  _info->x = &pass;
  _info->destroy = destroy_;
  _info->print = print_;
  _info->merge = merge_;
  _info->compare = compare_;
  _info->getInvalid = get_invalid_;
  _info->setInvalid = set_invalid_;

  pass.scale_1 = SCALE_1_;
  pass.scale_2 = SCALE_2_;

  key_value_array_new(&_array, _info, NUM_PAIRS_);
  key_value_pair_new(&_pair, _info);

  k0 = 5;
  k1 = 3;
  v1 = 2;
  construct_(_pair, _info, k0, k1, v1);
  key_value_array_donate(_array, _pair);

  /* Check that donation is working correctly.
   */
  for (f = 0; f < 6; f++) {
    EXPECT_ULEQ(0, key_value_pair_get_field(_pair, _info, f));
  }

  k0 = 2;
  k1 = 9;
  v1 = 11;
  construct_(_pair, _info, k0, k1, v1);
  key_value_array_donate(_array, _pair);

  k0 = 5;
  k1 = 3;
  v1 = 4;
  construct_(_pair, _info, k0, k1, v1);
  key_value_array_donate(_array, _pair);

  key_value_array_finalize(_array);
  pair = key_value_array_get(_array, 0);
  key_value_pair_set_invalid(pair, _info);
  _array->finalized = false;
  key_value_array_finalize(_array);

  pair = key_value_array_get(_array, 0);
  key_value_pair_set_invalid(pair, _info);
  _array->finalized = false;
  key_value_array_finalize(_array);

  key_value_array_delete(&_array);

  key_value_array_new(&_array, _info, NUM_PAIRS_);

  k0 = 5;
  k1 = 3;
  v1 = 2;
  construct_(_pair, _info, k0, k1, v1);
  key_value_array_donate(_array, _pair);

  k0 = 2;
  k1 = 9;
  v1 = 11;
  construct_(_pair, _info, k0, k1, v1);
  key_value_array_donate(_array, _pair);

  k0 = 5;
  k1 = 3;
  v1 = 2;
  construct_(_pair, _info, k0, k1, v1);
  key_value_array_donate(_array, _pair);

  key_value_array_finalize(_array);

  key_value_array_delete(&_array);
  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);
}

TEST(KeyValueArray, SaveLoadNewCopy) {
  ulong max_key = 4;
  KeyValueInfo *_info = null;
  KeyValuePair *g = null;
  KeyValuePair *_pair = null;
  KeyValueArray *_array = null;
  KeyValueArray *_copy = null;

  set_info_new(&_info, 1, &max_key);
  key_value_pair_new(&_pair, _info);
  key_value_array_new(&_array, _info, 4);
  key_value_pair_set_key(_pair, _info, 0, 3);
  key_value_array_donate(_array, _pair);
  key_value_pair_set_key(_pair, _info, 0, 1);
  key_value_array_donate(_array, _pair);
  key_value_pair_set_key(_pair, _info, 0, 4);
  key_value_array_donate(_array, _pair);
  key_value_pair_set_key(_pair, _info, 0, 2);
  key_value_array_donate(_array, _pair);
  key_value_array_finalize(_array);

  test_save_load(KeyValueArray, key_value_array, _array);

  key_value_array_new_copy(&_copy, _array);
  EXPECT_ULEQ(4, _copy->numPairs);
  g = key_value_array_get(_copy, 0);
  EXPECT_ULEQ(1, key_value_pair_get_key(g, _info, 0));
  g = key_value_array_get(_copy, 1);
  EXPECT_ULEQ(2, key_value_pair_get_key(g, _info, 0));
  g = key_value_array_get(_copy, 2);
  EXPECT_ULEQ(3, key_value_pair_get_key(g, _info, 0));
  g = key_value_array_get(_copy, 3);
  EXPECT_ULEQ(4, key_value_pair_get_key(g, _info, 0));
  key_value_array_delete(&_copy);
  key_value_array_delete(&_array);
  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);
}

TEST(KeyValueArray, NewHyperslice) {
  ulong max_key = 4;
  KeyValueInfo *_info = null;
  KeyValuePair *g = null;
  KeyValuePair *_pair = null;
  KeyValueArray *_array = null;
  KeyValueArray *_hyperslice = null;

  key_value_info_new(&_info, 1, &max_key, 0, null);
  key_value_pair_new(&_pair, _info);
  key_value_array_new(&_array, _info, 4);
  key_value_pair_set_key(_pair, _info, 0, 3);
  key_value_array_donate(_array, _pair);
  key_value_pair_set_key(_pair, _info, 0, 1);
  key_value_array_donate(_array, _pair);
  key_value_pair_set_key(_pair, _info, 0, 4);
  key_value_array_donate(_array, _pair);
  key_value_pair_set_key(_pair, _info, 0, 2);
  key_value_array_donate(_array, _pair);
  key_value_array_finalize(_array);
  key_value_array_new_hyperslice(&_hyperslice, _array, 1, 2);
  EXPECT_ULEQ(2, _hyperslice->numPairs);
  g = key_value_array_get(_hyperslice, 0);
  EXPECT_ULEQ(2, key_value_pair_get_key(g, _info, 0));
  g = key_value_array_get(_hyperslice, 1);
  EXPECT_ULEQ(3, key_value_pair_get_key(g, _info, 0));
  key_value_array_delete(&_hyperslice);
  key_value_array_delete(&_array);
  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);
}

TEST(KeyValueArray, NewTransform) {
  byte transform_info;
  ulong max_keys[] = {ULONG_MAX, 13, ULONG_MAX, 5000};
  ulong max_values[] = {9, ULONG_MAX, 0x100000000};
  Pass_1_ pass_1;
  Pass_2_ pass_2;
  KeyValuePair *_pair_1 = null;
  KeyValueInfo *_info_1 = null;
  KeyValueInfo *_info_2 = null;
  KeyValueArray *_array_1 = null;
  KeyValueArray *_array_2 = null;

  pass_1.scale = TR_SCALE_1_;

  key_value_info_new(&_info_1, 2, max_keys, 1, max_values);
  _info_1->x = &pass_1;
  _info_1->destroy = destroy_1_;
  _info_1->print = print_1_;
  _info_1->merge = merge_1_;
  _info_1->compare = compare_1_;

  key_value_array_new(&_array_1, _info_1, 10);
  key_value_pair_new(&_pair_1, _info_1);

  construct_1_(_pair_1, _info_1, 4, 11, 3);
  key_value_array_donate(_array_1, _pair_1);

  construct_1_(_pair_1, _info_1, 3, 4, 1);
  key_value_array_donate(_array_1, _pair_1);

  construct_1_(_pair_1, _info_1, 3, 4, 5);
  key_value_array_donate(_array_1, _pair_1);

  construct_1_(_pair_1, _info_1, 5, 10, 7);
  key_value_array_donate(_array_1, _pair_1);

  max_keys[1] = 1000;
  max_values[0] = 2000;

  key_value_info_new(&_info_2, 3, max_keys, 3, max_values);
  _info_2->x = &pass_2;
  _info_2->destroy = destroy_2_;
  _info_2->print = print_2_;
  _info_2->merge = merge_2_;
  _info_2->compare = compare_2_;

  pass_2.scale = SCALE_2_2_;
  transform_info = TRANSFORM_SCALE_;

  key_value_array_new_transform(
      &_array_2,
      _array_1,
      _info_2,
      transform_,
      &transform_info);

  key_value_array_delete(&_array_2);
  key_value_array_finalize(_array_1);

  key_value_array_new_transform(
      &_array_2,
      _array_1,
      _info_2,
      transform_,
      &transform_info);

  key_value_array_delete(&_array_2);
  key_value_array_delete(&_array_1);
  key_value_pair_delete(&_pair_1, _info_1);
  key_value_info_delete(&_info_2);
  key_value_info_delete(&_info_1);
}

TEST(KeyValueArray, Print) {
  ulong max_key = 9;
  ulong max_value = 6;
  KeyValueInfo *_info = null;
  KeyValuePair *_pair = null;
  KeyValueArray *_array = null;

  key_value_info_new(&_info, 1, &max_key, 1, &max_value);
  key_value_array_new(&_array, _info, 7);
  key_value_pair_new(&_pair, _info);
  key_value_pair_set_key(_pair, _info, 0, 7);
  key_value_pair_set_value(_pair, _info, 0, 3);
  key_value_array_donate(_array, _pair);
  key_value_pair_set_key(_pair, _info, 0, 4);
  key_value_pair_set_value(_pair, _info, 0, 1);
  key_value_array_donate(_array, _pair);

  key_value_array_finalize(_array);

  key_value_pair_delete(&_pair, _info);
  key_value_array_delete(&_array);
  key_value_info_delete(&_info);
}

TEST(KeyValueArray, TryMergeKeyValuePair) {
  ulong max_key = 9;
  ulong max_value = 6;
  KeyValueInfo *_info = null;
  KeyValuePair *_pair = null;
  KeyValueArray *_array = null;

  key_value_info_new(&_info, 1, &max_key, 1, &max_value);
  key_value_array_new(&_array, _info, 7);
  key_value_pair_new(&_pair, _info);
  key_value_pair_set_key(_pair, _info, 0, 7);
  key_value_pair_set_value(_pair, _info, 0, 3);
  key_value_array_donate(_array, _pair);
  key_value_pair_set_key(_pair, _info, 0, 4);
  key_value_pair_set_value(_pair, _info, 0, 1);
  key_value_array_donate(_array, _pair);
  key_value_array_finalize(_array);
  key_value_pair_set_key(_pair, _info, 0, 4);
  key_value_pair_set_value(_pair, _info, 0, 1);
  EXPECT_TRUE(key_value_array_try_merge_pair(_array, _pair));
  EXPECT_ULEQ(4, key_value_pair_get_key(_pair, _info, 0));
  key_value_pair_set_key(_pair, _info, 0, 3);
  EXPECT_FALSE(key_value_array_try_merge_pair(_array, _pair));
  EXPECT_ULEQ(3, key_value_pair_get_key(_pair, _info, 0));
  key_value_pair_delete(&_pair, _info);
  key_value_array_delete(&_array);
  key_value_info_delete(&_info);
}

TEST(KeyValueArray, SortZeroWidth) {
  ulong max;
  KeyValuePair *_pair = null;
  KeyValueInfo *_info = null;
  KeyValueArray *_array = null;

  /* Tests the case of sorting when the pairs have zero width, i.e. any fields
   * have maximum value of zero, so nothing needs to actually be stored, and
   * the data array is of zero bytes. Would hang if passed to the sorting
   * library, if the latter did not throw for zero element width. This might
   * seem to be a pathological case, but it is possible in principle and there
   * is no reason for the library to reject it.
   */
  max = 0;
  key_value_info_new(&_info, 1, &max, 0, null);
  key_value_array_new(&_array, _info, 2);
  key_value_pair_new(&_pair, _info);
  key_value_array_donate(_array, _pair);
  key_value_array_donate(_array, _pair);
  key_value_array_finalize(_array);
  key_value_array_delete(&_array);
  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);
}

TEST(KeyValueArray, SortNullMergeFunction) {
  ulong max;
  KeyValuePair *_pair = null;
  KeyValueInfo *_info = null;
  KeyValueArray *_array = null;

  /* Tests the case of sorting when the merge function is null.
   */
  max = 1;
  key_value_info_new(&_info, 1, &max, 0, null);
  key_value_array_new(&_array, _info, 2);
  key_value_pair_new(&_pair, _info);
  key_value_array_donate(_array, _pair);
  key_value_array_donate(_array, _pair);
  key_value_array_finalize(_array);
  key_value_array_delete(&_array);
  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);
}

TEST(KeyValueArray, Merge) {
  ulong max = 6;
  KeyValuePair *g = null;
  KeyValuePair *_pair = null;
  KeyValueInfo *_info = null;
  KeyValueArray *_array = null;
  KeyValueArray *_array2 = null;

  key_value_info_new(&_info, 1, &max, 0, null);
  key_value_pair_new(&_pair, _info);

  key_value_array_new(&_array, _info, 7);
  key_value_pair_set_key(_pair, _info, 0, 6);
  key_value_array_donate(_array, _pair);
  key_value_pair_set_key(_pair, _info, 0, 2);
  key_value_array_donate(_array, _pair);
  key_value_array_new(&_array2, _info, 13);
  key_value_pair_set_key(_pair, _info, 0, 1);
  key_value_array_donate(_array, _pair);
  key_value_pair_set_key(_pair, _info, 0, 5);
  key_value_array_donate(_array, _pair);
  key_value_pair_set_key(_pair, _info, 0, 3);
  key_value_array_donate(_array, _pair);
  key_value_array_finalize(_array);
  key_value_array_finalize(_array2);
  key_value_array_merge(_array, _array2);
  EXPECT_ULEQ(5, _array->numPairs);
  g = key_value_array_get(_array, 0);
  EXPECT_ULEQ(1, key_value_pair_get_key(g, _info, 0));
  g = key_value_array_get(_array, 1);
  EXPECT_ULEQ(2, key_value_pair_get_key(g, _info, 0));
  g = key_value_array_get(_array, 2);
  EXPECT_ULEQ(3, key_value_pair_get_key(g, _info, 0));
  g = key_value_array_get(_array, 3);
  EXPECT_ULEQ(5, key_value_pair_get_key(g, _info, 0));
  g = key_value_array_get(_array, 4);
  EXPECT_ULEQ(6, key_value_pair_get_key(g, _info, 0));
  key_value_array_delete(&_array2);
  key_value_array_delete(&_array);

  key_value_array_new(&_array, _info, 7);
  key_value_pair_set_key(_pair, _info, 0, 6);
  key_value_array_donate(_array, _pair);
  key_value_pair_set_key(_pair, _info, 0, 2);
  key_value_array_donate(_array, _pair);
  key_value_array_new(&_array2, _info, 13);
  key_value_pair_set_key(_pair, _info, 0, 2);
  key_value_array_donate(_array, _pair);
  key_value_pair_set_key(_pair, _info, 0, 6);
  key_value_array_donate(_array, _pair);
  key_value_pair_set_key(_pair, _info, 0, 3);
  key_value_array_donate(_array, _pair);
  key_value_array_finalize(_array);
  key_value_array_finalize(_array2);
  key_value_array_merge(_array, _array2);
  EXPECT_ULEQ(3, _array->numPairs);
  g = key_value_array_get(_array, 0);
  EXPECT_ULEQ(2, key_value_pair_get_key(g, _info, 0));
  g = key_value_array_get(_array, 1);
  EXPECT_ULEQ(3, key_value_pair_get_key(g, _info, 0));
  g = key_value_array_get(_array, 2);
  EXPECT_ULEQ(6, key_value_pair_get_key(g, _info, 0));
  key_value_array_delete(&_array2);
  key_value_array_delete(&_array);

  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);
}

TEST(KeyValueArray, Comprehensive) {
  ulong expected_num_finalized_records;
  ulong frequency;
  ulong i;
  ulong index_found;
  ulong j;
  ulong k;
  ulong key;
  ulong max_frequency;
  ulong num_keys;
  ulong num_pairs;
  ulong single_key;
  ulong *_max_keys = null;
  KeyValueInfo *_info = null;
  KeyValueArray *_array = null;
  NaiveKeyValueCollection *_ncollection = null;
  KeyValuePair *_pair = null;
  KeyValuePair *pairj;
  KeyValuePair *_pairf = null;
  KeyValuePair *pair_found;
  Random *_random = null;
  sbyte (*compare)(KeyValuePair *a, KeyValuePair *b, KeyValueInfo *info);

  random_new(&_random);

  for (i = 0; i < N_; i++) {
    num_pairs = random_modulo(_random, M_);
    num_keys = random_modulo(_random, 5);
    new(&_max_keys, num_keys);

    for (k = 0; k < num_keys; k++) {
      _max_keys[k] = random_modulo(
          _random,
          (uint)pow(num_pairs, 1.0 / num_keys) + 1) + 1;
    }

    max_frequency = random_modulo(_random, P_);

    /* The immutable-size memory is that of the KeyValueArray struct plus that
     * of the KeyValueInfo, tested separately.
     */
    key_frequency_info_new(
        &_info,
        num_keys,
        _max_keys,
        num_pairs * max_frequency);
    compare = _info->compare;

    key_value_array_new(&_array, _info, num_pairs);

    naive_key_value_collection_new(&_ncollection, _info, true);
    key_value_pair_new(&_pair, _info);
    key_value_pair_new(&_pairf, _info);
    expected_num_finalized_records = 0;

    for (j = 0; j < num_pairs; j++) {
      for (k = 0; k < num_keys; k++) {
        key = random_modulo(_random, _max_keys[k] + 1);
        key_value_pair_set_key(_pair, _info, k, key);
      }

      frequency = random_modulo(_random, max_frequency + 1) + 1;
      key_frequency_pair_set_frequency(_pair, _info, frequency);

      if (!naive_key_value_collection_get_value(_ncollection, _pair, 0)) {
        expected_num_finalized_records++;
      }

      naive_key_value_collection_add_values(_ncollection, _pair);
      key_value_array_donate(_array, _pair);
    }

    EXPECT_TRUE(key_value_array_is_full(_array));
    key_value_array_finalize(_array);
    key_value_array_verify_finalized(_array);

    EXPECT_ULEQ(expected_num_finalized_records, _array->numPairs);
    EXPECT_ULEQ(
        expected_num_finalized_records,
        key_value_array_cardinality(_array));

    for (j = 0, single_key = 0; j < _array->numPairs; j++, single_key++) {
      while (_ncollection->_values[0][single_key] == 0) {
        naive_key_value_collection_pair_from_single_key(
            _ncollection,
            single_key,
            _pairf);
        pair_found = key_value_array_find(_array, _pairf);
        EXPECT_NULL(pair_found);
        single_key++;
      }

      pairj = key_value_array_get(_array, j);
      frequency = key_frequency_pair_get_frequency(pairj, _info);
      EXPECT_ULEQ(frequency, _ncollection->_values[0][single_key]);
      naive_key_value_collection_pair_from_single_key(
          _ncollection,
          single_key,
          _pairf);
      pair_found = key_value_array_find(_array, _pairf);
      EXPECT_TRUE(pair_found != null);
      EXPECT_ULEQ(0, compare(pairj, pair_found, _info));
      EXPECT_TRUE(key_value_array_try_find_index(
            _array,
            _pairf,
            &index_found));
      EXPECT_ULEQ(j, index_found);
    }

    key_value_pair_delete(&_pair, _info);
    key_value_pair_delete(&_pairf, _info);
    key_value_array_delete(&_array);
    naive_key_value_collection_delete(&_ncollection);
    key_value_info_delete(&_info);
    deletev(&_max_keys);
  }

  random_delete(&_random);
}

TEST(KeyValueArray, Empty) {
  ulong max = 9;
  KeyValueInfo *_info = null;
  KeyValueArray *_array = null;

  set_info_new(&_info, 1, &max);
  key_value_array_new(&_array, _info, 10);
  key_value_array_finalize(_array);
  EXPECT_ULEQ(0, key_value_array_cardinality(_array));
  EXPECT_TRUE(key_value_array_is_empty(_array));
  key_value_array_delete(&_array);
  key_value_info_delete(&_info);
}

TEST(KeyValueArray, CardinalityIsEmptyGetMinimumMaximum) {
  ulong max = 9;
  KeyValuePair *pair;
  KeyValuePair *_pair = null;
  KeyValueInfo *_info = null;
  KeyValueArray *_array = null;

  set_info_new(&_info, 1, &max);
  key_value_pair_new(&_pair, _info);
  key_value_array_new(&_array, _info, 10);
  key_value_pair_set_key(_pair, _info, 0, 7);
  key_value_array_donate(_array, _pair);
  key_value_pair_set_key(_pair, _info, 0, 1);
  key_value_array_donate(_array, _pair);
  key_value_pair_set_key(_pair, _info, 0, 3);
  key_value_array_donate(_array, _pair);
  key_value_array_finalize(_array);
  EXPECT_ULEQ(3, key_value_array_cardinality(_array));
  EXPECT_FALSE(key_value_array_is_empty(_array));
  pair = key_value_array_minimum(_array);
  EXPECT_ULEQ(1, key_value_pair_get_key(pair, _info, 0));
  pair = key_value_array_maximum(_array);
  EXPECT_ULEQ(7, key_value_pair_get_key(pair, _info, 0));
  key_value_array_delete(&_array);
  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);
}

TEST(KeyValueArray, Transform) {
  byte k0;
  byte k1;
  byte loop;
  byte v1;
  uint transform_info;
  ulong max_keys[] = {7, 9, ULONG_MAX};
  ulong max_values[] = {ULONG_MAX, 35, 1};
  Pass_ pass;
  KeyValuePair *_pair = null;
  KeyValueInfo *_info = null;
  KeyValueArray *_array = null;

  for (loop = 0; loop < 2; loop++) {
    key_value_info_new(&_info, 3, max_keys, 3, max_values);
    _info->x = &pass;
    _info->destroy = destroy_;
    _info->print = print_;
    _info->merge = merge_;
    _info->compare = compare_;
    _info->getInvalid = get_invalid_;
    _info->setInvalid = set_invalid_;

    pass.scale_1 = SCALE_1_;
    pass.scale_2 = SCALE_2_;

    key_value_array_new(&_array, _info, NUM_PAIRS_);
    key_value_pair_new(&_pair, _info);

    k0 = 5;
    k1 = 3;
    v1 = 2;
    construct_(_pair, _info, k0, k1, v1);
    key_value_array_donate(_array, _pair);

    k0 = 4;
    k1 = 2;
    v1 = 11;
    construct_(_pair, _info, k0, k1, v1);
    key_value_array_donate(_array, _pair);

    k0 = 5;
    k1 = 3;
    v1 = 4;
    construct_(_pair, _info, k0, k1, v1);
    key_value_array_donate(_array, _pair);

    if (loop == 1) {
      key_value_array_finalize(_array);
    }

    transform_info = TRANSFORM_SCALE_;

    key_value_array_transform(
        _array,
        transform_2_,
        &transform_info);

    key_value_array_delete(&_array);
    key_value_pair_delete(&_pair, _info);
    key_value_info_delete(&_info);
  }
}

TEST(KeyValueArray, SetFull) {
  ulong max = 34;
  KeyValueInfo *_info = null;
  KeyValueArray *_array = null;

  set_info_new(&_info, 1, &max);
  key_value_array_new(&_array, _info, 7);
  key_value_info_delete(&_info);
  EXPECT_FALSE(key_value_array_is_full(_array));
  key_value_array_set_full(_array);
  EXPECT_TRUE(key_value_array_is_full(_array));
  key_value_array_delete(&_array);
}

FINISH


static void construct_(
    KeyValuePair *pair,
    KeyValueInfo *info,
    byte k0,
    byte k1,
    byte v1) {
  char *_s = null;
  Pass_ *pass;
  Object_ *_o = null;

  key_value_pair_set_key(pair, info, 0, k0);
  key_value_pair_set_key(pair, info, 1, k1);

  pass = info->x;

  newo(&_o);
  _o->dummy = DUMMY_;
  string_(&_o->s1, k0);
  string_(&_o->s2, k1);
  key_value_pair_donate_to_key(pair, info, 2, &_o);
  string_(&_s, v1 * pass->scale_1);
  key_value_pair_donate_to_value(pair, info, 0, &_s);
  key_value_pair_set_value(pair, info, 1, v1);
}

static void string_(char **p_this, byte len) {
  byte u;
  char *this;

  new(p_this, len + 1);
  this = *p_this;

  this[len] = '\0';

  for (u = 0; u < len; u++) {
    this[u] = 'a' + modulo(u, 26);
  }
}

static void destroy_(KeyValuePair *pair, KeyValueInfo *info) {
  char *_s = null;
  Pass_ *pass;
  Object_ *_o = null;

  pass = info->x;

  if (pass->scale_1 != SCALE_1_ || pass->scale_2 != SCALE_2_) {
    throw("Impossible");
  }

  if (!key_value_pair_try_donate_from_key(pair, info, 2, &_o)) {
    return;
  }

  if (_o->dummy != DUMMY_) {
    throw("Impossible");
  }

  string_delete(&_o->s1);
  string_delete(&_o->s2);
  key_value_pair_donate_from_value(pair, info, 0, &_s);
  string_delete(&_s);
  deleteo(&_o);
}

static void print_(KeyValuePairPrintInfo *kvppi, Printer *p) {
  KeyValuePair *pair;
  KeyValueInfo *info;
  Object_ *o;

  pair = kvppi->this;
  info = kvppi->info;

  o = (Object_ *)key_value_pair_get_key(pair, info, 2);

  if (o == null) {
    rprint(p, "<EMPTY>");

    return;
  }

  rprint(p, "Key 0: %lu", key_value_pair_get_key(pair, info, 0));
  rprint(p, "Key 1: %lu", key_value_pair_get_key(pair, info, 1));
  sprint(p, "Key 2");
  rprint(p, "String 1: '%s'", o->s1);
  rprint(p, "String 2: '%s'", o->s2);
  fprint(p);
  rprint(p,
      "Value 0: '%s'",
      (char *)key_value_pair_get_value(pair, info, 0));
  rprint(p, "Value 1: %lu", key_value_pair_get_value(pair, info, 1));
  rprint(p,
      "Value 2: Invalid %s",
      bool_string(key_value_pair_get_value(pair, info, 2)));
}

static void merge_(KeyValuePair *acc, KeyValuePair *add, KeyValueInfo *info) {
  byte v1acc;
  byte v1add;
  byte v1;
  char *_s = null;
  Pass_ *pass;
  Object_ *_o = null;

  pass = info->x;

  /* Delete the object in key 2 of 'add'.
   */
  key_value_pair_donate_from_key(add, info, 2, &_o);
  string_delete(&_o->s1);
  string_delete(&_o->s2);
  deleteo(&_o);

  /* Delete the string in value 0 of 'add'.
   */
  key_value_pair_donate_from_value(add, info, 0, &_s);
  string_delete(&_s);

  /* Delete the string currently in value 0 of 'acc'.
   */
  key_value_pair_donate_from_value(acc, info, 0, &_s);
  string_delete(&_s);

  /* Combine the v1s by adding them.
   */
  v1acc = key_value_pair_get_value(acc, info, 1);
  v1add = key_value_pair_get_value(add, info, 1);
  v1 = v1acc + v1add;

  /* Create the new string for value 0 in 'acc'.
   */
  string_(&_s, v1 * pass->scale_1);
  key_value_pair_donate_to_value(acc, info, 0, &_s);

  /* Set the new v1.
   */
  key_value_pair_set_value(acc, info, 1, v1);
}

static sbyte compare_(KeyValuePair *a, KeyValuePair *b, KeyValueInfo *info) {
  ulong k0a;
  ulong k0b;
  ulong k1a;
  ulong k1b;

  k0a = key_value_pair_get_key(a, info, 0);
  k1a = key_value_pair_get_key(a, info, 1);
  k0b = key_value_pair_get_key(b, info, 0);
  k1b = key_value_pair_get_key(b, info, 1);

  if (k0a < k0b) {
    return -1;
  }

  if (k0a > k0b) {
    return +1;
  }

  if (k1a < k1b) {
    return -1;
  }

  if (k1a > k1b) {
    return +1;
  }

  return 0;
}

static bool get_invalid_(KeyValuePair *pair, KeyValueInfo *info) {
  return key_value_pair_get_value(pair, info, 2) == 1;
}

static void set_invalid_(KeyValuePair *pair, KeyValueInfo *info) {
  key_value_pair_set_value(pair, info, 2, 1);
}

static void construct_1_(
    KeyValuePair *pair,
    KeyValueInfo *info,
    byte k0,
    byte k1,
    byte v0) {
  Pass_1_ *pass;
  Object_1_ *_o = null;

  pass = info->x;

  newo(&_o);
  string_(&_o->s1, k0);
  string_(&_o->s2, k1 * pass->scale);
  key_value_pair_donate_to_key(pair, info, 0, &_o);

  key_value_pair_set_key(pair, info, 1, k1);
  key_value_pair_set_value(pair, info, 0, v0);
}

static void destroy_1_(KeyValuePair *pair, KeyValueInfo *info) {
  Pass_1_ *pass;
  Object_1_ *_o = null;

  pass = info->x;

  if (pass->scale != TR_SCALE_1_) {
    throw("Impossible");
  }

  if (!key_value_pair_try_donate_from_key(pair, info, 0, &_o)) {
    return;
  }

  string_delete(&_o->s1);
  string_delete(&_o->s2);

  deleteo(&_o);
}

static void print_1_(KeyValuePairPrintInfo *kvppi, Printer *p) {
  KeyValuePair *pair;
  KeyValueInfo *info;
  Object_1_ *o;

  pair = kvppi->this;
  info = kvppi->info;

  sprint(p, "Key 0");
  o = key_value_pair_reference_from_key(pair, info, 0);
  rprint(p, "String 1: '%s'", o->s1);
  rprint(p, "String 2: '%s'", o->s2);
  fprint(p);
  rprint(p, "Key 1: %lu", key_value_pair_get_key(pair, info, 1));
  rprint(p, "Value 0: %lu", key_value_pair_get_value(pair, info, 0));
}

static void merge_1_(KeyValuePair *acc,
    KeyValuePair *add,
    KeyValueInfo *info) {
  byte v0;
  byte v0acc;
  byte v0add;
  Object_1_ *_o_add = null;

  v0acc = key_value_pair_get_value(acc, info, 0);
  v0add = key_value_pair_get_value(add, info, 0);

  v0 = v0acc + v0add;

  key_value_pair_set_value(acc, info, 0, v0);

  key_value_pair_donate_from_key(add, info, 0, &_o_add);
  string_delete(&_o_add->s1);
  string_delete(&_o_add->s2);
  deleteo(&_o_add);
}

static sbyte compare_1_(KeyValuePair *a, KeyValuePair *b, KeyValueInfo *info) {
  sbyte c;
  Object_1_ *oa;
  Object_1_ *ob;
  ulong k1a;
  ulong k1b;

  oa = key_value_pair_reference_from_key(a, info, 0);
  ob = key_value_pair_reference_from_key(b, info, 0);
  k1a = key_value_pair_get_key(a, info, 1);
  k1b = key_value_pair_get_key(b, info, 1);

  c = string_compare(oa->s1, ob->s1, null);

  if (c != 0) {
    return c;
  }

  if (k1a < k1b) {
    return -1;
  }

  if (k1a > k1b) {
    return +1;
  }

  return 0;
}

static void construct_2_(
    KeyValuePair *pair,
    KeyValueInfo *info,
    byte k0,
    byte k1,
    byte k2,
    byte v0,
    byte v2) {
  char *_s = null;
  Object_2_1_ *_o1 = null;
  Object_2_2_ *_o2 = null;

  newo(&_o1);
  string_(&_o1->s, k0);
  key_value_pair_donate_to_key(pair, info, 0, &_o1);

  key_value_pair_set_key(pair, info, 1, k1);

  newo(&_o2);
  string_(&_o2->s, k2);
  key_value_pair_donate_to_key(pair, info, 2, &_o2);

  key_value_pair_set_value(pair, info, 0, v0);

  string_(&_s, v0);
  key_value_pair_donate_to_value(pair, info, 1, &_s);

  key_value_pair_set_value(pair, info, 2, v2);
}

static void destroy_2_(KeyValuePair *pair, KeyValueInfo *info) {
  char *_s = null;
  Pass_2_ *pass;
  Object_2_1_ *_o1 = null;
  Object_2_2_ *_o2 = null;

  pass = info->x;

  if (pass->scale != SCALE_2_2_) {
    throw("Impossible");
  }

  if (!key_value_pair_try_donate_from_key(pair, info, 0, &_o1)) {
    return;
  }

  string_delete(&_o1->s);

  key_value_pair_donate_from_key(pair, info, 2, &_o2);
  string_delete(&_o2->s);

  key_value_pair_donate_from_value(pair, info, 1, &_s);
  string_delete(&_s);

  deleteo(&_o1);
  deleteo(&_o2);
}

static void print_2_(KeyValuePairPrintInfo *kvppi, Printer *p) {
  char *s;
  KeyValuePair *pair;
  KeyValueInfo *info;
  Object_2_1_ *o1;
  Object_2_2_ *o2;

  pair = kvppi->this;
  info = kvppi->info;
  sprint(p, "Key 0");
  o1 = key_value_pair_reference_from_key(pair, info, 0);
  rprint(p, "String: length %lu", string_length(o1->s));
  fprint(p);
  rprint(p, "Key 1: %lu", key_value_pair_get_key(pair, info, 1));
  sprint(p, "Key 2");
  o2 = key_value_pair_reference_from_key(pair, info, 2);
  rprint(p, "String: length %lu", string_length(o2->s));
  rprint(p, "Value 0: %lu", key_value_pair_get_value(pair, info, 0));
  sprint(p, "Value 1");
  s = key_value_pair_reference_from_value(pair, info, 1);
  rprint(p, "String: length %lu", string_length(s));
  rprint(p, "Value 2: %lu", key_value_pair_get_value(pair, info, 2));
  fprint(p);
  fprint(p);
}

static void merge_2_(KeyValuePair *acc,
    KeyValuePair *add,
    KeyValueInfo *info) {
  byte v0;
  byte v0acc;
  byte v0add;
  byte v1;
  byte v1acc;
  byte v1add;
  byte v2;
  byte v2acc;
  byte v2add;
  char *_s_acc = null;
  char *_s_add = null;
  Object_2_1_ *_o1_add = null;
  Object_2_2_ *_o2_add = null;

  /* Add v0s.
   */
  v0acc = key_value_pair_get_value(acc, info, 0);
  v0add = key_value_pair_get_value(add, info, 0);
  v0 = v0acc + v0add;
  key_value_pair_set_value(acc, info, 0, v0);

  /* Add v1s, where each is a string.
   */
  key_value_pair_donate_from_value(acc, info, 1, &_s_acc);
  v1acc = string_length(_s_acc);
  string_delete(&_s_acc);

  key_value_pair_donate_from_value(add, info, 1, &_s_add);
  v1add = string_length(_s_add);
  string_delete(&_s_add);

  v1 = v1acc + v1add;

  string_(&_s_acc, v1);
  key_value_pair_donate_to_value(acc, info, 1, &_s_acc);

  /* Add v2s.
   */
  v2acc = key_value_pair_get_value(acc, info, 2);
  v2add = key_value_pair_get_value(add, info, 2);
  v2 = v2acc + v2add;
  key_value_pair_set_value(acc, info, 2, v2);

  /* Delete k0 from add.
   */
  key_value_pair_donate_from_key(add, info, 0, &_o1_add);
  string_delete(&_o1_add->s);
  deleteo(&_o1_add);

  /* Delete k2 from add.
   */
  key_value_pair_donate_from_key(add, info, 2, &_o2_add);
  string_delete(&_o2_add->s);
  deleteo(&_o2_add);
}

static sbyte compare_2_(KeyValuePair *a, KeyValuePair *b, KeyValueInfo *info) {
  sbyte c;
  Object_2_1_ *o1a;
  Object_2_1_ *o1b;
  Object_2_2_ *o2a;
  Object_2_2_ *o2b;
  ulong k1a;
  ulong k1b;

  o1a = key_value_pair_reference_from_key(a, info, 0);
  o2a = key_value_pair_reference_from_key(a, info, 2);
  o1b = key_value_pair_reference_from_key(b, info, 0);
  o2b = key_value_pair_reference_from_key(b, info, 2);
  k1a = key_value_pair_get_key(a, info, 1);
  k1b = key_value_pair_get_key(b, info, 1);

  c = string_compare(o1a->s, o1b->s, null);

  if (c != 0) {
    return c;
  }

  c = string_compare(o2a->s, o2b->s, null);

  if (c != 0) {
    return c;
  }

  if (k1a < k1b) {
    return -1;
  }

  if (k1a > k1b) {
    return +1;
  }

  return 0;
}

static void transform_(
    KeyValuePair *pair_old,
    KeyValueInfo *info_old,
    KeyValuePair *pair_new,
    KeyValueInfo *info_new,
    void *x) {
  byte scale;
  byte *p_scale;
  ulong new_k0;
  ulong new_k1;
  ulong new_k2;
  ulong new_v0;
  ulong new_v2;
  ulong old_k0;
  ulong old_k1;
  ulong old_v0;
  Object_1_ *o1;

  p_scale = x;
  scale = *p_scale;

  o1 = key_value_pair_reference_from_key(pair_old, info_old, 0);
  old_k0 = string_length(o1->s1) / scale;
  old_k1 = key_value_pair_get_key(pair_old, info_old, 1);
  old_v0 = key_value_pair_get_value(pair_old, info_old, 0);

  new_k0 = old_k0 * scale;
  new_k1 = old_k1 / scale;
  new_k2 = old_k1 / scale;
  new_v0 = old_v0 * scale;
  new_v2 = old_v0 + 1;

  construct_2_(
      pair_new,
      info_new,
      new_k0,
      new_k1,
      new_k2,
      new_v0,
      new_v2);
}

static void transform_2_(
    KeyValuePair *pair_old,
    KeyValuePair *pair_new,
    KeyValueInfo *info,
    void *x) {
  byte k0;
  byte k1;
  byte v1;
  uint scale;
  uint *transform_info;

  transform_info = x;
  scale = *transform_info;

  k0 = key_value_pair_get_key(pair_old, info, 0);
  k1 = key_value_pair_get_key(pair_old, info, 1);
  v1 = key_value_pair_get_value(pair_old, info, 1);

  construct_(pair_new, info, k0 / scale, k1 / scale, v1 * scale);
}
