#include "KeyValueArrayTestDeath.h"

START_DEATH

TEST(KeyValueArray, NewCopy) {
  ulong max = 1;
  KeyValuePair *_pair = null;
  KeyValueInfo *_info = null;
  KeyValueArray *_array = null;
  KeyValueArray *_array2 = null;

  set_info_new(&_info, 1, &max);
  key_value_pair_new(&_pair, _info);
  key_value_array_new(&_array, _info, 2);
  EXPECT_DEATH(
      key_value_array_new_copy(&_array2, _array),
      "Cannot copy an unfinalized KeyValueArray");
  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);
}

TEST(KeyValueArray, NewHyperslice) {
  ulong max = 1;
  KeyValuePair *_pair = null;
  KeyValueInfo *_info = null;
  KeyValueArray *_array = null;
  KeyValueArray *_array2 = null;

  set_info_new(&_info, 1, &max);
  key_value_array_new(&_array, _info, 2);
  key_value_pair_new(&_pair, _info);
  key_value_array_donate(_array, _pair);
  key_value_array_donate(_array, _pair);
  EXPECT_DEATH(
      key_value_array_new_hyperslice(&_array2, _array, 0, 1),
      "Cannot create hyperslice of unfinalized KeyValueArray");
  key_value_array_finalize(_array);
  key_value_array_new_hyperslice(&_array2, _array, 0, 1);
  key_value_array_delete(&_array2);
  key_value_array_delete(&_array);

  key_value_array_new(&_array, _info, 2);
  key_value_array_donate(_array, _pair);
  key_value_pair_set_key(_pair, _info, 0, 1);
  key_value_array_donate(_array, _pair);
  key_value_array_finalize(_array);
  EXPECT_DEATH(
      key_value_array_new_hyperslice(&_array2, _array, 1, 2),
      "Hyperslice pair offset 1 with number of pairs 2 exceeds number of "
      "pairs in KeyValueArray (2)");
  key_value_array_delete(&_array);

  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);
}

TEST(KeyValueArray, Append) {
  ulong max = 0;
  KeyValuePair *_pair = null;
  KeyValueInfo *_info = null;
  KeyValueArray *_array = null;

  set_info_new(&_info, 1, &max);
  key_value_array_new(&_array, _info, 1);
  key_value_pair_new(&_pair, _info);
  key_value_array_donate(_array, _pair);
  EXPECT_DEATH(
      key_value_array_donate(_array, _pair),
      "Attempt to donate to a KeyValueArray that is full");
  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);
}

TEST(KeyValueArray, Get) {
  ulong max = 0;
  KeyValuePair *_pair = null;
  KeyValueInfo *_info = null;
  KeyValueArray *_array = null;

  set_info_new(&_info, 1, &max);
  key_value_array_new(&_array, _info, 1);
  EXPECT_DEATH(
      key_value_array_get(_array, 0),
      "Attempt to get element 0 from empty KeyValueArray");
  key_value_array_delete(&_array);

  key_value_array_new(&_array, _info, 1);
  key_value_pair_new(&_pair, _info);
  key_value_array_donate(_array, _pair);
  EXPECT_DEATH(
      key_value_array_get(_array, 1),
      "Attempt to get element 1 from KeyValueArray (max 0)");
  key_value_array_delete(&_array);

  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);
}

TEST(KeyValueArray, Cardinality) {
  ulong max = 0;
  KeyValueInfo *_info = null;
  KeyValueArray *_array = null;

  set_info_new(&_info, 1, &max);
  key_value_array_new(&_array, _info, 1);
  EXPECT_DEATH(
      key_value_array_cardinality(_array),
      "Cannot get the cardinality of an unfinalized KeyValueArray");
  key_value_array_delete(&_array);
  key_value_info_delete(&_info);
}

TEST(KeyValueArray, IsEmpty) {
  ulong max = 0;
  KeyValueInfo *_info = null;
  KeyValueArray *_array = null;

  set_info_new(&_info, 1, &max);
  key_value_array_new(&_array, _info, 1);
  EXPECT_DEATH(
      key_value_array_is_empty(_array),
      "Cannot get the cardinality of an unfinalized KeyValueArray");
  key_value_array_delete(&_array);
  key_value_info_delete(&_info);
}

TEST(KeyValueArray, GetMinimum) {
  ulong max = 0;
  KeyValueInfo *_info = null;
  KeyValueArray *_array = null;

  set_info_new(&_info, 1, &max);
  key_value_array_new(&_array, _info, 1);
  EXPECT_DEATH(
      key_value_array_minimum(_array),
      "Cannot get the minimum of an unfinalized KeyValueArray");
  key_value_array_delete(&_array);
  key_value_info_delete(&_info);
}

TEST(KeyValueArray, GetMaximum) {
  ulong max = 0;
  KeyValueInfo *_info = null;
  KeyValueArray *_array = null;

  set_info_new(&_info, 1, &max);
  key_value_array_new(&_array, _info, 1);
  EXPECT_DEATH(
      key_value_array_maximum(_array),
      "Cannot get the maximum of an unfinalized KeyValueArray");
  key_value_array_delete(&_array);
  key_value_info_delete(&_info);
}

TEST(KeyValueArray, Find) {
  ulong max = 0;
  KeyValuePair *_pair = null;
  KeyValueInfo *_info = null;
  KeyValueArray *_array = null;

  set_info_new(&_info, 1, &max);
  key_value_array_new(&_array, _info, 2);
  key_value_pair_new(&_pair, _info);
  key_value_array_donate(_array, _pair);
  key_value_array_donate(_array, _pair);
  EXPECT_DEATH(
      key_value_array_find(_array, _pair),
      "Cannot find in an unfinalized KeyValueArray");
  key_value_array_delete(&_array);
  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);
}

TEST(KeyValueArray, FindIndex) {
  ulong max = 1;
  KeyValuePair *_pair = null;
  KeyValueInfo *_info = null;
  KeyValueArray *_array = null;

  set_info_new(&_info, 1, &max);
  key_value_array_new(&_array, _info, 2);
  key_value_pair_new(&_pair, _info);
  key_value_array_donate(_array, _pair);
  key_value_array_donate(_array, _pair);
  EXPECT_DEATH(
      key_value_array_find_index(_array, _pair),
      "Cannot find in an unfinalized KeyValueArray");
  key_value_array_finalize(_array);
  key_value_pair_set_key(_pair, _info, 0, 1);
  EXPECT_DEATH(
      key_value_array_find_index(_array, _pair),
      "Could not find the pair in the array:\n. [KeyValuePair]\n"
      ". . Key: 1 (max 1)\n. [/KeyValuePair]\n");
  key_value_array_delete(&_array);
  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);
}

TEST(KeyValueArray, TryMergeKeyValuePair) {
  ulong max = 0;
  KeyValuePair *_pair = null;
  KeyValueInfo *_info = null;
  KeyValueArray *_array = null;

  set_info_new(&_info, 1, &max);
  key_value_array_new(&_array, _info, 2);
  key_value_pair_new(&_pair, _info);
  key_value_array_donate(_array, _pair);
  key_value_array_donate(_array, _pair);
  EXPECT_DEATH(
      key_value_array_try_merge_pair(_array, _pair),
      "Cannot merge into an unfinalized KeyValueArray");
  key_value_array_delete(&_array);
  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);
}

FINISH_DEATH
