#include "KeyValueByteTest.h"

#define N1_ 5000000
#define N2_ 100
#define N3_ 100

static void do_(Random *_random, bool test_print, bool save_load);


START

TEST(KeyValueByte, Comprehensive) {
  ulong u;
  Random *_random = null;

  random_new(&_random);

  for (u = 0; u < N1_; u++) {
    CALL(do_(_random, false, false));
  }

  random_delete(&_random);
}

TEST(KeyValueByte, Print) {
  ulong u;
  Random *_random = null;

  random_new(&_random);

  for (u = 0; u < N2_; u++) {
    CALL(do_(_random, true, false));
  }

  random_delete(&_random);
}

TEST(KeyValueByte, LoadSave) {
  ulong u;
  Random *_random = null;

  random_new(&_random);

  for (u = 0; u < N3_; u++) {
    CALL(do_(_random, false, true));
  }

  random_delete(&_random);
}

FINISH


static void do_(Random *_random, bool test_print, bool save_load) {
  byte read_byte_mask;
  byte write_byte_mask;
  sbyte shift;
  char *_e = null;
  char *_s = null;
  ulong value_mask;
  KeyValueByte *_byte = null;

  value_mask = random_ulong(_random);
  shift = (sbyte)random_modulo(_random, 64);
  read_byte_mask = (byte)random_modulo(_random, 256);
  write_byte_mask = ~read_byte_mask;

  key_value_byte_new(&_byte, value_mask, shift, read_byte_mask);

  if (test_print) {
    aprint(&_s, 0, key_value_byte, _byte);
    string_new_f(
        &_e,
        "[KeyValueByte]\n. Value mask: 0x%lx\n. Shift: %d\n"
        ". Read byte mask: 0x%02x\n. Write byte mask: 0x%02x\n"
        "[/KeyValueByte]\n",
        value_mask,
        shift,
        read_byte_mask,
        write_byte_mask);
    string_renew_add_separators(&_e);
    EXPECT_STREQ(_e, _s);
    string_delete(&_e);
    string_delete(&_s);
  }

  EXPECT_ULEQ(value_mask, _byte->valueMask);
  EXPECT_ULEQ(shift, _byte->shift);
  EXPECT_ULEQ(read_byte_mask, _byte->readByteMask);
  EXPECT_ULEQ(write_byte_mask, _byte->writeByteMask);

  if (save_load) {
    test_save_load(KeyValueByte, key_value_byte, _byte);
  }

  key_value_byte_delete(&_byte);
}
