#include "KeyValueInfoTest.h"

START

TEST(KeyValueInfo, New) {
  ulong max_keys[] = {0, 1, 2, 4, 256};
  ulong max_values[] = {5000000000};
  KeyValueInfo *_info = null;
  KeyValueInfo *_info2 = null;

  /* Invariant-size memory for each of the 6 fields:
   *   a ulong max value (8 bytes)
   *   a pointer to the KeyValueField object (8 bytes)
   *   the KeyValueField object
   *
   * The keys require 0, 1, 2, 3, and 9 bits respectively, and the value
   * requires 33 bits, which is a total of 48 bits, which fits in 6 bytes.
   * However, the number of (partial) bytes covered by each key is 0, 1, 1, 1,
   * and 2, and by the value is 5, so that there are 10 KeyValueBytes needed in
   * total in the KeyValueFields.
   */
  key_value_info_new(&_info, 5, max_keys, 1, max_values);
  EXPECT_ULEQ(5, _info->numKeys);
  EXPECT_ULEQ(48, _info->numBits);
  EXPECT_ULEQ(6, _info->numBytes);
  test_save_load(KeyValueInfo, key_value_info, _info);
  key_value_info_delete(&_info);

  key_value_info_new(&_info, 5, max_keys, 1, max_values);
  EXPECT_ULEQ(5, _info->numKeys);
  EXPECT_ULEQ(48, _info->numBits);
  EXPECT_ULEQ(6, _info->numBytes);
  test_save_load(KeyValueInfo, key_value_info, _info);

  key_value_info_new_copy(&_info2, _info);
  EXPECT_TRUE(key_value_info_equals(_info, _info2));
  key_value_info_delete(&_info);
  EXPECT_ULEQ(5, _info2->numKeys);
  EXPECT_ULEQ(48, _info2->numBits);
  EXPECT_ULEQ(6, _info2->numBytes);
  key_value_info_delete(&_info2);
}

TEST(KeyValueInfo, Print) {
  char *_e = null;
  char *_s = null;
  ulong max_keys[] = {1};
  ulong max_values[] = {7};
  KeyValueInfo *_info = null;

  /* Invariant-size memory for each of the 2 fields described above.
   *
   * The key requires 1 bit and the value requires 3 bits, which is a total of
   * 4 bits, which fits in 1 byte. However, the key and value each fall in one
   * byte, so 2 KevValueBytes are needed.
   */
  key_value_info_new(&_info, 1, max_keys, 1, max_values);
  EXPECT_ULEQ(1, _info->numKeys);
  EXPECT_ULEQ(4, _info->numBits);
  EXPECT_ULEQ(1, _info->numBytes);
  aprint(&_s, 0, key_value_info, _info);
  string_new_concat(
      &_e,
      "[KeyValueInfo]\n. Number of keys: 1\n. Number of values: 1\n"
      ". Number of fields: 2\n. Number of bits: 4\n. Number of bytes: 1\n"
      ". [Key 0]\n. . [KeyValueField]\n. . . Max value: 1\n"
      ". . . Bit offset: 0\n. . . Bit width: 1\n"
      ". . . Number of KeyValueBytes: 1\n. . . First offset: 0\n"
      ". . . [Byte 0]\n. . . . [KeyValueByte]\n. . . . . Value mask: 0x1\n"
      ". . . . . Shift: -7\n. . . . . Read byte mask: 0x80\n"
      ". . . . . Write byte mask: 0x7f\n. . . . [/KeyValueByte]\n"
      ". . . [/Byte 0]\n. . [/KeyValueField]\n",
      ". [/Key 0]\n. [Value 0]\n. . [KeyValueField]\n. . . Max value: 7\n"
      ". . . Bit offset: 1\n. . . Bit width: 3\n"
      ". . . Number of KeyValueBytes: 1\n. . . First offset: 0\n"
      ". . . [Byte 0]\n. . . . [KeyValueByte]\n. . . . . Value mask: 0x7\n"
      ". . . . . Shift: -4\n. . . . . Read byte mask: 0x70\n"
      ". . . . . Write byte mask: 0x8f\n. . . . [/KeyValueByte]\n"
      ". . . [/Byte 0]\n. . [/KeyValueField]\n. [/Value 0]\n"
      ". Last key byte = 0\n. Last key byte mask = 0x80\n[/KeyValueInfo]\n");
  EXPECT_STREQ(_e, _s);
  string_delete(&_s);
  string_delete(&_e);
  key_value_info_delete(&_info);
}

FINISH
