#include "KeyValueNodeTest.h"

static void create_tree_(KeyValueNode **p_node);


START

TEST(KeyValueNode, New) {
  KeyValueNode *_node = null;

  key_value_node_new(&_node, null);
  EXPECT_NULL(_node->parent);
  key_value_node_delete(&_node);
}

TEST(KeyValueNode, Memory) {
  ulong max = 0;
  KeyValueInfo *_info = null;
  KeyValueArray *_array = null;
  KeyValueNode *_node = null;

  key_value_node_new(&_node, null);

  set_info_new(&_info, 1, &max);
  key_value_array_new(&_array, _info, 1);
  key_value_info_delete(&_info);
  key_value_array_finalize(_array);
  key_value_node_add_chunk(_node, &_array, 0, 0);

  key_value_node_delete(&_node);
}

TEST(KeyValueNode,KeyValueNodeTest TryMergePair) {
  ulong max = 2;
  KeyValueInfo *_info = null;
  KeyValueNode *_node = null;
  KeyValuePair *_pair = null;
  KeyValueArray *_array = null;

  key_value_node_new(&_node, null);
  set_info_new(&_info, 1, &max);
  key_value_pair_new(&_pair, _info);
  key_value_pair_set_key(_pair, _info, 0, 1);
  EXPECT_FALSE(key_value_node_try_merge_pair(_node, _pair));
  EXPECT_ULEQ(1, key_value_pair_get_key(_pair, _info, 0));
  key_value_array_new(&_array, _info, 3);
  key_value_array_donate(_array, _pair);
  EXPECT_ULEQ(1, key_value_pair_get_key(_pair, _info, 0));
  key_value_array_finalize(_array);
  key_value_node_add_chunk(_node, &_array, 0, 0);
  key_value_pair_set_key(_pair, _info, 0, 1);
  EXPECT_TRUE(key_value_node_try_merge_pair(_node, _pair));
  EXPECT_ULEQ(1, key_value_pair_get_key(_pair, _info, 0));
  key_value_pair_set_key(_pair, _info, 0, 2);
  EXPECT_FALSE(key_value_node_try_merge_pair(_node, _pair));
  EXPECT_ULEQ(2, key_value_pair_get_key(_pair, _info, 0));
  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);
  key_value_node_delete(&_node);
}

TEST(KeyValueNode, ForceMergeChildren) {
  KeyValueNode *_node = null;

  CALL(create_tree_(&_node));
  EXPECT_NULL(_node->_array);
  EXPECT_NONNULL(_node->_left);
  EXPECT_NONNULL(_node->_right);
  key_value_node_force_merge_children(_node);
  EXPECT_NONNULL(_node->_array);
  EXPECT_NULL(_node->_left);
  EXPECT_NULL(_node->_right);
  EXPECT_ULEQ(1, _node->_array->numPairs);
  key_value_node_delete(&_node);
}

FINISH


static void create_tree_(KeyValueNode **p_node) {
  ulong max = 1;
  KeyValueNode *node;
  KeyValueInfo *_info = null;
  KeyValueNode *_node2 = null;
  KeyValuePair *_pair = null;
  KeyValueArray *_array = null;

  key_value_node_new(p_node, null);
  node = *p_node;
  set_info_new(&_info, 1, &max);
  key_value_pair_new(&_pair, _info);
  EXPECT_FALSE(key_value_node_try_merge_pair(node, _pair));
  key_value_array_new(&_array, _info, 3);
  key_value_array_donate(_array, _pair);
  key_value_array_finalize(_array);
  key_value_node_add_chunk(node, &_array, 0, 0);
  key_value_node_new(&_node2, null);
  _node2->_left = node;
  node->parent = _node2;
  node = *p_node = null;
  key_value_array_new(&_array, _info, 7);
  key_value_array_donate(_array, _pair);
  key_value_array_finalize(_array);
  key_value_node_add_chunk(_node2, &_array, 1, 1);
  key_value_node_new(p_node, null);
  node = *p_node;
  node->_left = _node2;
  _node2->parent = node;
  _node2 = null;
  key_value_array_new(&_array, _info, 8);
  key_value_array_donate(_array, _pair);
  key_value_array_finalize(_array);
  key_value_node_add_chunk(node, &_array, 2, 2);
  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);
}
