#include "KeyValuePairTest.h"
#include <limits.h>
#include <math.h>

#define N_ 1000
#define P_ 3000

static void destroy_(KeyValuePair *pair, KeyValueInfo *info);

static bool get_invalid_(KeyValuePair *pair, KeyValueInfo *info);

static void set_invalid_(KeyValuePair *pair, KeyValueInfo *info);


START

TEST(KeyValuePair, Print) {
  char *_s = null;
  ulong max_keys[] = {1, 2, 4};
  ulong max_values[] = {7, 3};
  KeyValueInfo *_info = null;
  KeyValuePair *_pair = null;
  KeyValuePairPrintInfo kvppi;

  key_value_info_new(&_info, 1, max_keys, 1, max_values);
  key_value_pair_new(&_pair, _info);
  key_value_pair_set_key(_pair, _info, 0, 1);
  key_value_pair_set_value(_pair, _info, 0, 3);
  kvppi.this = _pair;
  kvppi.info = _info;
  aprint(&_s, 0, key_value_pair, &kvppi);
  EXPECT_STREQ(
      "[KeyValuePair]\n. Key: 1 (max 1)\n. Value: 3 (max 7)\n"
      "[/KeyValuePair]\n",
      _s);
  string_delete(&_s);
  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);

  key_value_info_new(&_info, 3, max_keys, 2, max_values);
  key_value_pair_new(&_pair, _info);
  key_value_pair_set_key(_pair, _info, 0, 1);
  key_value_pair_set_key(_pair, _info, 1, 2);
  key_value_pair_set_key(_pair, _info, 2, 3);
  key_value_pair_set_value(_pair, _info, 0, 3);
  key_value_pair_set_value(_pair, _info, 1, 1);
  kvppi.this = _pair;
  kvppi.info = _info;
  aprint(&_s, 0, key_value_pair, &kvppi);
  EXPECT_STREQ(
      "[KeyValuePair]\n. Key 0: 1 (max 1)\n. Key 1: 2 (max 2)\n"
      ". Key 2: 3 (max 4)\n. Value 0: 3 (max 7)\n. Value 1: 1 (max 3)\n"
      "[/KeyValuePair]\n",
      _s);
  string_delete(&_s);
  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);
}

TEST(KeyValuePair, Comprehensive) {
  int bcomp;
  int dcomp;
  int dcomp_second;
  ulong f;
  ulong field;
  ulong k;
  ulong max_key;
  ulong max_value;
  ulong num_keys;
  ulong num_values;
  ulong r_key;
  ulong r_value;
  ulong s_key;
  ulong u;
  ulong v;
  ulong *_max_keys = null;
  ulong *_max_values = null;
  ulong *_r_keys = null;
  ulong *_r_values = null;
  ulong *_s_keys = null;
  ulong *_s_values = null;
  KeyValueInfo *_info = null;
  KeyValueInfo *_info_new = null;
  KeyValuePair *_r = null;
  KeyValuePair *_r_new = null;
  KeyValuePair *_s = null;
  Random *_random = null;

  random_new(&_random);

  for (u = 0; u < N_; u++) {
    num_keys = (ulong)floor(
        exp(log(P_ + 1) * random_double_unit_cc(_random)) - 1);
    num_values = (ulong)floor(
        exp(log(P_ + 1) * random_double_unit_cc(_random))
        - (num_keys > 0 ? 1 : 0));
    new(&_max_keys, num_keys);
    new(&_r_keys, num_keys);
    new(&_s_keys, num_keys);

    for (k = 0; k < num_keys; k++) {
      _max_keys[k] = (ulong)floor(exp(
            log((double)ULONG_MAX) * random_double_unit_cc(_random)) - 1);
      _r_keys[k] = random_modulo(_random, _max_keys[k] + 1);
      _s_keys[k] = random_modulo(_random, _max_keys[k] + 1);
    }

    new(&_max_values, num_values);
    new(&_r_values, num_values);
    new(&_s_values, num_values);

    for (v = 0; v < num_values; v++) {
      _max_values[v] = (ulong)floor(
          exp(log((double)ULONG_MAX) * random_double_unit_cc(_random)) - 1);
      _r_values[v] = random_modulo(_random, _max_values[v] + 1);
      _s_values[v] = random_modulo(_random, _max_values[v] + 1);
    }

    key_value_info_new(
        &_info,
        num_keys,
        _max_keys,
        num_values,
        _max_values);
    key_value_pair_new(&_r, _info);
    key_value_pair_new(&_s, _info);

    for (k = 0; k < num_keys; k++) {
      key_value_pair_set_key(_r, _info, k, _r_keys[k]);
      key_value_pair_set_key(_s, _info, k, _s_keys[k]);
    }

    for (v = 0; v < num_values; v++) {
      key_value_pair_set_value(_r, _info, v, _r_values[v]);
      key_value_pair_set_value(_s, _info, v, _s_values[v]);
    }

    dcomp = 0;
    dcomp_second = 0;

    for (k = 0; k < num_keys; k++) {
      r_key = key_value_pair_get_key(_r, _info, k);
      s_key = key_value_pair_get_key(_s, _info, k);

      if (dcomp == 0) {
        if (r_key > s_key) {
          dcomp = +1;
        } else if (r_key < s_key) {
          dcomp = -1;
        }
      }

      if (dcomp_second == 0 && k > 0) {
        if (r_key > s_key) {
          dcomp_second = +1;
        } else if (r_key < s_key) {
          dcomp_second = -1;
        }
      }

      EXPECT_ULEQ(_r_keys[k], r_key);
    }

    for (v = 0; v < num_values; v++) {
      r_value = key_value_pair_get_value(_r, _info, v);
      EXPECT_ULEQ(_r_values[v], r_value);
    }

    bcomp = _info->compare(_r, _s, _info);
    EXPECT_ULEQ(bcomp, dcomp);

    for (k = 0; k < num_keys; k++) {
      do {
        max_key = (_max_keys[k] + 1) * (random_modulo(_random, 100) + 1);
      } while (max_key <= _max_keys[k]);

      _max_keys[k] = max_key;
    }

    for (v = 0; v < num_values; v++) {
      do {
        max_value = (_max_values[v] + 1) * (random_modulo(_random, 100) + 1);
      } while (max_value <= _max_values[v]);

      _max_values[v] = max_value;
    }

    key_value_info_new(
        &_info_new,
        num_keys,
        _max_keys,
        num_values,
        _max_values);
    key_value_pair_new(&_r_new, _info_new);

    key_value_pair_convert(_r, _info, _r_new, _info_new);

    for (f = 0; f < num_keys + num_values; f++) {
      EXPECT_ULEQ(
          field = key_value_pair_get_field(_r, _info, f),
          key_value_pair_get_field(_r_new, _info_new, f));
      key_value_pair_set_field(_r, _info, f, field);
      EXPECT_ULEQ(field, key_value_pair_get_field(_r, _info, f));
    }

    key_value_pair_delete(&_r_new, _info_new);
    key_value_info_delete(&_info_new);

    key_value_pair_delete(&_r, _info);
    key_value_pair_delete(&_s, _info);
    key_value_info_delete(&_info);

    deletev(&_r_values);
    deletev(&_s_values);
    deletev(&_max_values);
    deletev(&_r_keys);
    deletev(&_s_keys);
    deletev(&_max_keys);
  }

  random_delete(&_random);
}

TEST(KeyValuePair, AlwaysZeroSecondKeyNoValues) {
  ulong max_keys[] = {255, 0};
  KeyValueInfo *_info = null;
  KeyValuePair *_r = null;
  KeyValuePair *_s = null;

  key_value_info_new(&_info, 2, max_keys, 0, null);
  key_value_pair_new(&_r, _info);
  key_value_pair_new(&_s, _info);
  EXPECT_ULEQ(0, _info->compare(_r, _s, _info));
  key_value_pair_delete(&_r, _info);
  key_value_pair_delete(&_s, _info);
  key_value_info_delete(&_info);
}

TEST(KeyValuePair, Invalid) {
  ulong max_value = 10;
  KeyValueInfo *_info = null;
  KeyValuePair *_pair = null;

  key_value_info_new(&_info, 0, null, 1, &max_value);
  _info->getInvalid = get_invalid_;
  _info->setInvalid = set_invalid_;
  key_value_pair_new(&_pair, _info);
  EXPECT_FALSE(key_value_pair_get_invalid(_pair, _info));
  key_value_pair_set_invalid(_pair, _info);
  EXPECT_TRUE(key_value_pair_get_invalid(_pair, _info));
  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);
}

TEST(KeyValuePair, DonateReference) {
  byte *b;
  byte *_b = null;
  ulong max = ULONG_MAX;
  KeyValuePair *_pair = null;
  KeyValueInfo *_info = null;

  newo(&_b);
  b = _b;

  key_value_info_new(&_info, 1, &max, 0, null);
  key_value_pair_new(&_pair, _info);
  key_value_pair_donate_to_key(_pair, _info, 0, &_b);
  EXPECT_NULL(_b);
  EXPECT_ULEQ((ulong)b, key_value_pair_get_key(_pair, _info, 0));
  EXPECT_PEQ(b, key_value_pair_reference_from_key(_pair, _info, 0));
  EXPECT_NULL(_b);
  EXPECT_ULEQ((ulong)b, key_value_pair_get_key(_pair, _info, 0));
  key_value_pair_donate_from_key(_pair, _info, 0, &_b);
  EXPECT_PEQ(b, _b);
  EXPECT_ULEQ(0, key_value_pair_get_key(_pair, _info, 0));
  EXPECT_FALSE(key_value_pair_try_donate_from_key(_pair, _info, 0, &_b));
  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);

  key_value_info_new(&_info, 0, null, 1, &max);
  key_value_pair_new(&_pair, _info);
  key_value_pair_donate_to_value(_pair, _info, 0, &_b);
  EXPECT_NULL(_b);
  EXPECT_ULEQ((ulong)b, key_value_pair_get_value(_pair, _info, 0));
  EXPECT_PEQ(b, key_value_pair_reference_from_value(_pair, _info, 0));
  EXPECT_NULL(_b);
  EXPECT_ULEQ((ulong)b, key_value_pair_get_value(_pair, _info, 0));
  key_value_pair_donate_from_value(_pair, _info, 0, &_b);
  EXPECT_PEQ(b, _b);
  EXPECT_ULEQ(0, key_value_pair_get_value(_pair, _info, 0));
  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);

  key_value_info_new(&_info, 1, &max, 1, &max);
  key_value_pair_new(&_pair, _info);
  key_value_pair_donate_to_key(_pair, _info, 0, &_b);
  EXPECT_NULL(_b);
  EXPECT_ULEQ((ulong)b, key_value_pair_get_key(_pair, _info, 0));
  EXPECT_PEQ(b, key_value_pair_reference_from_key(_pair, _info, 0));
  key_value_pair_donate_from_key(_pair, _info, 0, &_b);
  EXPECT_PEQ(b, _b);
  EXPECT_ULEQ(0, key_value_pair_get_key(_pair, _info, 0));
  key_value_pair_donate_to_value(_pair, _info, 0, &_b);
  EXPECT_NULL(_b);
  EXPECT_ULEQ((ulong)b, key_value_pair_get_value(_pair, _info, 0));
  EXPECT_PEQ(b, key_value_pair_reference_from_value(_pair, _info, 0));
  key_value_pair_donate_from_value(_pair, _info, 0, &_b);
  EXPECT_PEQ(b, _b);
  EXPECT_ULEQ(0, key_value_pair_get_value(_pair, _info, 0));
  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);

  deleteo(&_b);
}

TEST(KeyValuePair, Move) {
  ulong max = ULONG_MAX;
  KeyValuePair *_pair_1 = null;
  KeyValuePair *_pair_2 = null;
  KeyValueInfo *_info = null;

  /* No clearing of source pair if there is no destructor.
   */
  key_value_info_new(&_info, 1, &max, 0, null);
  key_value_pair_new(&_pair_1, _info);
  key_value_pair_set_key(_pair_1, _info, 0, 24);
  key_value_pair_new(&_pair_2, _info);
  key_value_pair_move(_pair_1, _pair_2, _info);
  EXPECT_ULEQ(24, key_value_pair_get_key(_pair_1, _info, 0));
  EXPECT_ULEQ(24, key_value_pair_get_key(_pair_2, _info, 0));
  key_value_pair_delete(&_pair_2, _info);
  key_value_pair_delete(&_pair_1, _info);
  key_value_info_delete(&_info);

  /* Source pair is cleared if there is a destructor.
   */
  key_value_info_new(&_info, 1, &max, 0, null);
  _info->destroy = destroy_;
  key_value_pair_new(&_pair_1, _info);
  key_value_pair_set_key(_pair_1, _info, 0, 24);
  key_value_pair_new(&_pair_2, _info);
  key_value_pair_move(_pair_1, _pair_2, _info);
  EXPECT_ULEQ(0, key_value_pair_get_key(_pair_1, _info, 0));
  EXPECT_ULEQ(24, key_value_pair_get_key(_pair_2, _info, 0));
  key_value_pair_delete(&_pair_2, _info);
  key_value_pair_delete(&_pair_1, _info);
  key_value_info_delete(&_info);
}

FINISH


static void destroy_(KeyValuePair *pair, KeyValueInfo *info) {
}

static bool get_invalid_(KeyValuePair *pair, KeyValueInfo *info) {
  return key_value_pair_get_value(pair, info, 0) == 7;
}

static void set_invalid_(KeyValuePair *pair, KeyValueInfo *info) {
  key_value_pair_set_value(pair, info, 0, 7);
}
