#include "KeyValuePairTestDeath.h"

START_DEATH

TEST(KeyValuePair, SetGetKeyValue) {
  ulong max_key = 1;
  ulong max_value = 2;
  KeyValuePair *_pair = null;
  KeyValuePair *_pair2 = null;
  KeyValueInfo *_info = null;
  KeyValueInfo *_info2 = null;

  key_value_info_new(&_info, 1, &max_key, 1, &max_value);
  key_value_pair_new(&_pair, _info);
  EXPECT_DEATH(
      key_value_pair_set_key(_pair, _info, 1, 0),
      "Key index 1 is invalid (max 0)");
  EXPECT_DEATH(
      key_value_pair_set_key(_pair, _info, 0, 2),
      "Value (2) of field 0 greater than maximum allowed (1)");
  EXPECT_DEATH(
      key_value_pair_get_key(_pair, _info, 1),
      "Key index 1 is invalid (max 0)");
  EXPECT_DEATH(
      key_value_pair_set_value(_pair, _info, 1, 0),
      "Value index 1 is invalid (max 0)");
  EXPECT_DEATH(
      key_value_pair_set_value(_pair, _info, 0, 3),
      "Value (3) of field 1 greater than maximum allowed (2)");
  EXPECT_DEATH(
      key_value_pair_get_value(_pair, _info, 1),
      "Value index 1 is invalid (max 0)");
  EXPECT_DEATH(
      key_value_pair_set_field(_pair, _info, 2, 0),
      "Field index 2 is invalid (max 1)");
  EXPECT_DEATH(
      key_value_pair_get_field(_pair, _info, 2),
      "Field index 2 is invalid (max 1)");

  key_value_info_new(&_info2, 1, &max_key, 0, null);
  key_value_pair_new(&_pair2, _info2);
  EXPECT_DEATH(
      key_value_pair_convert(_pair, _info, _pair2, _info2),
      "Number of fields wrong (2 v 1)");

  key_value_pair_delete(&_pair2, _info2);
  key_value_info_delete(&_info2);

  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);
}

TEST(KeyValuePair, Donate) {
  byte *_b = null;
  ulong max = ULONG_MAX;
  ulong max_keys[] = {1, 2, 3};
  ulong max_values[] = {2, 7, 11, 13};
  KeyValuePair *_pair = null;
  KeyValueInfo *_info = null;

  key_value_info_new(&_info, 1, &max, 0, null);
  key_value_pair_new(&_pair, _info);
  newo(&_b);
  EXPECT_DEATH(
      key_value_pair_donate_to_key(_pair, _info, 2, &_b),
      "Key index 2 is invalid (max 0)");
  EXPECT_DEATH(
      key_value_pair_donate_from_key(_pair, _info, 2, &_b),
      "Key index 2 is invalid (max 0)");
  EXPECT_DEATH(
      key_value_pair_donate_to_value(_pair, _info, 0, &_b),
      "Invalid: No values in KeyValueInfo");
  EXPECT_DEATH(
      key_value_pair_donate_from_value(_pair, _info, 0, &_b),
      "Invalid: No values in KeyValueInfo");
  EXPECT_DEATH(
      key_value_pair_donate_to_field(_pair, _info, 1, &_b),
      "Field index 1 is invalid (max 0)");
  EXPECT_DEATH(
      key_value_pair_donate_from_field(_pair, _info, 1, &_b),
      "Field index 1 is invalid (max 0)");
  deleteo(&_b);
  EXPECT_DEATH(
      key_value_pair_donate_to_key(_pair, _info, 0, null),
      "Pointer to source pointer null");
  EXPECT_DEATH(
      key_value_pair_donate_from_key(_pair, _info, 0, null),
      "Pointer to destination pointer null");
  EXPECT_DEATH(
      key_value_pair_donate_to_key(_pair, _info, 0, &_b),
      "Source pointer empty");
  EXPECT_DEATH(
      key_value_pair_donate_from_key(_pair, _info, 0, &_b),
      "Source pointer empty");
  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);

  key_value_info_new(&_info, 0, null, 1, &max);
  key_value_pair_new(&_pair, _info);
  newo(&_b);
  EXPECT_DEATH(
      key_value_pair_donate_to_key(_pair, _info, 0, &_b),
      "Invalid: No keys in KeyValueInfo");
  EXPECT_DEATH(
      key_value_pair_donate_from_key(_pair, _info, 0, &_b),
      "Invalid: No keys in KeyValueInfo");
  EXPECT_DEATH(
      key_value_pair_donate_to_value(_pair, _info, 2, &_b),
      "Value index 2 is invalid (max 0)");
  EXPECT_DEATH(
      key_value_pair_donate_from_value(_pair, _info, 2, &_b),
      "Value index 2 is invalid (max 0)");
  EXPECT_DEATH(
      key_value_pair_donate_to_field(_pair, _info, 1, &_b),
      "Field index 1 is invalid (max 0)");
  EXPECT_DEATH(
      key_value_pair_donate_from_field(_pair, _info, 1, &_b),
      "Field index 1 is invalid (max 0)");
  deleteo(&_b);
  EXPECT_DEATH(
      key_value_pair_donate_to_value(_pair, _info, 0, null),
      "Pointer to source pointer null");
  EXPECT_DEATH(
      key_value_pair_donate_from_value(_pair, _info, 0, null),
      "Pointer to destination pointer null");
  EXPECT_DEATH(
      key_value_pair_donate_to_value(_pair, _info, 0, &_b),
      "Source pointer empty");
  EXPECT_DEATH(
      key_value_pair_donate_from_value(_pair, _info, 0, &_b),
      "Source pointer empty");
  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);

  key_value_info_new(&_info, 1, &max, 1, &max);
  key_value_pair_new(&_pair, _info);
  newo(&_b);
  EXPECT_DEATH(
      key_value_pair_donate_to_key(_pair, _info, 1, null),
      "Key index 1 is invalid (max 0)");
  EXPECT_DEATH(
      key_value_pair_donate_from_key(_pair, _info, 1, null),
      "Key index 1 is invalid (max 0)");
  EXPECT_DEATH(
      key_value_pair_donate_to_value(_pair, _info, 1, null),
      "Value index 1 is invalid (max 0)");
  EXPECT_DEATH(
      key_value_pair_donate_from_value(_pair, _info, 1, null),
      "Value index 1 is invalid (max 0)");
  EXPECT_DEATH(
      key_value_pair_donate_to_field(_pair, _info, 2, null),
      "Field index 2 is invalid (max 1)");
  EXPECT_DEATH(
      key_value_pair_donate_from_field(_pair, _info, 2, null),
      "Field index 2 is invalid (max 1)");
  deleteo(&_b);
  EXPECT_DEATH(
      key_value_pair_donate_to_key(_pair, _info, 0, null),
      "Pointer to source pointer null");
  EXPECT_DEATH(
      key_value_pair_donate_from_key(_pair, _info, 0, null),
      "Pointer to destination pointer null");
  EXPECT_DEATH(
      key_value_pair_donate_to_key(_pair, _info, 0, &_b),
      "Source pointer empty");
  EXPECT_DEATH(
      key_value_pair_donate_from_key(_pair, _info, 0, &_b),
      "Source pointer empty");
  EXPECT_DEATH(
      key_value_pair_donate_to_value(_pair, _info, 0, null),
      "Pointer to source pointer null");
  EXPECT_DEATH(
      key_value_pair_donate_from_value(_pair, _info, 0, null),
      "Pointer to destination pointer null");
  EXPECT_DEATH(
      key_value_pair_donate_to_value(_pair, _info, 0, &_b),
      "Source pointer empty");
  EXPECT_DEATH(
      key_value_pair_donate_from_value(_pair, _info, 0, &_b),
      "Source pointer empty");
  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);

  key_value_info_new(&_info, 3, max_keys, 4, max_values);
  key_value_pair_new(&_pair, _info);
  EXPECT_DEATH(
      key_value_pair_donate_to_key(_pair, _info, 2, &_b),
      "Cannot store a pointer in field 2 (key 2) as its maximum value is 3: "
      "must be ULONG_MAX = 18,446,744,073,709,551,615");
  EXPECT_DEATH(
      key_value_pair_donate_to_key(_pair, _info, 2, &_b),
      "Cannot store a pointer in field 2 (key 2) as its maximum value is 3: "
      "must be ULONG_MAX = 18,446,744,073,709,551,615");
  EXPECT_DEATH(
      key_value_pair_donate_to_value(_pair, _info, 1, &_b),
      "Cannot store a pointer in field 4 (value 1) as its maximum value is 7: "
      "must be ULONG_MAX = 18,446,744,073,709,551,615");
  deleteo(&_b);
  key_value_pair_delete(&_pair, _info);
  key_value_info_delete(&_info);
}

FINISH_DEATH
