#include "RandomWeightedTestDeath.h"

START_DEATH

TEST(RandomWeighted, Add) {
  byte a;
  RandomWeighted *_ = null;

  random_weighted_new(&_, null);
  EXPECT_DEATH(
      random_weighted_add(_, null, 1),
      "Object pointer cannot be null");
  EXPECT_DEATH(
      random_weighted_add(_, &a, DBL_POSINF),
      "Weight (inf) cannot be infinite or NaN");
  EXPECT_DEATH(
      random_weighted_add(_, &a, DBL_NEGINF),
      "Weight (-inf) cannot be infinite or NaN");
  EXPECT_DEATH(
      random_weighted_add(_, &a, DBL_NAN),
      "Weight (nan) cannot be infinite or NaN");
  random_weighted_delete(&_);
}

TEST(RandomWeighted, Get) {
  int i = 0;
  RandomWeighted *_ = null;

  random_weighted_new(&_, null);
  EXPECT_DEATH(
      random_weighted_get(_, 0),
      "Cannot get from empty collection");
  random_weighted_add(_, &i, 4);
  EXPECT_DEATH(
      random_weighted_get_from_weight(_, 0, 4.001),
      "Invalid weight 4.001: must be in [0, 4]");
  random_weighted_delete(&_);
}

TEST(RandomWeighted, RemoveGet) {
  int i;
  RandomWeightedNode *n0;
  RandomWeightedNode *n1;
  RandomWeighted *_ = null;

  random_weighted_new(&_, null);
  random_weighted_add(_, &i, 4);
  n0 = random_weighted_get(_, 0);
  random_weighted_remove(_, n0);
  EXPECT_DEATH(
      random_weighted_remove(_, n0),
      "Object has already been removed");
  EXPECT_DEATH(
      random_weighted_get(_, 0),
      "Cannot get from empty collection");

  random_weighted_add(_, &i, 3);
  n0 = random_weighted_get_from_weight(_, 0, 0);
  random_weighted_add(_, &i, 2);
  n1 = random_weighted_get_from_weight(_, 0, 4);
  random_weighted_remove(_, n0);
  EXPECT_DEATH(
      random_weighted_remove(_, n0),
      "Object has already been removed");
  random_weighted_remove(_, n1);
  EXPECT_DEATH(
      random_weighted_remove(_, n1),
      "Object has already been removed");
  EXPECT_DEATH(
      random_weighted_get(_, 0),
      "Cannot get from empty collection");

  random_weighted_add(_, &i, 3);
  n0 = random_weighted_get_from_weight(_, 0, 0);
  random_weighted_add(_, &i, 2);
  random_weighted_remove(_, n0);
  EXPECT_DEATH(
      random_weighted_get_from_weight(_, 0, 2.001),
      "Invalid weight 2.001: must be in [0, 2]");

  random_weighted_delete(&_);
}

TEST(RandomWeighted, VetoGet) {
  int i;
  RandomWeightedNode *n0;
  RandomWeightedNode *n1;
  RandomWeighted *_ = null;

  random_weighted_new(&_, null);
  random_weighted_add(_, &i, 4);
  n0 = random_weighted_get(_, 0);
  random_weighted_veto(_, n0);
  EXPECT_DEATH(
      random_weighted_veto(_, n0),
      "Object has already been vetoed");
  EXPECT_DEATH(
      random_weighted_get(_, 0),
      "Cannot get from empty collection");

  random_weighted_add(_, &i, 3);
  n0 = random_weighted_get_from_weight(_, 0, 0);
  random_weighted_add(_, &i, 2);
  n1 = random_weighted_get_from_weight(_, 0, 4);
  random_weighted_veto(_, n0);
  EXPECT_DEATH(
      random_weighted_veto(_, n0),
      "Object has already been vetoed");
  random_weighted_veto(_, n1);
  EXPECT_DEATH(
      random_weighted_veto(_, n1),
      "Object has already been vetoed");
  EXPECT_DEATH(
      random_weighted_get(_, 0),
      "Cannot get from empty collection");

  random_weighted_add(_, &i, 3);
  n0 = random_weighted_get_from_weight(_, 0, 0);
  random_weighted_add(_, &i, 2);
  random_weighted_veto(_, n0);
  EXPECT_DEATH(
      random_weighted_get_from_weight(_, 0, 2.001),
      "Invalid weight 2.001: must be in [0, 2]");

  random_weighted_delete(&_);
}

TEST(RandomWeighted, UnvetoGet) {
  int i;
  RandomWeightedNode *n0;
  RandomWeighted *_ = null;

  random_weighted_new(&_, null);
  random_weighted_add(_, &i, 4);
  n0 = random_weighted_get(_, 0);
  EXPECT_DEATH(
      random_weighted_unveto(_, n0),
      "Object is not vetoed");
  random_weighted_delete(&_);
}

FINISH_DEATH
