#include "TimeBudgetTest.h"

START

TEST(TimeBudget, ParseStringNew) {
  char *_ = null;
  TimeBudget tb;

  /* "<S>s"
   */
  tb = time_budget_parse("1s");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 second", _);
  string_delete(&_);

  tb = time_budget_parse("59s");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("59 seconds", _);
  string_delete(&_);

  tb = time_budget_parse("60s");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 minute (60 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("61s");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 minute and 1 second (61 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("62s");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 minute and 2 seconds (62 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("120s");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("2 minutes (120 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("121s");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("2 minutes and 1 second (121 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("3600s");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 hour (3600 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("3601s");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 hour and 1 second (3601 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("3660s");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 hour and 1 minute (3660 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("3661s");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 hour, 1 minute, and 1 second (3661 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("7322s");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("2 hours, 2 minutes, and 2 seconds (7322 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("12345s");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("3 hours, 25 minutes, and 45 seconds (12345 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("921599s");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("255 hours, 59 minutes, and 59 seconds (921599 seconds)", _);
  string_delete(&_);

  /* "<M>m"
   */
  tb = time_budget_parse("1m");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 minute (60 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("1.25m");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 minute and 15 seconds (75 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("60m");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 hour (3600 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("60.5m");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 hour and 30 seconds (3630 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("15359.983333333333333m");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("255 hours, 59 minutes, and 59 seconds (921599 seconds)", _);
  string_delete(&_);

  /* "<H>h"
   */
  tb = time_budget_parse("1h");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 hour (3600 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("0.75h");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("45 minutes (2700 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("255.999722222222222h");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("255 hours, 59 minutes, and 59 seconds (921599 seconds)", _);
  string_delete(&_);

  /* "<M>:<SS>"
   */
  tb = time_budget_parse("0:01");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 second", _);
  string_delete(&_);

  tb = time_budget_parse("00:01");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 second", _);
  string_delete(&_);

  tb = time_budget_parse("0:59");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("59 seconds", _);
  string_delete(&_);

  tb = time_budget_parse("00:59");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("59 seconds", _);
  string_delete(&_);

  tb = time_budget_parse("1:00");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 minute (60 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("01:00");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 minute (60 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("1:01");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 minute and 1 second (61 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("01:01");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 minute and 1 second (61 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("59:59");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("59 minutes and 59 seconds (3599 seconds)", _);
  string_delete(&_);

  /* "<H>:<MM>:<SS>"
   */
  tb = time_budget_parse("0:00:01");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 second", _);
  string_delete(&_);

  tb = time_budget_parse("00:00:01");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 second", _);
  string_delete(&_);

  tb = time_budget_parse("0:01:23");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 minute and 23 seconds (83 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("00:01:23");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 minute and 23 seconds (83 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("1:23:45");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 hour, 23 minutes, and 45 seconds (5025 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("01:23:45");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 hour, 23 minutes, and 45 seconds (5025 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("100:00:00");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("100 hours (360000 seconds)", _);
  string_delete(&_);

  tb = time_budget_parse("255:59:59");
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("255 hours, 59 minutes, and 59 seconds (921599 seconds)", _);
  string_delete(&_);
}

TEST(TimeBudget, FromUs) {
  char *_ = null;
  TimeBudget tb;

  tb = time_budget_from_us(500000);
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 second", _);
  string_delete(&_);

  tb = time_budget_from_us(1500000);
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("2 seconds", _);
  string_delete(&_);

  tb = time_budget_from_us(87000000);
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("1 minute and 27 seconds (87 seconds)", _);
  string_delete(&_);

  tb = time_budget_from_us(12345000000);
  string_new_time_budget(&_, tb);
  EXPECT_STREQ("3 hours, 25 minutes, and 45 seconds (12345 seconds)", _);
  string_delete(&_);
}

FINISH
