#include "TimeBudgetTestDeath.h"

START_DEATH

TEST(TimeBudget, Parse) {
  EXPECT_DEATH(
      time_budget_parse(""),
      "Budget string is empty");

  EXPECT_DEATH(
      time_budget_parse("h"),
      "Could not parse 'h' as '<H>h', '<M>m', or '<S>s'");
  EXPECT_DEATH(
      time_budget_parse(".h"),
      "Could not parse '.h' as '<H>h', '<M>m', or '<S>s'");
  EXPECT_DEATH(
      time_budget_parse(" h"),
      "Could not parse ' h' as '<H>h', '<M>m', or '<S>s'");
  EXPECT_DEATH(
      time_budget_parse("a1h"),
      "Could not parse 'a1h' as '<H>h', '<M>m', or '<S>s'");

  EXPECT_DEATH(
      time_budget_parse("m"),
      "Could not parse 'm' as '<H>h', '<M>m', or '<S>s'");
  EXPECT_DEATH(
      time_budget_parse(".m"),
      "Could not parse '.m' as '<H>h', '<M>m', or '<S>s'");
  EXPECT_DEATH(
      time_budget_parse(" m"),
      "Could not parse ' m' as '<H>h', '<M>m', or '<S>s'");
  EXPECT_DEATH(
      time_budget_parse("a1m"),
      "Could not parse 'a1m' as '<H>h', '<M>m', or '<S>s'");

  EXPECT_DEATH(
      time_budget_parse("s"),
      "Could not parse 's' as '<H>h', '<M>m', or '<S>s'");
  EXPECT_DEATH(
      time_budget_parse(".s"),
      "Could not parse '.s' as '<H>h', '<M>m', or '<S>s'");
  EXPECT_DEATH(
      time_budget_parse(" s"),
      "Could not parse ' s' as '<H>h', '<M>m', or '<S>s'");
  EXPECT_DEATH(
      time_budget_parse("a1s"),
      "Could not parse 'a1s' as '<H>h', '<M>m', or '<S>s'");

  EXPECT_DEATH(
      time_budget_parse("5g"),
      "Cannot parse '5g': final character 'g' must be 'h', 'm', or 's'");

  EXPECT_DEATH(
      time_budget_parse("257h"),
      "Time budget must be less than 256 hours");

  EXPECT_DEATH(
      time_budget_parse("1"),
      "Time budget ('1') not in '<H>:<MM>:<SS>' or '<M>:<SS>' format");
  EXPECT_DEATH(
      time_budget_parse("15"),
      "Time budget ('15') not in '<H>:<MM>:<SS>' or '<M>:<SS>' format");
  EXPECT_DEATH(
      time_budget_parse("0:60"),
      "Time budget ('0:60') not in '<H>:<MM>:<SS>' or '<M>:<SS>' format");
  EXPECT_DEATH(
      time_budget_parse("99:15"),
      "Time budget ('99:15') not in '<H>:<MM>:<SS>' or '<M>:<SS>' format");
  EXPECT_DEATH(
      time_budget_parse("0:1"),
      "Time budget ('0:1') not in '<H>:<MM>:<SS>' or '<M>:<SS>' format");
  EXPECT_DEATH(
      time_budget_parse("0:0:15"),
      "Time budget ('0:0:15') not in '<H>:<MM>:<SS>' or '<M>:<SS>' format");
}

TEST(TimeBudget, FromUs) {
  EXPECT_DEATH(
      time_budget_from_us(-100),
      "Rounded seconds (0) from microseconds (-100) must be positive");
  EXPECT_DEATH(
      time_budget_from_us(-123456789),
      "Rounded seconds (-123) from microseconds (-123,456,789) "
      "must be positive");
  EXPECT_DEATH(
      time_budget_from_us(0),
      "Rounded seconds (0) from microseconds (0) must be positive");
  EXPECT_DEATH(
      time_budget_from_us(499999),
      "Rounded seconds (0) from microseconds (499,999) must be positive");
}

FINISH_DEATH
