#include "double_test.h"
#include <float.h>
#include <math.h>

START

ulong g_DOUBLE_nan_ = 0x7ff0000000000001;
ulong g_DOUBLE_neginf_ = 0xfff0000000000000;
ulong g_DOUBLE_negzero_ = 0x8000000000000000;
ulong g_DOUBLE_posinf_ = 0x7ff0000000000000;
ulong g_DOUBLE_subnormal_ = 0x1;



TEST(Double, Values) {
  double d;

  d = 0;
  EXPECT_ULEQ(0, *(ulong *)&d);

  d = 1;
  EXPECT_ULEQ(0x3ff0000000000000, *(ulong *)&d);

  d = -1;
  EXPECT_ULEQ(0xbff0000000000000, *(ulong *)&d);

  d = 0.5;
  EXPECT_ULEQ(0x3fe0000000000000, *(ulong *)&d);

  d = -2;
  EXPECT_ULEQ(0xc000000000000000, *(ulong *)&d);

  d = 256;
  EXPECT_ULEQ(0x4070000000000000, *(ulong *)&d);

  d = -1.2345e-123;
  EXPECT_ULEQ(0xa66a1d465e62dda8, *(ulong *)&d);

  d = 4.5678901234567890123e234;
  EXPECT_ULEQ(0x70a6fc7df1b972a8, *(ulong *)&d);
}

TEST(Double, Constants) {
  double d;

  d = DBL_POSINF;
  EXPECT_ULEQ(0x7ff0000000000000, *(ulong *)&d);

  d = DBL_NEGINF;
  EXPECT_ULEQ(0xfff0000000000000, *(ulong *)&d);

  d = DBL_NEGZERO;
  EXPECT_ULEQ(0x8000000000000000, *(ulong *)&d);

  d = DBL_NAN;
  EXPECT_ULEQ(0x7ff0000000000001, *(ulong *)&d);

  d = DBL_SUBNORMAL;
  EXPECT_ULEQ(0x1, *(ulong *)&d);
}

TEST(Double, IsNegativeZero) {
  EXPECT_TRUE(double_is_negative_zero(DBL_NEGZERO));
  EXPECT_FALSE(double_is_negative_zero(0));
  EXPECT_FALSE(double_is_negative_zero(-0.123));
  EXPECT_FALSE(double_is_negative_zero(DBL_SUBNORMAL));
}

TEST(Double, IsPositiveInfinity) {
  EXPECT_TRUE(double_is_positive_infinity(DBL_POSINF));
  EXPECT_FALSE(double_is_positive_infinity(0));
  EXPECT_FALSE(double_is_positive_infinity(-0.123));
  EXPECT_FALSE(double_is_positive_infinity(DBL_NEGINF));
  EXPECT_FALSE(double_is_positive_infinity(DBL_NAN));
}

TEST(Double, IsNegativeInfinity) {
  EXPECT_TRUE(double_is_negative_infinity(DBL_NEGINF));
  EXPECT_FALSE(double_is_negative_infinity(0));
  EXPECT_FALSE(double_is_negative_infinity(-0.123));
  EXPECT_FALSE(double_is_negative_infinity(DBL_POSINF));
  EXPECT_FALSE(double_is_negative_infinity(DBL_NAN));
}

TEST(Double, IsNan) {
  EXPECT_TRUE(double_is_nan(DBL_NAN));
  EXPECT_FALSE(double_is_nan(0));
  EXPECT_FALSE(double_is_nan(DBL_POSINF));
  EXPECT_FALSE(double_is_nan(DBL_NEGINF));
}

TEST(Double, IsSubnormal) {
  EXPECT_TRUE(double_is_subnormal(DBL_SUBNORMAL));
  EXPECT_TRUE(double_is_subnormal(-DBL_SUBNORMAL));
  EXPECT_FALSE(double_is_subnormal(0));
  EXPECT_FALSE(double_is_subnormal(-0.123));
  EXPECT_FALSE(double_is_subnormal(DBL_MIN));
  EXPECT_FALSE(double_is_subnormal(-DBL_MIN));
  EXPECT_FALSE(double_is_subnormal(DBL_POSINF));
  EXPECT_FALSE(double_is_subnormal(DBL_NEGINF));
}

TEST(Double, IsNegativeZeroOrSubnormal) {
  EXPECT_TRUE(double_is_negative_zero_or_subnormal(DBL_NEGZERO));
  EXPECT_TRUE(double_is_negative_zero_or_subnormal(DBL_SUBNORMAL));
  EXPECT_TRUE(double_is_negative_zero_or_subnormal(-DBL_SUBNORMAL));
  EXPECT_FALSE(double_is_negative_zero_or_subnormal(0));
  EXPECT_FALSE(double_is_negative_zero_or_subnormal(DBL_MIN));
  EXPECT_FALSE(double_is_negative_zero_or_subnormal(-DBL_MIN));
  EXPECT_FALSE(
      double_is_negative_zero_or_subnormal(DBL_POSINF));
  EXPECT_FALSE(
      double_is_negative_zero_or_subnormal(DBL_NEGINF));
  EXPECT_FALSE(double_is_negative_zero_or_subnormal(DBL_NAN));
}

TEST(Double, IsInfiniteOrNan) {
  EXPECT_TRUE(double_is_infinite_or_nan(DBL_POSINF));
  EXPECT_TRUE(double_is_infinite_or_nan(DBL_NEGINF));
  EXPECT_TRUE(double_is_infinite_or_nan(DBL_NAN));
  EXPECT_FALSE(double_is_infinite_or_nan(0));
  EXPECT_FALSE(double_is_infinite_or_nan(-0.123));
  EXPECT_FALSE(double_is_infinite_or_nan(DBL_NEGZERO));
  EXPECT_FALSE(double_is_infinite_or_nan(DBL_SUBNORMAL));
}

FINISH
