#include "gamma_test.h"

START

TEST(Gamma, GammLn) {
  EXPECT_NEAR(0.5723649, gammln(0.5), 1e-7);
  EXPECT_NEAR(0, gammln(1), 1e-8);
  EXPECT_NEAR(-0.1207822, gammln(1.5), 1e-7);
  EXPECT_NEAR(0, gammln(2), 1e-8);
  EXPECT_NEAR(0.69314718, gammln(3), 1e-7);
  EXPECT_NEAR(1.79175946, gammln(4), 1e-7);
  EXPECT_NEAR(12.801827, gammln(10), 1e-6);
  EXPECT_NEAR(359.1342, gammln(100), 1e-4);
  EXPECT_NEAR(5905.2204, gammln(1000), 1e-3);
}

TEST(Gamma, GammP) {
  EXPECT_NEAR(0.7768698, gammp(1, 1.5), 1e-6);
  EXPECT_NEAR(0.59399415, gammp(2, 2), 1e-7);
  EXPECT_NEAR(0.965212219, gammp(2.5, 6), 1e-8);
  EXPECT_NEAR(0.00059560696, gammp(3.7, 0.3), 1e-10);
}

TEST(Gamma, GammQ) {
  EXPECT_NEAR(0.2231301, gammq(1, 1.5), 1e-6);
  EXPECT_NEAR(0.40600584, gammq(2, 2), 1e-7);
  EXPECT_NEAR(0.03478778, gammq(2.5, 6), 1e-8);
  EXPECT_NEAR(0.99940439, gammq(3.7, 0.3), 1e-7);
}

TEST(Gamma, Erf) {
  double d;

  for (d = -5; d <= 5; d += 0.5) {
    EXPECT_NEAR(erff(d), erf(d), 1e-7);
    EXPECT_NEAR(erffc(d), 1 - erf(d), 1e-7);
  }
}

TEST(Gamma, ChiSqP) {
  EXPECT_NEAR(0.24817037, chisqp(0.1, 1), 1e-7);
  EXPECT_NEAR(0.68268949, chisqp(1, 1), 1e-7);
  EXPECT_NEAR(0.84270079, chisqp(2, 1), 1e-7);
  EXPECT_NEAR(0.97465268, chisqp(5, 1), 1e-7);
  EXPECT_NEAR(0.0762173, chisqp(8, 15), 1e-7);
  EXPECT_NEAR(0.54858279, chisqp(15, 15), 1e-7);
  EXPECT_NEAR(0.9880785, chisqp(30, 15), 1e-7);
}

TEST(Gamma, ChiSqQ) {
  EXPECT_NEAR(0.75182963, chisqq(0.1, 1), 1e-7);
  EXPECT_NEAR(0.31731051, chisqq(1, 1), 1e-7);
  EXPECT_NEAR(0.15729921, chisqq(2, 1), 1e-7);
  EXPECT_NEAR(0.02534732, chisqq(5, 1), 1e-7);
  EXPECT_NEAR(0.9237827, chisqq(8, 15), 1e-7);
  EXPECT_NEAR(0.45141721, chisqq(15, 15), 1e-7);
  EXPECT_NEAR(0.0119215, chisqq(30, 15), 1e-7);
}

TEST(Gamma, CritChiSqQ) {
  EXPECT_NEAR(1.5722545e-08, critchisqq(0.9999, 1), 1e-14);

  EXPECT_NEAR(3.8414585, critchisqq(0.05, 1), 1e-5);
  EXPECT_NEAR(5.9914646, critchisqq(0.05, 2), 1e-5);
  EXPECT_NEAR(18.3070373, critchisqq(0.05, 10), 1e-5);

  EXPECT_NEAR(6.6348967, critchisqq(0.01, 1), 1e-5);
  EXPECT_NEAR(9.2103395, critchisqq(0.01, 2), 1e-5);
  EXPECT_NEAR(23.2092514, critchisqq(0.01, 10), 1e-5);

  EXPECT_NEAR(67.5048141, critchisqq(0.05, 50), 1e-5);
  EXPECT_NEAR(76.1539001, critchisqq(0.01, 50), 1e-5);
  EXPECT_NEAR(86.6608123, critchisqq(0.001, 50), 1e-5);

  EXPECT_NEAR(68.6692962, critchisqq(0.05, 51), 1e-5);
  EXPECT_NEAR(77.3859634, critchisqq(0.01, 51), 1e-5);
  EXPECT_NEAR(87.9679794, critchisqq(0.001, 51), 1e-5);

  EXPECT_NEAR(79.0819244, critchisqq(0.05, 60), 1e-5);
  EXPECT_NEAR(90.5312347, critchisqq(0.05, 70), 1e-5);
  EXPECT_NEAR(101.879470, critchisqq(0.05, 80), 1e-5);
  EXPECT_NEAR(124.342071, critchisqq(0.05, 100), 1e-5);
  EXPECT_NEAR(1074.67687, critchisqq(0.05, 1000), 1e-5);
}

TEST(Gamma, ApproxCritChiSqQ) {
  EXPECT_NEAR(1.5722545e-08, approx_critchisqq(0.9999, 1), 1e-14);

  EXPECT_NEAR(3.8414585, approx_critchisqq(0.05, 1), 1e-7);
  EXPECT_NEAR(5.9914646, approx_critchisqq(0.05, 2), 1e-7);
  EXPECT_NEAR(18.3070373, approx_critchisqq(0.05, 10), 1e-7);

  EXPECT_NEAR(6.6348967, approx_critchisqq(0.01, 1), 1e-7);
  EXPECT_NEAR(9.2103395, approx_critchisqq(0.01, 2), 1e-7);
  EXPECT_NEAR(23.2092514, approx_critchisqq(0.01, 10), 1e-7);

  EXPECT_NEAR(67.5048141, approx_critchisqq(0.05, 50), 1e-7);
  EXPECT_NEAR(76.1539001, approx_critchisqq(0.01, 50), 1e-7);
  EXPECT_NEAR(86.6608123, approx_critchisqq(0.001, 50), 1e-7);

  EXPECT_NEAR(67.6122055, approx_critchisqq(0.05, 51), 1e-7);
  EXPECT_NEAR(74.4949645, approx_critchisqq(0.01, 51), 1e-7);
  EXPECT_NEAR(82.2098159, approx_critchisqq(0.001, 51), 1e-7);

  EXPECT_NEAR(78.0184707, approx_critchisqq(0.05, 60), 1e-7);
  EXPECT_NEAR(89.4621734, approx_critchisqq(0.05, 70), 1e-7);
  EXPECT_NEAR(100.805938, approx_critchisqq(0.05, 80), 1e-6);
  EXPECT_NEAR(123.261741, approx_critchisqq(0.05, 100), 1e-6);
  EXPECT_NEAR(1073.56005, approx_critchisqq(0.05, 1000), 1e-5);
}

FINISH
