#include "random_normal_test.h"
#include <float.h>
#include <limits.h>
#include <math.h>

#define N_ 20000000
#define SIGMAS_ 4

static char *version_(
    int n,
    double mn,
    double mx,
    bool normal,
    double gen(Random *random));


START

TEST(Random, Normal) {
  EXPECT(version_(N_ / 10, -DBL_MAX, DBL_MAX, true, random_normal));
}

FINISH


static char *version_(
    int n,
    double mn,
    double mx,
    bool normal,
    double gen(Random *random)) {
  char *_f = null;
  int i;
  double r;
  double sum;
  double sum_sq;
  double mu;
  double s_sq;
  double e_mu;
  double se_mu;
  double e_s_sq;
  double se_s_sq;
  double z;
  Random *_random = null;

  random_new(&_random);
  sum = sum_sq = 0;

  for (sum = 0, i = 0; i < n; i++) {
    r = gen(_random);

    if (r < mn) {
      string_new_f(&_f, "Returned %.16g but minimum %.16g", r, mn);

      return _f;
    }

    if (r > mx) {
      string_new_f(&_f, "Returned %.16g but maximum %.16g", r, mx);

      return _f;
    }

    sum += r;
    sum_sq += r * r;
  }

  mu = sum / n;
  s_sq = sum_sq / (n - 1) - mu * mu;
  e_s_sq = normal ? 1 : (mx - mn) * (mx - mn) / 12;
  se_s_sq = sqrt(2 * e_s_sq * e_s_sq / (n - 1));
  e_mu = normal ? 0 : (mn + mx) / 2;
  se_mu = sqrt(e_s_sq / n);
  z = (mu - e_mu) / se_mu;

  if (fabs(z) > SIGMAS_) {
    string_new_f(
        &_f,
        "Mean %.16g incorrect (expected %.16g) at %.16g sigmas "
        "(actual %.16g sigmas)",
        mu,
        e_mu,
        SIGMAS_,
        z);

    return _f;
  }

  z = (s_sq - e_s_sq) / se_s_sq;

  if (fabs(z) > SIGMAS_) {
    string_new_f(
        &_f,
        "Standard deviation %.16g incorrect (expected %.16g) at %.16g sigmas "
        "(actual %.16g sigmas)",
        sqrt(s_sq),
        sqrt(e_s_sq),
        SIGMAS_,
        z);

    return _f;
  }

  random_delete(&_random);

  return null;
}
